/*
 * ClassPathSpec.java
 * Resource specification for a team's class path.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * @JANOSVM_JVMLIB_LICENSE@
 */

package edu.utah.janosvm.resources;

/**
 * Specification for the class path resource.
 *
 * @see ClassPathResource
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class ClassPathSpec
    extends ResourceSpec
{
    /**
     * Constructs the native part of the class.
     */
    private native void nativeConstructor(String entries[]);

    /**
     * Finalizes the native part of the class.
     */
    private native void nativeFinalizer();
    
    /**
     * Construct a class path specification with given paths.
     *
     * @param entries An array of strings specifying each part of the classpath.
     */
    public ClassPathSpec(String entries[])
    {
	this.nativeConstructor(entries);
    }
    
    public void finalize()
    {
	this.nativeFinalizer();
    }
    
}
