/*
 * MemorySpec.java
 * Resource specification for memory
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * @JANOSVM_JVMLIB_LICENSE@
 */

package edu.utah.janosvm.resources;

/**
 * Specification for memory resources.  There are currently two
 * details in a memory specification: the amount of memory and the
 * policy for the GC.  The amount of memory is specified in bytes,
 * but will be rounded to the GC's internal page granularity.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class MemorySpec
    extends ResourceSpec
{
    /**
     * Constructs the native part of the class.
     */
    private native void nativeConstructor(int max, String policyClass);
    
    /**
     * Construct a memory resource specification that requires the given
     * amount of memory.
     *
     * @param max The amount of memory, in bytes, required for the team to run.
     */
    public MemorySpec(int max)
    {
	this.nativeConstructor(max, null);
    }

    /**
     * Construct a memory resource specification that requires the given
     * amount of memory and needs the given GCPolicy class to be used.
     *
     * @param max The amount of memory, in bytes, required for the team to run.
     * @param policyClass The name of the policy class the new team should
     * use when initializing the GC.
     */
    public MemorySpec(int max, String policyClass)
    {
	this.nativeConstructor(max, policyClass);
    }
    
    private native void nativeFinalizer();
    
    protected void finalize()
    {
	this.nativeFinalizer();
    }
}
