/*
 * NameSpaceGroup.java
 * Helper class for NameSpaceSpec
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * @JANOSVM_JVMLIB_LICENSE@
 */

package edu.utah.janosvm.resources;

import edu.utah.janosvm.sys.Team;

/**
 * Helper class for NameSpaceSpec that specifies which class groups to
 * import from which team.
 * <p>
 * XXX what's a viewgroup and what's a classGroup?
 *
 * @see NameSpaceSpec
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class NameSpaceGroup
{
    /**
     * The team to import the classes from.
     */
    private final Team team;
    /**
     * The class group type to import.
     */
    private final String classGroup;
    /**
     * The view group type to import.
     */
    private final String viewGroup;

    /**
     * Construct a NameSpaceGroup object with the given parameters.
     *
     * @param team The team to import classes from.
     * @param cg The name of the class group type.
     * @param vg The name of the view group type.
     */
    public NameSpaceGroup(Team team, String cg, String vg)
    {
	this.team = team;
	this.classGroup = cg;
	this.viewGroup = vg;
    }

    /**
     * @return The team the classes will be imported from.
     */
    public Team getTeam()
    {
	return this.team;
    }
    
    /**
     * @return The name of the class group type that will be imported.
     */
    public String getClassGroup()
    {
	return this.classGroup;
    }
    
    /**
     * @return The name of the view group type that will be imported.
     */
    public String getViewGroup()
    {
	return this.viewGroup;
    }
    
}
