/*
 * NameSpaceSpec.java
 * Resource specification for the root name space.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * @JANOSVM_JVMLIB_LICENSE@
 */

package edu.utah.janosvm.resources;

/**
 * Resource specification for the root name space of a team.  This allows you
 * to specify which classes should be imported from other teams, thus allowing
 * the teams to share objects.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class NameSpaceSpec
    extends ResourceSpec
{
    /**
     * Construct the native part of the class.
     */
    private native void nativeConstructor(NameSpaceGroup nsg[]);

    /**
     * Finalize the native part of the class.
     */
    private native void nativeFinalizer();

    /**
     * Construct a name space resource specification with the given class
     * imports.
     *
     * @param nsg An array of NameSpaceGroup's which describe which class
     * groups should be imported into the team.
     */
    public NameSpaceSpec(NameSpaceGroup nsg[])
    {
	this.nativeConstructor(nsg);
    }

    public void finalize()
    {
	this.nativeFinalizer();
    }
    
}
