/*
 * KernelReaper.java
 * The kernel reaper.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * @JANOSVM_JVMLIB_LICENSE@
 */

package edu.utah.janosvm.sys;

import java.util.Vector;

/**
 *
 * @see RemoteTeam
 * @author Patrick Tullmann, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class KernelReaper
{
    private static final int Q_SIZE = 10;
    private static final RemoteTeam[] queue = new RemoteTeam[Q_SIZE];
    private static final int[] queueOp = new int[Q_SIZE];
    private static int queueHead = 0;
    private static int queueTail = 0;
    private static final int QOP_TEAMTERMINATE = 1;
    private static final int QOP_TEAMCOLLECT   = 2;

    // never instantiated
    private KernelReaper()
    {
    }

    static void startReaper()
    {
	Thread reaper;

	reaper = new Thread(new Runnable() {
	    public void run() {
		doTheReap();
	    }
	}, "Kernel Reaper Thread");
	reaper.setDaemon(true);
	reaper.start();
    }

    static void enqueueForCollection(RemoteTeam team)
    {
	//require(RemoteTeam.getCurrentTeam() == RemoteTeam.getKernelTeam());
	enqueue(team, QOP_TEAMCOLLECT);
    }

    private static void enqueue(RemoteTeam team, int op)
    {
	//require(RemoteTeam.getCurrentTeam() == RemoteTeam.getKernelTeam());

	synchronized(queue)
	{
	    if ((queueHead+1 % Q_SIZE) == queueTail)
		throw new InternalError("Reaper Queue has filled up.  Replace fixed-size queue with a linked list, please.");

	    queueHead = (queueHead + 1) % Q_SIZE;
	    queue[queueHead] = team;
	    queueOp[queueHead] = op;
	    
	    queue.notify();
	}
    }

    /**
     * The kernel reaper work function.  Never returns.
     */
    private static void doTheReap()
    {
	while (true)
	{
	    RemoteTeam rt = null;
	    int op = -1;

	    try
	    {
		// XXX THIS IS LIFO, which is slightly bad (but easy)
		synchronized(queue)
		{
		    while(queueHead == queueTail)
			queue.wait();
		    
		    rt = queue[queueHead];
		    op = queueOp[queueHead];
		    
		    queueHead = (queueHead - 1) % Q_SIZE;
		}
		
		// Do the actual termination outside of the synchronized
		// queue block.
		switch(op)
		{
		case QOP_TEAMCOLLECT:
		    rt.reaperCollect();
		    break;
		default:
		    throw new InternalError("Unknown QOP in doTheReap(op=" +Integer.toString(op)+ "; rt=" +rt+ "):");
		}
	    }
	    catch (Throwable th)
	    {
		System.debug("Ignoring exception in doTheReap(op=" +Integer.toString(op)+ "; rt=" +rt+ "):");
		System.debugE(th);
	    }
	}
    }
}
