/*
 * ObjectExporter.java
 * Export a generic object.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * @JANOSVM_JVMLIB_LICENSE@
 */

package edu.utah.janosvm.sys;

/**
 * A simple means of exporting a generic object.
 *
 * @see ObjectExporter
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class ObjectExporter
    extends Exportable
{
    private Object obj;

    /**
     * Construct an export for the given object.
     *
     * @param obj The object to export.
     */
    public ObjectExporter(Object obj)
    {
	this.obj = obj;
    }

    /**
     * Construct and export the object under the given identifier.
     *
     * @param id The identifier to export this object under.
     * @param obj The object to export.
     * @exception Throws DuplicateExportException if there is already an export
     * with the given Identifier.
     */
    public ObjectExporter(Object id, Object obj)
	throws DuplicateExportException
    {
	super(id);
	this.obj = obj;
    }

    /**
     * @return The object being exported.
     */
    public Object getObject()
    {
	return this.obj;
    }
}
