#!/bin/sh
#
# Copyright (c) 1998, 1999, 2000, 2001 The University of Utah. All rights reserved.
#
# This file is licensed under the terms of the GNU Public License.  
# See the file "license.terms" for restrictions on redistribution 
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# Contributed by the Flux Research Group at the University of Utah.
# Authors: Patrick Tullmann

#
# ld-nodeos.sh
#
# This is a magic wrapper for the link step in Kaffe.  It 
# supplies all of the necessary NodeOS libraries (of which
# there are many all of which have odd ordering dependencies).
#
# The --oskit=* and --nodeos=* options are set by the nodeos-configure
# shell script.
#
# XXX could use a --bootloader=foo option
#

# the default OSKit install directory
oskit=/usr/local/

# the default NodeOS install directory
nodeos=/usr/local/

# the default Click install directory (null means don't use)
click=

# The gcc version to use.
gcc=gcc

# Uncomment to pass '-v' to get a verbose GCC
#gcc_verbose=-v

# The bootloader-specific .o to link in
bootloaderObject=multiboot.o
#bootloaderObject=dos.o

# Uncomment/Comment this line to include/remove memdebug support
loskit_memdebug="-loskit_memdebug"

# Parse the arguments to this script.  Most just pass right through
# to the linker.  Except:
#   -v : exec's gcc immediately with -v option (for configure).
#   -lm: remove all the -lm's from the library list.  We need to link
#        against the proper oskit_<OS>_m library
#   --oskit: pull off the oskit directory
#   --nodeos: pull off the nodeos directory
#   --click: pull off the click directory
#   --oskitunix: Build kernel for OSKit/UNIX.
#   kaffeh: XXX ACK!  If we're trying to build kaffeh, JUST SKIP IT.

args="";
is_unix="";

for a in $*; do
    case $a in
    --oskit=*)
	oskit=`echo "$a" | sed 's/[-_a-zA-Z0-9]*=//'`;
	;;
    --click=*)
	click=`echo "$a" | sed 's/[-_a-zA-Z0-9]*=//'`;
	;;
    --nodeos=*)
	nodeos=`echo "$a" | sed 's/[-_a-zA-Z0-9]*=//'`;
	;;
    -v) 
	## XXX A hack to make configure through this script work.
	exec `$gcc --print-prog-name=ld` -v;
	;;
    -lm) 
	;;
    --oskitunix) 
	is_unix=true;
	;;
    kaffeh)
	### XXX This is a bad hack to prevent trying to link kaffeh
	rm -f kaffeh
	echo "#!/bin/sh" > kaffeh
	echo "echo KAFFEH HACK HAS STRUCK.  See $0" >> kaffeh
	echo "#eof" > kaffeh
	echo "KAFFEH NOT REALLY BUILT.  THIS IS A HACK BECAUSE IT WILL NOT LINK AGAINST THE OSKIT."
	chmod +x kaffeh
	exit 0
	;;
    *) 
	args="$args $a";
	;;
    esac
done

## Figure out the click link magic
libdir_click=
library_click=
if test -n "$click"; then
    libdir_click="-L${click}/lib"
    library_click="-lclick"
fi

if test -z "$is_unix"; then
    ### Regular, bare-hardware OSKit link line (see why we hid it?)
    $gcc $gcc_verbose -static -nostartfiles -Wl,-Ttext -Wl,100000	\
	${oskit}/lib/oskit/${bootloaderObject}		\
	$args						\
	-nostdlib 					\
	-L${oskit}/lib					\
	-L${nodeos}/lib \
	-u pthread_init \
	${libdir_click} \
	-Wl,--start-group				\
	-loskit_startup  				\
	-loskit_clientos				\
	-lnodeos \
	-lmoabutil \
	${library_click} \
	-loskit_threads  				\
	-loskit_svm 					\
	-loskit_amm					\
	-loskit_bootp					\
	-loskit_freebsd_net				\
	-loskit_linux_dev 				\
	-loskit_dev					\
	-loskit_netbsd_fs				\
	-loskit_lmm					\
	-loskit_diskpart 				\
	-loskit_memfs					\
	-loskit_fsnamespace_r				\
	-loskit_com					\
	-loskit_kern					\
	-loskit_freebsd_m 				\
	$loskit_memdebug 				\
	-loskit_freebsd_c_r				\
	-Wl,--end-group					\
	${oskit}/lib/oskit/crtn.o 
else
    ### OSKit/UNIX link line
    $gcc $gcc_verbose -e _start			\
	${oskit}/lib/oskit/unix/crt1.o 		\
	${oskit}/lib/oskit/unix/crti.o		\
        ${oskit}/lib/oskit/unix_support_pthreads.o \
  	$args					\
  	-nostdlib  				\
	-L${oskit}/lib				\
	-L${nodeos}/lib 			\
	${libdir_click} \
	-lnodeos 				\
	-lmoabutil \
	${library_click} \
        -loskit_startup  			\
	-loskit_clientos 			\
        -loskit_threads 			\
        -loskit_bootp	 			\
	-loskit_fsnamespace_r			\
	$loskit_memdebug 			\
	-loskit_unix 				\
	-loskit_freebsd_net 			\
	-loskit_linux_dev 			\
	-loskit_dev				\
  	-loskit_freebsd_c_r  			\
	-loskit_com				\
  	-loskit_freebsd_m 			\
	-loskit_freebsd_c_r			\
        -loskit_threads				\
        -loskit_lmm				\
	${oskit}/lib/oskit/unix/crtn.o
fi

#eof

