/*
 * assert_pp.h
 * Precondition and postcondition asserts.
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef ASSERTPP_H
#define ASSERTPP_H

/*
 * Provides requires/ensure macros for testing preconditions and
 * postconditions.
 */

#include <stdio.h>
#include <stdlib.h>

#if !defined(DEBUG)
#define NO_PRECONDITIONS
#define NO_POSTCONDITIONS
#endif

#if !defined(NO_POSTCONDITIONS) || !defined(NO_PRECONDITIONS)
#if defined(__GNUC__)
#define ASSERT_FUNCATTR_NORETURN __attribute__((__noreturn__))
#else
#define ASSERT_FUNCATTR_NORETURN
#endif
/* 
 * The routine which prints failure messages.  Defined
 * in kaffe/kaffevm/debug.c, kaffe/kaffeh/main.c, 
 * kaffe/jsihey/main.c, and...
 */
extern void pp_failed(char *file, unsigned int line, 
		      char* func, char *type, char *expr) 
	ASSERT_FUNCATTR_NORETURN;
#endif


#ifndef NO_PRECONDITIONS
#define pp_decl(decl) decl
#define pp_block(block) block
#define require(precon) \
	((void)((precon) ? 0 : \
                (pp_failed(__FILE__,__LINE__,__PRETTY_FUNCTION__,"precondition",__STRING(precon)))))
#else
#define require(x)
#define pp_decl(x)
#define pp_block(x)
#endif

#ifndef NO_POSTCONDITIONS
#define ensure(postcon) \
	((void)((postcon) ? 0 : \
                (pp_failed(__FILE__,__LINE__,__PRETTY_FUNCTION__,"postcondition",__STRING(postcon)))))
#else
#define ensure(x)
#endif

#endif
