/*
 * classPath-inline.h
 * Provides access to the class path for the jvm.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/* Inline convenience functions for fileTypeHandlers */
KAFFE_INLINE
classPathEntry *fthCreateEntry(fileTypeHandler *fth, char *path)
{
	require(fth != 0);
	require(path != 0);
	
	return( fth->createEntry(fth, path) );
}

KAFFE_INLINE
void fthDeleteEntry(classPathEntry *cpe)
{
	require(cpe != 0);
	require(cpe->fileType != 0);
	require(!(cpe->flags & CPEF_PUBLISHED));
	require(!(cpe->flags & CPEF_HIDDEN));
	
	return cpe->fileType->deleteEntry(cpe->fileType, cpe);
}

KAFFE_INLINE
int fthMatchPath(fileTypeHandler *fth, char *path, struct stat *st, int fd,
		 char *buf)
{
	require(fth != 0);
	require(path != 0);
	require(st != 0);
	
	return fth->matchPath(fth, path, st, fd, buf);
}

KAFFE_INLINE
int fthPreloadEntry(classPathEntry *cpe)
{
	require(cpe != 0);
	require(cpe->fileType != 0);
	
	return cpe->fileType->preloadEntry(cpe->fileType, cpe);
}

KAFFE_INLINE
int fthLoadEntryFile(classPathEntry *cpe, char *fname,
		     classFile *cf, errorInfo *einfo)
{
	require(cpe != 0);
	require(cpe->fileType != 0);
	require(fname != 0);
	require(cf != 0);
	require(einfo != 0);
	
	return cpe->fileType->loadEntryFile(cpe->fileType,
					    cpe,
					    fname,
					    cf,
					    einfo);
}

KAFFE_INLINE
int fthSnapshotEntry(classPathEntry *cpe)
{
	require(cpe != 0);
	require(cpe->fileType != 0);
	
	return cpe->fileType->snapshotEntry(cpe->fileType, cpe);
}
