/*
 * configFile.h
 * Manages a configuration file for the jvm.
 * 
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _config_file_h
#define _config_file_h

#include <misc/sectionFile.h>

#include "resource.h"

/*
 * A configFilePart is used to collect all of the file names that should be
 * processed when configuring the VM.
 *
 * `next' - Link to the next structure in the list.
 * `filename' - The name of the file to read into the sectionFile.
 */
typedef struct _configFilePart {
	struct _configFilePart *next;
	char *filename;
} configFilePart;

/*
 * Add a configuration file to the set that will make up the kernel/metal
 * team's configuration.
 *
 * `cfp' - An initialized configFilePart object to link into the internal list.
 */
void addConfigFile(configFilePart *cfp);

/*
 * The resource structure definition for the configuration file resource.
 */
typedef struct _configResource {
	resource link;
} configResource;

/*
 * The client structure for the configuration file resource.
 *
 * `client' - The resourceClient header.
 * `sf' - The sectionFile containing the configuration data.
 * `files' - The linked list of files that hold the config data.  Only here
 *  so you can create and clone the configuration spec.
 */
typedef struct _configClient {
	resourceClient client;
	sectionFile *sf;
	configFilePart *files;
} configClient;

/*
 * The specification structure for the configuration file resource.
 *
 * `spec' - The resourceSpec header.
 * `files' - The linked list of files that hold the config data.
 */
typedef struct _configSpec {
	resourceSpec spec;
	configFilePart *files;
} configSpec;

/*
 * returns - The configuration data for the current team.
 */
sectionFile *currentConfigFile(void);

/*
 * AddModule: configFileModule
 *   ModuleDependency: resourceModule.internalResources
 *   ModuleDependency: memResourceModule.initMetal
 *   ModuleDependency: logResourceModule.initMetal
 * Transition: initMetal
 *   ModuleDependency: sectionFileModule registryModule
 */

#endif
