/*
 * jteam-inline.h
 * Maintains separation of code running in the jvm.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jteam_inline_h
#define _jteam_inline_h

#include "jteam.h"

/*
 * Get the current thread's current team.
 */
KAFFE_INLINE
jteam *currentTeam(void)
{
	jteam *retval;

	require(jthread_current() != 0);
	
	retval = jthread_get_data(jthread_current())->currentTeam;

	ensure(retval != 0);
	return( retval );
}

/*
 * Get the current thread's owner team.
 */
KAFFE_INLINE
jteam *ownerTeam(void)
{
	jteam *retval;

	require(jthread_current() != 0);
	
	retval = jthread_get_data(jthread_current())->ownerTeam;
	
	ensure(retval != 0);
	return( retval );
}

/*
 * Get the resourceClient for a given resource in the given team.
 */
KAFFE_INLINE
resourceClient *getTeamInternalResource(jteam *jt, resource *res)
{
	require(res);
	require(res->internalIndex != -1);

	if( !jt )
		jt = currentTeam();
	return jt->internalResources[res->internalIndex];
}

KAFFE_INLINE
resourceClient *getTeamExternalResource(jteam *jt, resource *res, int id)
{
	resourceClient *curr;

	require(res);

	if( !jt )
		jt = currentTeam();

	if (res->internalIndex != -1)
		return NULL;

	curr = jt->externalResources;
	while( curr )
	{
		if( (curr->handler == res) && (curr->id == id) )
			return curr;
		curr = curr->next;
	}

	return NULL;
}

KAFFE_INLINE
int hasTeamDependency(jteam *jt, jteam *dep)
{
	jteamDependency *jd;
	int lpc, retval = 0;

	jd = &jt->deps;
	while( jd && !retval )
	{
		for( lpc = 0; (lpc < JTEAM_DEPENDENCY_POOL) && !retval; lpc++ )
		{
			if( jd->deps[lpc] == dep )
				retval = 1;
		}
		jd = jd->next;
	}
	return( retval );
}

#endif
