/*
 * leak.h
 * A debugging tool for tracking memory leaks.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _leak_h
#define _leak_h

/* Uncomment the following to turn leak tracking on */
// #define DEBUG_LEAK

#if !defined(DEBUG) || defined(KAFFEH) || !defined(DEBUG_LEAK)

#define trackingFile(name, file)
#define trackingLeaks() 0
#define trackLeaks(yes_no)
#define registerLeak(ptr, note)
#define unregisterLeak(ptr)
#define clearLeaks()
#define isLeak(ptr) 0
#define printLeak(ptr, from)
#define printLeaks()

#else

/*
 * returns - the current leak tracking state.
 */
int trackingLeaks(void);
/*
 * Turn leak tracking on/off.
 */
void trackLeaks(int yes_no);
/*
 * Register a leak to track and an accompanying note.
 *
 * `ptr' - The leaking pointer.
 * `note' - A note concerning the leaking pointer.
 *
 * `file' - The file where the registerLeak macro was called.
 * `line' - The line number in the file where the registerLeak macro was called
 */
#define registerLeak(ptr, note) (tracking_leaks ? real_registerLeak(ptr, note, __FILE__, __LINE__) : (ptr))
void *real_registerLeak(void *ptr, char *note, char *file, int line);
/*
 * Unregister the leaked memory.
 *
 * `ptr' - The leaked pointer as passed to registerLeak.
 */
void unregisterLeak(void *ptr);
/*
 * Unregister all of the registered leaks.
 */
void clearLeaks(void);
/*
 * Test if a pointer is a leak.
 *
 * `ptr' - The suspected leak pointer.
 * returns - True if `ptr' is a registered leak, false otherwise.
 */
int isLeak(void *ptr);
/*
 * Print a leak and what is keeping it alive.
 *
 * `ptr' - The leaking pointer.
 * `from' - The object that references `ptr'.
 */
void printLeak(void *ptr, void *from);
/*
 * Print all leaks currently being tracked.
 */
void printLeaks(void);

extern int tracking_leaks;

#endif

#endif
