/*
 * listNode.h
 * Functions for manipulating doubly linked lists
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _list_node_h
#define _list_node_h

/* Node struct that is used for doubly linked list */
struct lnMinNode {
	struct lnMinNode *ln_Succ;
	struct lnMinNode *ln_Pred;
};

struct lnNode {
	struct lnNode *ln_Succ;
	struct lnNode *ln_Pred;
	int ln_Pri;
	char *ln_Name;
};

/* List struct itself, just think of it as a combined header and tail node */
struct lnMinList {
	struct lnMinNode *lh_Head; /* points to the head node */
	struct lnMinNode *lh_Tail; /* always == 0 */
	struct lnMinNode *lh_TailPred; /* points to the tail node */
};

struct lnList {
	struct lnNode *lh_Head;
	struct lnNode *lh_Tail;
	struct lnNode *lh_TailPred;
	int lh_Pri;
	char *lh_Name;
};

/* Remove a node from the list */
void lnRemove(struct lnMinNode *node);
/* Initialize a list */
void lnNewList(struct lnMinList *list);
/* Transfer the nodes on src to dest, overwriting any nodes on dest */
void lnMoveList(struct lnMinList *dest, struct lnMinList *src);
/* Append the nodes from src to dest */
void lnAppendList(struct lnMinList *dest, struct lnMinList *src);
/* Add a node to the head of the list */
void lnAddHead(struct lnMinList *list, struct lnMinNode *node);
/* Add a node to the tail of the list */
void lnAddTail(struct lnMinList *list, struct lnMinNode *node);
/*
 * Remove and return the node at the head of the list, or NULL if the list is
 * empty
 */
struct lnMinNode *lnRemHead(struct lnMinList *list);
/*
 * Remove and return the node at the tail of the list, or NULL if the list is
 * empty
 */
struct lnMinNode *lnRemTail(struct lnMinList *list);
/* Check if a list is empty */
#define lnEmptyList(list) ((list)->lh_TailPred == (struct lnMinNode *)(list))
/* Insert the node after the pred node */
void lnInsert(struct lnMinNode *pred, struct lnMinNode *node);
/* Insert a node into a prioritized list */
void lnEnqueue(struct lnList *list, struct lnNode *node);

#endif
