/*
 * logResource.h
 * Resource to managed standard I/O for each team.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _log_resource_h
#define _log_resource_h

#include <stdio.h>

#include <misc/logger.h>

#include "resource.h"

typedef struct _logResource {
	resource link;
} logResource;

typedef struct _logClient {
	resourceClient client;
	iLock *lock;
	logger log;
} logClient;

typedef struct _logSpec {
	resourceSpec spec;
} logSpec;

/*
 * Add a message to the current team's log.
 *
 * `cl' - The classification for the message.
 * `msg' - The message itself.
 */
#define teamLog(cl, msg) \
	teamLog_real(__FILE__, __LINE__, cl, msg)
void teamLog_real(char *srcFile, int srcLine, char *cl, char *msg);
/*
 * Add a generic entry to the current team's log.
 *
 * `cl' - The classification for the message.
 * `...' - A tag list describing the log entry.
 */
#define teamLogEntry(cl, args...) \
	teamLogEntry_real(__FILE__, __LINE__, cl, ##args)
void teamLogEntry_real(char *srcFile, int srcLine, char *cl, ...);

/*
 * Add a log analyzer to the current team's logger.
 *
 * `lam' - The logAnalyzerMask to add to the team's logger.
 */
void addTeamLogAnalyzer(logAnalyzerMask *lam);
/*
 * Remote a log analyzer from the current team's logger.
 *
 * `lam' - The logAnalyzerMask to remove from the team's logger.
 */
void remTeamLogAnalyzer(logAnalyzerMask *lam);

/*
 * AddModule: logResourceModule
 *   ModuleDependency: resourceModule.internalResources
 *   ModuleDependency: memResourceModule.initMetal
 * Transition: initMetal
 *   ModuleDependency: resourceModule.reinstall
 */

#endif
