/*
 * nativeClass-inline.h
 * Manage native data attached to java objects
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _native_class_inline_h
#define _native_class_inline_h

#include "assert_pp.h"
#include "soft.h"
#include "native.h"
#include "classMethod.h"

/*
 * Convert an object reference to a pointer to the native data section.
 */
KAFFE_INLINE
void *OBJECT2NATIVE(nativeClass *type, Hjava_lang_Object *obj)
{
	void *retval;

	require(type != 0);
	require(type->peer != 0);
	require(type->offset > 0);
	require(obj != 0);
	require(instanceof(type->peer, obj->dtable->class));
	
	retval = (void *)(((uintp)(obj)) + type->offset);
	return( retval );
}

/*
 * Convert a native pointer to an object reference.
 */
KAFFE_INLINE
Hjava_lang_Object *NATIVE2OBJECT(nativeClass *type, void *nd)
{
	struct Hjava_lang_Object *retval;

	require(type != 0);
	require(type->peer != 0);
	require(nd != 0);
	
	retval = (Hjava_lang_Object *)(((uintp)(nd)) - type->offset);

	ensure(retval != 0);
	ensure(retval->dtable != 0);
	ensure(retval->dtable->class != 0);
	ensure(instanceof(type->peer, retval->dtable->class));
	
	return( retval );
}

#endif /* _native_class_inline_h */
