/*
 * nativeClass.c
 * Manage native data attached to java objects
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "gtypes.h"
#include "support.h"
#include "thread.h"
#include "classMethod.h"
#include "jteam.h"
#include "mungWallProxy.h"
#include "nativeClass.h"

#define NATIVE_ALIGN(x) \
	(((x) + (ALIGNMENTOF_VOIDP - 1)) & ~(ALIGNMENTOF_VOIDP -1))

int bindNativeClass(nativeClass *nc, Hjava_lang_Class *clazz, errorInfo *einfo)
{
	int alignedSize, newSize, retval = 0;
	jteam *owner;
	int *newMap;

	require(nc != 0);
	require(nc->size > 0);
	require(clazz != 0);
	require(clazz->state >= CSTATE_LINKED);
	require(einfo != 0);
	
	owner = gc_get_resource(clazz)->client.owner;
	if( switchToTeam(owner, 0) == 0 )
	{
		/* Is this even possible? */
		postExceptionMessage(einfo, DEAD_TEAM_EXCEPTION_S,
				     "Native class owner team is dead in "
				     __FUNCTION__);
		return 0;
	}
	/* Use the current size of the class's objects for the offset */
	alignedSize = NATIVE_ALIGN(CLASS_FSIZE(clazz));
	nc->offset = alignedSize;
	newSize = alignedSize + NATIVE_ALIGN(nc->size);
	/* Allocate a new gc bitmap that includes space for the native data */
	if( (newMap = BITMAP_NEW(newSize/ALIGNMENTOF_VOIDP)) )
	{
		/*
		 * Copy the old map over, since we don't want the gc to walk
		 * the native part we don't need to set any bits in the new
		 * one.
		 */
		BITMAP_COPY(newMap,
			    clazz->gc_layout,
			    nc->offset / ALIGNMENTOF_VOIDP);
		BITMAP_DELETE(clazz->gc_layout);
		/* Install the new size and gc map */
		CLASS_FSIZE(clazz) = newSize;
		clazz->gc_layout = newMap;
		nc->peer = clazz;
		clazz->nativeclass = nc;
		retval = 1;

		ensure_valid_memory(newMap);
	}
	else
	{
		postOutOfMemory(einfo);
	}
	returnFromTeam();
	return( retval );
}

void unbindNativeClass(nativeClass *nc)
{
	require(nc != 0);
	require(nc->peer != 0);

	nc->peer->nativeclass = 0;
	nc->peer = 0;
	nc->offset = 0;
}
