/*
 * nativeClass.h
 * Manage native data attached to java objects
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _native_class_h
#define _native_class_h

/*
 * Native class is used to insert native data into a java class.  This makes
 * it easier to work with objects that have vm internal peers.
 */

#include "gtypes.h"
#include "errors.h"
#include "native.h"

struct Hjava_lang_Class;

/*
 * A nativeClass stores the description of a native structure that is to be
 * included in object of a specific class.
 *
 * `peer' - The class that native data is appended to.
 * `offset' - The offset from the start of the java object to the native data.
 * `size' - The aligned size of the native data.
 */
typedef struct _nativeClass {
	struct Hjava_lang_Class *peer;
	int offset;
	int size;
} nativeClass;

/*
 * Bind some native data type to a given java class.
 *
 * `nc' - A nativeClass object with its size field initialized.
 * `clazz' - The java class to bind native data to.
 * `einfo' - Stores any error information.
 * returns - True if the binding succeeded, false otherwise.  Also the `nc'
 *  object will have its other fields initialized.
 */
int bindNativeClass(nativeClass *nc,
		    struct Hjava_lang_Class *clazz,
		    errorInfo *einfo);
/*
 * Unbind some native data type from its java peer.
 *
 * `nc' - The native class to unbind.
 */
void unbindNativeClass(nativeClass *nc);

#endif
