/*
 * registry.c
 * A wrapper for the jsi registry functions
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <assert_pp.h>
#include <jsi/jsi.h>

#include "module.h"
#include "stringSupport.h"

#include "registry.h"

char *registryName;
struct Hjava_lang_String *jRegistryName;

/** Module declarations **/
static int initRegistryModule(int stage);
static void killRegistryModule(int stage);

module_t registryModule = {
	"registry",
	0,
	initRegistryModule,
	killRegistryModule,
	0
};

/** Module functions **/
int initRegistryModule(int stage)
{
	int retval = 1;

	if( registryName )
	{
		jsiRegisterName(registryName);
		if( (jRegistryName = stringC2Java(registryName)) )
		{
			gc_add_ref(jRegistryName);
		}
	}
	return( retval );
}

void killRegistryModule(int stage)
{
	if( registryName )
	{
		jsiUnregisterName(registryName);
	}
}

/** Registry functions **/
void registerExport(char *export, char *fmt, ...)
{
	va_list args;

	require(export != 0);
	require(fmt != 0);
	
	va_start(args, fmt);
	jsiVRegisterExport(registryName, export, fmt, args);
	va_end(args);
}

struct Hjava_lang_String *java_lang_System_getName(void)
{
	return( jRegistryName );
}

struct Hjava_lang_String *edu_utah_janosvm_sys_VMInstance_getName(void)
{
	return( jRegistryName );
}
