/*
 * registry.h
 * A wrapper for the jsi registry functions
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _registry_h
#define _registry_h

#include <config-std.h>
#include <jsi/jsi.h>

/* The name of this JanosVM instance */
extern char *registryName;

/*
 * Compute the file path for the given export within the context of this
 * jvm instance.
 */
KAFFE_INLINE
void getRegistryPath(char *path, char *export, int create)
{
	jsiGetRegistryPath(path, registryName, export, create);
}
/*
 * Register an export with the given value
 */
void registerExport(char *export, char *fmt, ...);
/*
 * Unregister an export
 */
KAFFE_INLINE
void unregisterExport(char *export)
{
	jsiUnregisterExport(registryName, export);
}
/*
 * Get the value of an export
 */
KAFFE_INLINE
char *findExport(char *export)
{
	return( jsiFindExport(registryName, export) );
}
KAFFE_INLINE
int exportExists(char *export)
{
	return( jsiExportExists(registryName, export) );
}

/*
 * AddModule: registryModule
 *   ModuleDependency: gcModule
 *   Comment: Register this VM instance
 */

#endif
