/*
 * resource-inline.h
 * Manages VM internal resources
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "assert_pp.h"

KAFFE_INLINE
int rsrcInstall(resource *res)
{
	require(res != 0);
	require(res->funcs != 0);
	
	if( res->funcs->install )
		return( res->funcs->install(res) );
	else
		return( 1 );
}

KAFFE_INLINE
void rsrcUninstall(resource *res)
{
	require(res != 0);
	require(res->funcs != 0);
	
	if( res->funcs->uninstall )
		res->funcs->uninstall(res);
}

KAFFE_INLINE
int rsrcConfigure(resource *res, sectionFile *sf, tagSectionData *tsd)
{
	require(res != 0);
	require(res->funcs != 0);
	
	if( res->funcs->configure )
		return( res->funcs->configure(res, sf, tsd) );
	else
		return( 1 );
}

KAFFE_INLINE
int rsrcFillSpec(resource *res, resourceSpec *rs,
		 sectionFile *sf, tagSectionData *tsd)
{
	require(res != 0);
	require(res->funcs != 0);
	require(rs != 0);
	require(sf != 0);
	require(tsd != 0);
	
	if( res->funcs->fillSpec )
		return( res->funcs->fillSpec(res, rs, sf, tsd) );
	else
		return( 1 );
}

KAFFE_INLINE
resourceSpec *rsrcCreateJSISpec(resource *res, struct _jsiMessage *jm)
{
	require(res != 0);
	require(jm != 0);

	if( res->funcs->createJSISpec )
		return( res->funcs->createJSISpec(res, jm) );
	else
		return( 0 );
}

KAFFE_INLINE
void rsrcDeleteJSISpec(resourceSpec *rs)
{
	require(rs != 0);

	if( rs->handler->funcs->deleteJSISpec )
		rs->handler->funcs->deleteJSISpec(rs->handler, rs);
}

KAFFE_INLINE
resourceSpec *rsrcDefaultSpec(resource *res)
{
	require(res != 0);

	if( res->funcs->defaultSpec )
		return( res->funcs->defaultSpec(res) );
	else
		return( 0 );
}

KAFFE_INLINE
int rsrcSubscribe(resourceClient *rc, resourceSpec *rs)
{
	require(rc != 0);
	
	if( rc->handler->funcs->subscribe )
		return( rc->handler->funcs->subscribe(rc->handler, rc, rs) );
	else
		return( 1 );
}

KAFFE_INLINE
int rsrcEnable(resourceClient *rc)
{
	require(rc != 0);

	if( rc->handler->funcs->enable )
		return( rc->handler->funcs->enable(rc->handler, rc) );
	else
		return( 1 );
}

KAFFE_INLINE
void rsrcUnsubscribe(resourceClient *rc)
{
	require(rc != 0);
	
	if( rc->handler->funcs->unsubscribe )
		rc->handler->funcs->unsubscribe(rc->handler, rc);
}
