/*
 * scripting.h
 * An implementation for providing jsi scripting in the vm.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _scripting_h
#define _scripting_h

#include "sparseArray.h"

#include <jsi/jsi.h>

/* The tcp port we should listen on for scripting messages */
extern char *jsiProviderName;

/*
 * Simple structure to hold a list of files that can be used for sending
 * and receiving jsiMessages.
 *
 * `next' - Link to the next element in the list.
 * `filename' - The filename to use for the jsiChannel.
 */
typedef struct _scriptFile {
	struct _scriptFile *next;
	char *filename;
} scriptFile;

/*
 * Structure defining a jsi proxy connection.
 *
 * `handler' - The handler header.
 * `provider' - The provider specification that will accept incoming
 *  connections.
 * `destination' - The destination specification that messages will be
 *  forwarded too.
 */
typedef struct _proxyProvider {
	jsiConnectionHandler handler;
	sparseElement *elem;
	char *provider;
	char *destination;
	int disabled;
	struct _proxyProvider *next;
} proxyProvider;

/*
 * Add a script file to the internal list.  These will all have handler
 * threads started up for them.
 */
void addScriptFile(scriptFile *sf);
/*
 * Add a proxy to the list.
 */
int addScriptProxy(proxyProvider *pp);

/*
 * XXX Kind of a hack to make the channel available to the handler of the
 * message.
 */
typedef struct _scriptThreadData {
	jsiChannel *channel;
} scriptThreadData;

/*
 * returns - The scriptThreadData structure for the current thread.
 */
scriptThreadData *getScriptThreadData(void);

/*
 * Set whether threads should stall when calling stallVM().
 *
 * `stalled' - Boolean indicating whether any threads calling stallVM should
 *  block or not.  (True == no stall, false == stall).
 */
void setStallState(int stalled);
/*
 * Stall thread the current thread until setStallState(false) is called.
 * Typically this would be done by a script message.
 */
void stallVM(void);

extern jsiNamedHandler *jvmHandlerList;

/*
 * AddModule: scriptingModule
 *   ModuleDependency: registryModule threadModule
 *   Comment: Start up a JSI listener thread.
 */

#endif
