/*
 * jsiImpl.h
 * Low level functions required by the JanosVM Scripting Interface layer
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jsi_impl_h
#define _jsi_impl_h

/*
 * The jsiImpl layer is used to make it easier to use the JSI code outside
 * of JanosVM/Kaffe (e.g. an extension to a scripting language).
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <misc/stringParsing.h>

#include "assert_pp.h"

/*
 * If DO_JSI_INLINE is defined then we should inline these implementation
 * functions.
 */
#ifdef DO_JSI_INLINE
#define JSI_INLINE extern inline
#endif

struct sockaddr;
struct hostent;
struct stat;

struct _iLock;
#define JSI_LOCK_DECL struct _iLock *lock;
#define JSI_LOCK_PARAM struct _iLock **
#define jsiLock(x) _jsiLock(&((x)->lock), &iLockRoot)
#define jsiUnlock(x) _jsiUnlock(&((x)->lock), &iLockRoot)
#define jsiSignal(x) _jsiSignal(&((x)->lock))
#define jsiWait(x, y) _jsiWait(&((x)->lock), y)

#define JSI_RANDOM_SEQ_NUMBERS
#define JSI_RANDOM_CHANNEL_IDS

void jsiLog(char *fmt, ...);

#ifndef JSI_INLINE

/* These are basically the same as defined in `include/jsyscall.h' */
void *jsiMalloc(int size);
void *jsiRealloc(void *old, int size);
void jsiFree(void *mem);

void _jsiLock(JSI_LOCK_PARAM lock, int *iLockRoot);
void _jsiUnlock(JSI_LOCK_PARAM lock, int *iLockRoot);
int _jsiWait(JSI_LOCK_PARAM lock, int ms_timeout);
void _jsiSignal(JSI_LOCK_PARAM lock);

int jsiOpen(char *filename, int flags, int mode, int *out_fd);
int jsiFStat(int fd, struct stat *st);
int jsiStat(char *filename, struct stat *st);
int jsiRemove(char *filename);

int jsiMkDir(char *dirname, int mode);
int jsiRmDir(char *dirname);

int jsiSocket(int domain, int type, int protocol, int *out_fd);
int jsiConnect(int sockfd, struct sockaddr *serv, int addrlen);
int jsiBind(int sockfd, struct sockaddr *my, int addrlen);
int jsiListen(int sockfd, int backlog);
int jsiAccept(int sockfd, struct sockaddr *addr, int *addrlen, int *out_fd);

int jsiRecvFrom(int fd, void *buf, int len, unsigned int flags,
		struct sockaddr *from, int *fromlen, int *out_read);
int jsiRecvFromTimeout(int fd, void *buf, int len, unsigned int flags,
		       struct sockaddr *from, int *fromlen,
		       int ms_timeout, int *out_read);
int jsiSend(int fd, void *buf, int len, unsigned int flags);
int jsiSendTo(int fd, void *buf, int len, unsigned int flags,
	      struct sockaddr *to, int tolen);

int jsiSeek(int fd, off_t offset, int whence, off_t *out_offset);
int jsiRead(int fd, void *buffer, size_t bufferSize, size_t *out_read);
int jsiReadAndWait(int fd, void *buffer, size_t bufferSize, int sample_time,
		   size_t *out_read);
int jsiWrite(int fd, void *buffer, size_t bufferSize, size_t *out_written);

int jsiShutdown(int fd, int how);
int jsiClose(int fd);

int jsiGetHostByName(char *name, struct hostent **he);
int jsiGetHostByAddr(char *addr, int len, int type, struct hostent **he);

int jsiSetSockOpt(int sock, int level, int optname, void *optval, int optlen);

#else /* JSI_INLINE */

/*
 * There is some nasty subtlety here...  We want to be able to override the
 * implementation easily so we just make our own `jsiImpl-inline.h' file.
 * However, if we `#include "jsiImpl-inline.h"' it will still include the file
 * in the jsi directory.  So, we have to use `#include <jsiImpl-inline.h>' to
 * make sure it gets the file in the directory we're working in...
 */
#include <jsiImpl-inline.h>

#endif /* JSI_INLINE */

#endif
