/*
 * jsiPlexProvider.h
 * Provider for plex channels
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "jsi.h"
#include "jsiImpl.h"
#include "jsiPlexProvider.h"
#include "jsiPlexLink.h"

static
jsiProvider *jsiCreatePlexProvider(jsiConnectionHandler *ch)
{
	jsiProvider *retval = 0;
	jsiPlexProvider *pp;

	require(ch != 0);
	
	if( (pp = (jsiPlexProvider *)jsiMalloc(sizeof(jsiPlexProvider))) )
	{
		pp->provider.ops = &jsiPlexProviderImplementation;
		pp->provider.handleConnection = ch;
		pp->link = 0;
		retval = &pp->provider;
	}
	return( retval );
}

static
void jsiDeletePlexProvider(jsiProvider *jp)
{
	if( jp )
	{
		jsiDetachProvider(jp);
		jsiFree(jp);
	}
}

static
int jsiAttachPlexProvider(jsiProvider *jp, char *format)
{
	jsiPlexProvider *pp = (jsiPlexProvider *)jp;
	parsedString protocol, link, rest;
	int retval = 0;

	require(jp != 0);
	require(format != 0);

	if( parseString(format,
			SPO_NonEmptyString, &protocol,
			SPO_Expect, ":",
			SPO_NonEmptyString, &link,
			SPO_Expect, ":",
			SPO_String, &rest,
			SPO_End) )
	{
		jsiPlexLink *pl;
		
		require(!strncmp(protocol.data, "plex", 4));

		pl = jsiFindPlexLink(link.data, link.len);
		if( pl )
		{
			jsiPlexInLink *pil;
			int iLockRoot;
			
			jsiObtainPlexLink(pl);
			if( (pil = jsiGetPlexInLink(pl, link.data)) )
			{
				pp->link = pil;
				pil->provider = jp;
				retval = 1;
			}
			else
			{
#if JSI_PLEX_DEBUG >= JSI_PLEX_DEBUG_ERRORS
				fprintf(stderr,
					"Couldn't construct plex in link from "
					"format `%s'\n",
					link.data);
#endif
			}
			jsiReleasePlexLink(pl);
		}
		else
		{
#if JSI_PLEX_DEBUG >= JSI_PLEX_DEBUG_ERRORS
			fprintf(stderr,
				"Couldn't find plex link `%s'\n",
				link.data);
#endif
		}
	}
	return( retval );
}

static
void jsiDetachPlexProvider(jsiProvider *jp)
{
	jsiPlexProvider *pp = (jsiPlexProvider *)jp;

	require(jp != 0);

	if( pp->link != 0 )
	{
		pp->link->provider = 0;
		jsiDropPlexInLink(pp->link);
		pp->link = 0;
	}
}

static
void jsiHandlePlexProvider(jsiProvider *jp)
{
	jsiPlexProvider *pp = (jsiPlexProvider *)jp;
	jsiChannel *jc;
	int done = 0;
	
	require(jp != 0);
	require(pp->link != 0);

	while( !done && (jc = jsiHandlePlexLink(pp->link)) )
	{
		done = jp->handleConnection->func(jp, jc);
	}
}

static
char *jsiPlexProviderNames[] = {
	"plex",
	0
};

jsiProviderFunctions jsiPlexProviderImplementation = {
	jsiPlexProviderNames,
	jsiCreatePlexProvider,
	jsiDeletePlexProvider,
	jsiAttachPlexProvider,
	jsiDetachPlexProvider,
	jsiHandlePlexProvider
};
