/*
 * jsiProvider.c
 * Functions for manipulating jsiProviders
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "jsi.h"
#include "jsiImpl.h"
#include "jsiStdProvider.h"
#include "jsiPlexProvider.h"

static
jsiProviderFunctions *jsiProviderImplementations[] = {
	&jsiStdProviderImplementation,
	&jsiPlexProviderImplementation,
	0
};

void jsiPrintProviderImplementations(FILE *file)
{
	int lpc;

	fprintf(file, "Providers:\n");
	for( lpc = 0; jsiProviderImplementations[lpc]; lpc++ )
	{
		fprintf(file,
			"  %s\n",
			jsiProviderImplementations[lpc]->names[0]);
	}
}

jsiProvider *jsiAttachProviderTo(char *format,
				 char *defproto,
				 jsiConnectionHandler *ch)
{
	parsedString protocol, rest;
	jsiProvider *retval = 0;
	int parsed;

	require(format != 0);
	require(ch != 0);
	
	if( (parsed = parseString(format,
				  SPO_NonEmptyString, &protocol,
				  SPO_Expect, ":",
				  SPO_NonEmptyString, &rest,
				  SPO_End)) ||
	    defproto )
	{
		jsiProviderFunctions *pf = 0;
		int lpc, lpc2, found = 0;
		jsiProvider *jp;

		if( !parsed )
		{
			protocol.data = defproto;
			protocol.len = strlen(defproto);
		}
		for( lpc = 0;
		     jsiProviderImplementations[lpc] && !found;
		     lpc++ )
		{
			pf = jsiProviderImplementations[lpc];
			for( lpc2 = 0;
			     !found && !retval && pf->names[lpc2];
			     lpc2++ )
			{
				if( !cmpPStrStr(&protocol, pf->names[lpc2]) )
				{
					found = 1;
				}
			}
		}
		if( found && (jp = pf->create(ch)) )
		{
			require(jp->ops != 0);
			
			if( pf->attach(jp, format) )
			{
				retval = jp;
			}
			else
			{
				jsiDeleteProvider(jp);
			}
		}
	}
	return( retval );
}

void jsiDeleteProvider(jsiProvider *jp)
{
	require(jp != 0);
	
	jp->ops->delete(jp);
}

void jsiDetachProvider(jsiProvider *jp)
{
	require(jp != 0);
	
	jp->ops->detach(jp);
}

void jsiHandleProvider(jsiProvider *jp)
{
	require(jp != 0);
	
	jp->ops->handle(jp);
}
