/*
 * jsiStdChannel.h
 * Channel implementation that runs over TCP/Files
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jsi_std_channel_h
#define _jsi_std_channel_h

/*
 * A jsiStdChannel is direct binding to IP transports, these are only provided
 * as a convenience, and not meant as the only available set transports for JSI
 * messages.
 *
 * `protocol' - Which IP protocol to use.
 * `inputFD' - The file descriptor used for input.  If its -1 then it isn't
 *    open.
 * `outputFD' - The file descriptor used for output.  If its -1 then it isn't
 *    open.
 */

/* JSI Channel Protocol options */
enum {
	JCP_NONE,
	JCP_SELF,	/* Send the message to this process */
	JCP_UDP,	/* Use UDP as the transport */
	JCP_TCP,	/* Use TCP as the transport */
	JCP_FILE,	/* Use a file as the transport */
	JCP_MAX
};

typedef struct _jsiStdChannel {
	jsiChannel channel;
	int protocol;
	int inputFD;
	int outputFD;
} jsiStdChannel;

/*
 * Allocate and initialize a jsiChannel object.
 */
jsiChannel *jsiCreateStdChannel(void);
/*
 * Connect the channel to something, the method of connection is determined
 * by the JCA_* tags passed in.
 */
int jsiConnectStdChannel(jsiChannel *jc, int tag, ...);

extern jsiChannelFunctions jsiStdChannelImplementation;

#endif
