/*
 * heyActions.h
 * Actions that jsihey can perform on a reply message
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _hey_actions_h
#define _hey_actions_h

/*
 * The heyActions are provided to act on the JSI reply message.  If no action
 * is specified the message is printed out in a predetermined format.  However,
 * this format might be inconvenient so we provide a simple alternative through
 * the actions framework.
 */

#include "jsi.h"

#include "heyTools.h"

/*
 * The heyMsgAction structure is used to contain the message to be acted on
 * as well as the action and its arguments.
 *
 * `msg' - The message to process.
 * `action' - The action we should perform on `msg'.
 * `actionValues' - The arguments for the action.
 * `actionValueCount' - The size of the `actionValues' array.
 */

enum {
	ACTION_ERROR,
	ACTION_NONE,
	ACTION_NOTHING,
	ACTION_DEFAULT,
	ACTION_FOR_INSTANCE,
	ACTION_PRINT,
	ACTION_PRINT_SUITES,
};

typedef struct _heyMsgAction {
	jsiMessage *msg;
	int action;
	char **actionValues;
	int actionValueCount;
} heyMsgAction;

/* Execute the action over the message */
int doAction(heyMsgAction *hma);

/* A mapping of action names to the internal integers */
extern stringIntMap actionMappings[];

#endif
