/*
 * heyTools.c
 * Support functions for jsihey
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "assert_pp.h"

#include "heyTools.h"

/* These functions are all pretty straightforward */

int mapStringToInt(stringIntMap *map, char *str)
{
	int lpc, retval = 0;

	require(map != 0);
	require(str != 0);
	
	for( lpc = 0; (retval == 0) && map[lpc].str; lpc++ )
	{
		if( !strcasecmp(str, map[lpc].str) )
			retval = map[lpc].id;
	}
	return( retval );
}

char *mapIntToString(stringIntMap *map, int what)
{
	char *retval = 0;
	int lpc;

	require(map != 0);

	for( lpc = 0; !retval && map[lpc].str; lpc++ )
	{
		if( map[lpc].id == what )
			retval = map[lpc].str;
	}
	return( retval );
}

/* Linked list of localVariables */
static
localVariable *heyVariables = 0;

void addVariable(localVariable *lv)
{
	require(lv != 0);
	
	lv->next = heyVariables;
	heyVariables = lv;
}

void remVariable(localVariable *lv)
{
	localVariable *curr, **prev;

	require(lv != 0);
	
	prev = &heyVariables;
	curr = heyVariables;
	while( curr != lv )
	{
		prev = &curr->next;
		curr = curr->next;
	}
	*prev = lv->next;
}

localVariable *findVariable(char *name)
{
	localVariable *lv, *retval = 0;

	require(name != 0);
	
	lv = heyVariables;
	while( lv )
	{
		if( !strcmp(lv->name, name) )
			retval = lv;
		lv = lv->next;
	}
	return( retval );
}

/* XXX defined in main */
extern char *progName;

void printError(char *msg, ...)
{
	va_list args;

	require(msg != 0);
	
	va_start(args, msg);
	fprintf(stderr, "%s Error: ", progName);
	vfprintf(stderr, msg, args);
	fprintf(stderr, "\n");
	va_end(args);
}

void printWarning(char *msg, ...)
{
	va_list args;

	require(msg != 0);
	
	va_start(args, msg);
	fprintf(stderr, "%s Warning: ", progName);
	vfprintf(stderr, msg, args);
	fprintf(stderr, "\n");
	va_end(args);
}
