/*
 * heyTools.h
 * Support functions for jsihey
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _hey_tools_h
#define _hey_tools_h

/*
 * The heyTools are just some simple helper functions for other parts of the
 * `jsihey' program.
 */

#include "jsi.h"

/* A map from a given string to an integer value */
typedef struct _stringIntMap {
	char *str;
	int id;
} stringIntMap;

/* Map a string to an integer using the given mapping */
int mapStringToInt(stringIntMap *map, char *str);
/* Map the given integer value to a string using the given mapping */
char *mapIntToString(stringIntMap *map, int i);

/*
 * A localVariable structure is used to track any integer variables that are
 * declared/used by the actions.
 *
 * `next' - Link to the next variable in the global list.
 * `name' - The name of the variable.
 * `value' - The current value of the variable.
 */
typedef struct _localVariable {
	struct _localVariable *next;
	char *name;
	int value;
} localVariable;

/* Add a variable to the namespace */
void addVariable(localVariable *lv);
/* Remove the variable from the namespace */
void remVariable(localVariable *lv);
/* Find a variable with the given name */
localVariable *findVariable(char *name);

/* Print an error message */
void printError(char *msg, ...);
/* Print a warning message */
void printWarning(char *msg, ...);

#endif
