/*
 * external.h
 * Handle method calls to other languages.
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#if 1

#include <janosvm/nativeResource.h>

#else

#ifndef __external_h
#define __external_h

#define	MAXSTUBLEN	1024
#define	MAXLIBPATH	1024
#define	MAXLIBS		16

/* Default library info. */
#if !defined(LIBRARYPATH)
#define	LIBRARYPATH	"KAFFELIBRARYPATH"
#endif
#define	NATIVELIBRARY	"native"
#define	VMLIBRARY	"kaffevm"
#define	MISCLIBRARY	"kaffemisc"

struct _methods;
struct _errorInfo;

/*
 * Library name break down:
 *
 * /usr/local/lib/libkaffevm.la
 *
 * ^^^^^^^^^^^^^^^^^^^^^^^^^     The full library path
 * ^^^^^^^^^^^^^^^               A library path entry
 *                ^^^            Implementation specific prefix
 *                   ^^^^^^^     The base library name
 *                               Implementation specific suffix (none here)
 *                          ^^^  Suffix handled by libltdl
 */

/*
 * Load a native library into the system, if the library has already been
 * loaded then it will increment a reference count and return.  Otherwise,
 * it will try to load the library from the file system and run any module
 * initializers in the library.
 *
 * `path' - The full path for the library to load.  Use the findLibrary
 *   function if you don't know the full path and prefix of the library.
 * `errbuf' - A character array that can hold an error string.
 * `errsiz' - The size of the `errbuf' array.
 * returns - -1, if the load failed, otherwise its an index number that can
 *   be passed to unloadNativeLibrary to unload this library.
 */
int	loadNativeLibrary(char *path, char *errbuf, size_t errsiz);
/*
 * Similar to the above except it allows you specify the reference count to use
 * when the library is first loaded.  This is primarily useful when preloading
 * libraries so that you can give it a zero reference count thus allowing the
 * library to eventually be unloaded without you having to call unloadLibrary.
 * NOTE: This should only be called from within the kernel team.
 */
int	loadNativeLibrary2(char *path, int default_refs,
			   char*errbuf, size_t errsiz);
/*
 * Unload the native library from the system.  Basically, this will just
 * decrement the reference count of the library and then release it if it
 * reaches zero.
 *
 * `index' - The library index as returned by loadNativeLibrary.
 */
void	unloadNativeLibrary(int index);
/**
 * Locate the given symbol in one of the loaded libraries and return a pointer
 * to its value.
 *
 * `name' - The name of the symbol.
 * returns - A pointer to the symbol or NULL if it wasn't found.
 */
void*	loadNativeLibrarySym(char *name);
/*
 * Locate and construct a stub that will call the native implementation of the
 * given method.  Since Kaffe supports two different native interfaces this
 * function will first try to bind to the KNI function and then try the
 * JNI version.  If both of these fail then it will install a pointer to a
 * predefined error stub that will just returns zero.  (XXX Is this wise?
 * should it throw an exception?)
 *
 * `method' - The native method to construct a stub for.
 * `einfo' - An errorInfo structure to hold any exception info.
 * returns - True if the stub was constructed successfully, false otherwise.
 */
bool	native(struct _methods *method, struct _errorInfo *einfo);
/*
 * returns - The library path that Kaffe is using.
 */
char*	getLibraryPath(void);
/*
 * Find and load a native library with the given name.  The function will
 * search through all of the library directories until it either finds the
 * library or runs out of paths.
 *
 * `baseName' - The name of the library to load, this doesn't include any
 * paths, prefixes, or suffixes.
 * returns - -1, if the load failed, otherwise it returns the index number
 * as returned by loadNativeLibrary.
 */
int	findLibrary(char *baseName);

/*
 * Add Module: externalModule
 *   Module Dependency: gcModule.heap
 *   Com ment: Initialize the library paths and load libkaffevm/libnative
 */

#endif

#endif
