/*
 * gc.c
 * The garbage collector.
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

/* Use the incremental garbage collector */
/* XXX put this in a make that a separate file !!! */
#include "mem/gc-incremental.c"

#include <janosvm/memResource.h>
#include <janosvm/jteam.h>

Collector *main_collector;

void*
jmalloc(size_t sz)
{
	void *retval = 0;

	if( switchToTeam(kernelTeam, 0) )
	{
		retval = gc_malloc(sz, GC_ALLOC_FIXED);
		returnFromTeam();
	}
	return( retval );
}

void*
jrealloc(void* mem, size_t sz)
{
	void *retval = 0;

	if( switchToTeam(kernelTeam, 0) )
	{
		retval = gc_realloc(mem, sz, GC_ALLOC_FIXED);
		returnFromTeam();
	}
	return( retval );
}

void
jfree(void* mem)
{
	if( switchToTeam(kernelTeam, 0) )
	{
		gc_free(mem);
		returnFromTeam();
	}
	else
	{
		fprintf(stderr,
			"Ack!  Unable to free kernel allocated memory!\n");
	}
}

/*
 * Functions versions of gc_malloc/gc_realloc/gc_free for use in native
 * libraries that do not include gc.h/jthread.h
 */
void*
_gc_malloc(size_t sz)
{
	return (gc_malloc(sz, GC_ALLOC_FIXED));
}

void
_gc_free(void *mem)
{
	gc_free(mem);
}
