/*
 * module.c
 * A simple system for managing program modules
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdio.h>
#include <stdlib.h>

#include <assert_pp.h>

#include "module.h"
#include "moduleList.h"
#include "debug.h"

int initModules(moduleInitializer_t *mi)
{
	int retval = 1;
	int lpc;

	require(mi != 0);
	
	for( lpc = 0; retval && mi[lpc].module; lpc++ )
	{
		DBG(MODULES,
		    dprintf("Initializing module %s (%d)\n", mi[lpc].module->name, mi[lpc].stage);
		    );
		if( mi[lpc].module->init )
		{
			if( !(retval = mi[lpc].module->init(mi[lpc].stage)) )
			{
				fprintf(stderr,
					"Stage %d of module `%s' failed to "
					"initialize.\n",
					mi[lpc].stage,
					mi[lpc].module->name);
			}
		}
	}
	return( retval );
}

void killModules(moduleInitializer_t *mi)
{
	int lpc;

	require(mi != 0);
	
	/*
	 * Walk to the end of the array so we can do the finalize in the
	 * reverse order of the initialization.
	 */
	for( lpc = 0; mi[lpc].module; lpc++ );
	for( lpc--; lpc >= 0; lpc-- )
	{
		DBG(MODULES,
		    dprintf("Killing module %s (%d)\n", mi[lpc].module->name, mi[lpc].stage);
		    );
		if( mi[lpc].module->kill )
		{
			mi[lpc].module->kill(mi[lpc].stage);
		}
	}
}

int testModules(module_t **mods)
{
	int lpc, retval = 1;

	require(mods != 0);
	
	for( lpc = 0; retval && mods[lpc]; lpc++ )
	{
		if( mods[lpc]->test )
		{
			if( !(retval = mods[lpc]->test()) )
			{
				fprintf(stderr,
					"The internal test function of module "
					"`%s' failed.\n",
					mods[lpc]->name);
			}
		}
	}
	return( retval );
}

void diagnoseModules(module_t **mods)
{
	int lpc;

	require(mods != 0);
	
	for( lpc = 0; mods[lpc]; lpc++ )
	{
		if( mods[lpc]->diagnose )
		{
			mods[lpc]->diagnose();
		}
	}
}
