/*
 * mungWallProxy.h
 * A gcProxy that tries to detect overflow/underflow in memory buffers.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef mungWallProxy_h
#define mungWallProxy_h

#include <assert_pp.h>

#include "debug.h"
#include "gcProxy.h"

/*
 * Wrap a mungWallProxy around the given collector.
 *
 * `gc' - The collector to wrap with mungWall functions.
 * returns - True if the wrapping was successful, false otherwise.
 */
int addMungWallProxy(Collector *gc);
/*
 * Wrap a mungWallProxy around the given collector.
 *
 * `gc' - The collector to wrap with mungWall functions.
 * `mem' - The proxy object to use when wrapping the `gc'.
 * returns - True if the wrapping was successful, false otherwise.
 */
int addMungWallProxy_mem(Collector *gc, gcProxy *mem);
/*
 * Check the given memory to see if its walls have been broken.  If a wall
 * has been broken then it will print out the memory and abort the process.
 *
 * `storage' - The memory to validate.  NOTE: This will not validate java
 *   and other non-GC'ed objects.
 */
void mungValidateMemory(void *storage);

/*
 * Precondition macro that validates the memory pointer.
 */
#ifndef NO_PRECONDITIONS
#define require_valid_memory(ptr) \
do \
{ \
	DBG(MUNGWALL, mungValidateMemory(ptr)); \
} while(0);
#else
#define require_valid_memory(ptr)
#endif

/*
 * Postcondition macro that validates the memory pointer.
 */
#ifndef NO_POSTCONDITIONS
#define ensure_valid_memory(ptr) \
do \
{ \
	DBG(MUNGWALL, mungValidateMemory(ptr)); \
} while(0);
#else
#define ensure_valid_memory(ptr)
#endif

#endif
