/*
 * Copyright (c) 1999, 2000 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * jthread's various flavors of "current thread"
 */

#ifndef anjthread_current_inline_h
#define anjthread_current_inline_h

#include "anjthread_types_priv.h"

static const size_t jthread_ntidOffset = ((size_t)(&(((jthread_t)NULL)->native_thread)));

ANJTHREAD_INLINE jthread_t 
jthread_fromanthread(an_thread_t const * const ath) __UNUSED__;

ANJTHREAD_INLINE jthread_t
jthread_fromanthread(an_thread_t const * const ath)
{
	void *jth;

	require(ath);
	
	assert(jthread_ntidOffset > 0);
	assert(jthread_ntidOffset < sizeof(struct _jthread));
	
	jth = ((void*)ath) - jthread_ntidOffset;
	assert(jth);
	assert((&((jthread_t)jth)->native_thread) == ath);
	
	return (jthread_t)jth;
}

/* 
 * Return the current underlying native thread identifier.
 * 
 * XXX this shouldn't be in the public header file?  Change the
 * jthread_ prefix at least.
 */
ANJTHREAD_INLINE an_thread_t*
jthread_currentNative(void)
{
	return an_thread_current();
}

ANJTHREAD_INLINE jthread_t
jthread_currentJthread(void)
{
	an_thread_t* anth;
	jthread_t jth;
	anth = an_thread_current();
	assert(anth);
	jth = jthread_fromanthread(anth);
	assert(jth);
	assert((jth->daemon == 0) || (jth->daemon == 1));
	return jth;
}

ANJTHREAD_INLINE jthread_t
jthread_current(void) 
{ 
	return jthread_currentJthread();
}

/*
 * Find a native thread's cookie (i.e., the jthread associated with it).
 *
 * XXX rename to jthread_currentJavaThread()
 */
ANJTHREAD_INLINE struct Hjava_lang_Thread* 
jthread_getcookie(const jthread_t jtid)
{
	assert(jtid);
	return jtid->jlThread;
}

#endif /* anjthread_current_inline_h */
