/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * Jthread initialization routines
 */

#ifndef anjthread_init_h
#define anjthread_init_h

/*
 * Init the root thread with some very basic (statically defined)
 * information.  Enough so it can take a lock.
 */
void
anjthread_preinitroot(struct _jthread* rootthread);

/* 
 * initialize the threading system
 */
struct _jthread*
jthread_init(
	int preemptive,			/* preemptive scheduling */
	int maxpr, 			/* maximum priority */
	int minpr, 			/* minimum priority */
        void *(*_allocator)(size_t),	/* memory allocator XXX deprecated */
	void (*_deallocator)(void*),	/* memory deallocator XXX deprecated */
	void (*_destructor1)(void*),	/* called when a thread exits */ 
	void (*_onstop)(void),		/* called when a thread is stopped */
	void (*_ondeadlock)(void));     /* called on deadlock */


/*
 * Create a NodeOS jthread context to go with the initial thread in the system.
 * (The initial thread is created in jthread_init, above.
 */
struct _jthread*
jthread_createfirst(size_t mainThreadStackSize, 
		    unsigned char prio, 
		    void* jlThread);

/*
 * Migrate the root thread and any other preallocated threads to the kernel
 * team.
 */
struct _jteam;
void
jthread_migrate_root(struct _jteam *jt);

/*
 * set a function to be run when all non-daemon threads have exited
 *
 * May only be called once (to register a single function).
 */
void
jthread_atexit(void (*handler)(void));

#endif /* anjthread_init_h */

