/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * jthread interruption-related routines.  Signals/interrupts/stop/suspendall.
 */

#ifndef anjthread_intr_h
#define anjthread_intr_h

/*                                                                      
 * Initialize handlers for null pointer accesses and div by zero        
 */             
void jthread_initexceptions(jthread_exchandler_t _nullHandler,
			    jthread_exchandler_t _floatingHandler);

/*
 * XXX 
 */
void _jthread_intr_handler(int type, void *arg);

/*
 * Handle an anjthread interruption (i.e, one of the JTHREAD_FLAG_*
 * interrupts.  Will not return if the thread was actually
 * interrupted for slaughter/death.
 */
void jthread_handleInterrupt(jthread_t jtid);

/*
 * Interrupt a thread.  Silently ignore interrupts
 * of the current thread.  Interruption will wake a thread from 
 * sleep(), wait(), or any interruptible system call.
 */
void jthread_interrupt(struct _jthread* tid);

/*
 * Stop a thread in its (java) tracks.  Causes the target thread
 * to jthread_die() --- throwing a ThreadDeath exception.
 *
 * NOTE: the target thread is not stopped immediately.  It is
 * asked to stop itself.  It will eventually get around to that.
 */
void jthread_stop(struct _jthread* jtid);

/*
 * Slaughter the given thread.  It does not "die" by throwing a
 * ThreadDeath exception, it just disappears into a void never to
 * return.
 *
 * NOTE: the thread is not slaughtered immediately.  It is asked
 * to slaughter itself. It will eventually get around to that.
 */
int jthread_slaughter(struct _jthread* jtid);

/*
 * Disallow _stop() or _slaughter() of the current thread.
 */
void jthread_disable_stop(void);

/*
 * Reallow _stop (or _slaughter) of the current thread.
 * If a stop/slaughter is pending, the thread immediately
 * performs the stop/slaughter.
 */
void jthread_enable_stop(void);

/*
 * Suspend all (other) threads in this flow.
 */
void jthread_suspendall(void);

/*
 * Undo previous suspension.
 */
void jthread_unsuspendall(void);

/*
 * Part of the signals-based API of jthreads.  Not useful on Moab.
 */
void unblockAsyncSignals(void);
void blockAsyncSignals(void);

#include "anjthread_inline.h"
#include "anjthread_intr-inline.h"

#endif /* anjthread_intr_h */

