/*
 * Copyright (c) 1998, 1999, 2000 The University of Utah. All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */
/*
 * Locking Subsystem.
 *
 * All the other functions are defined (inline) in lock-impl.h
 */

#include "anjthread_types.h"
#include "anjthread_types_priv.h"
#include "anjthread_lock.h"

jbool
jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout)
{
	an_timespec_t abstime;
	struct timeval now;

	if (timeout == (jlong)0) {
		an_cond_wait(cv, lock);
		return true;
	}

	/* Need to convert timeout to an abstime. Very dumb! */
	gettimeofday(&now, 0);
	TIMEVAL_TO_TIMESPEC(&now, &abstime);

	abstime.tv_sec  += timeout / 1000;
	abstime.tv_nsec += (timeout % 1000) * 1000000;
	if (abstime.tv_nsec > 1000000000) {
		abstime.tv_sec  += 1;
		abstime.tv_nsec -= 1000000000;
	}
	an_cond_timedwait(cv, lock, &abstime);
	
	return true;  /* XXX what should I be returning?? */
}
