/*
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * jthread status functions (inline variety)
 */

#ifndef anjthread_status_inline_h
#define anjthread_status_inline_h

#include "config.h"
#include "anjthread_types.h"
#include "anjthread_types_priv.h"
#include "anjthread_current-inline.h"
#include "gtypes.h"
#include "jtypes.h"
#include "debug.h"

#ifdef DEBUG
#define	ANJTHREAD_REDZONE	2048 /* XXX put this in _globals.h */
#else
#define	ANJTHREAD_REDZONE	1024 /* XXX put this in _globals.h */
#endif

ANJTHREAD_INLINE threadData *
jthread_get_data(jthread_t th)
{
	return &(th->threadData);
}

ANJTHREAD_INLINE void
jthread_relaxstack(int yes)
{
	if( yes )
		jthread_current()->flags |= JTHREAD_FLAG_RELAXED_STACK;
	else
		jthread_current()->flags &= ~JTHREAD_FLAG_RELAXED_STACK;
}

ANJTHREAD_INLINE void*
jthread_stacklimit(void)
{
	void *stackbase;
	int stacksize;
	void *stackptr;

	an_thread_stackinfo(an_thread_current(),
			    &stackbase,
			    &stacksize,
			    &stackptr);

#if defined(STACK_GROWS_UP)
#error FIXME
#else
	if( jthread_current()->flags & JTHREAD_FLAG_RELAXED_STACK )
		return (void*)(((long)(stackbase)));
	else
		return (void*)(((long)(stackbase)) + ANJTHREAD_REDZONE);
#endif
}

/*
 * Determine whether an address lies on the stack frame
 * of the current thread.
 */
ANJTHREAD_INLINE int
jthread_on_current_stack(void *bp)
{
	int rc;
	void *stackbase;
	int stacksize;
	void *stackptr;
	an_thread_t* curth;
	an_error_t err;
#ifndef NDEBUG
	void *fp = __builtin_frame_address(0);
#endif
	
	if (bp == NULL)
		return 0;

	curth = an_thread_current();
	err = an_thread_stackinfo(curth,
				  &stackbase,
				  &stacksize,
				  &stackptr);
	assert(err == 0); /* XXX */

#if defined(STACK_GROWS_UP)
#error must write code for up growing stacks
#else
	/*     |***********|--ACTIVE-STACK---|          */
	/* stackbase    stackptr    stackbase+stacksize */

	assert(fp > stackbase);
	assert(fp < (stackbase + stacksize));
	assert(fp >= stackptr);

        rc = (bp >= stackbase)
		&& (bp < (stackbase + stacksize));
#endif


DBG(JTHREADDETAIL,
	dprintf("on_current_stack(): jth=%p ptr=%p; base=%p size=%d %s\n",
		curth, bp,
		stackbase, stacksize, (rc ? "yes" : "no"));
    )
	return rc;
}       

#endif /* anjthread_status_inline_h */
