/*
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * "Internal" types to anjthread.  (Not really private, as we have
 * lots of inline functions.)
 */

#ifndef anjthread_types_priv_h
#define anjthread_types_priv_h

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "assert_pp.h"
#include "gtypes.h"
#include "thread.h"
#include "threadData.h"

#include <misc/logger.h>

#include "anjthread_types.h"

/*
 * This is our internal structure representing a "jthread" which is mapped
 * on top of a NodeOS thread.
 */
typedef struct _jthread {
#if defined(THREAD_ANALYZER)
	logAnalyzerMask		lam;
#endif
	struct _jthread*	nextlive;	  /* Live threads chain */
	int			daemon;		  /* Is a daemon thread? XXX put in FLAGS.  XXX dupl in jlThread.*/
	/* XXX keeping this around for the life of thread is stupid. */
	void			(*func)(void *);  /* Startup function */
	threadData		threadData;       /* thread-specific data managed by ../../thread.c */
	struct Hjava_lang_Thread*	jlThread; /* java_lang_thread ptr */
	unsigned char		status;		  /* Thread status */
	unsigned char		flags; 		  /* Thread flags */
	/* XXX we track this only because pthreads DROPS it after the thread exits. */ 
	void*			stackptr;
	/* XXX the stopcounter should revert to a single bit */
	unsigned char		stopCounter;	  /* Count nested enable_stop calls  */
	an_thread_t		native_thread;    /* NodeOS thread object. */
} *jthread_t;

/* 
 * Thread status.  Mutually exclusive states.  (Ie, not flags).
 *
 * The status is protected by a spin-lock on the thread.
 */
#define JTHREAD_STATUS_NONE             0
#define JTHREAD_STATUS_NEWBORN          1 /* explicit threads only */
#define JTHREAD_STATUS_INNODEOS		2 /* partial threads only */
#define JTHREAD_STATUS_RUNNING          3
#define JTHREAD_STATUS_DYING            4
#define JTHREAD_STATUS_DEAD             5
#define JTHREAD_STATUS_MAX              6

/* 
 * Thread flags.  Any subset may be enabled/disabled. 
 *
 * The flags are protected by a spin-lock on the thread.
 */
#define JTHREAD_FLAG_ROOTTHREAD		0x01
#define JTHREAD_FLAG_DYING		0x02
#define JTHREAD_FLAG_INTR_SLAUGHTER	0x04
#define JTHREAD_FLAG_INTR_STOP		0x08
#define JTHREAD_FLAG_RELAXED_STACK	0x10
#define JTHREAD_FLAG_INTR_MASK		(JTHREAD_FLAG_INTR_SLAUGHTER|JTHREAD_FLAG_INTR_STOP)

#endif /* anjthread_types_priv_h */
