/*
 * Copyright (c) 1999, 2000 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * jthread walk functions.
 */

#include "anjthread_types.h"
#include "anjthread_types_priv.h"
#include "anjthread_globals.h"
#include "anjthread_walk.h"

/*
 * iterate over all live threads
 */
void
jthread_walkLiveThreads(void *collector, void (*func)(void *c, jthread_t jtid, void *mem))
{
        jthread_t tid;

	an_mutex_lock(&jthread_creationLock);
        for (tid = jthread_liveThreads; tid != NULL; tid = tid->nextlive) {
                func(collector, tid, tid->jlThread);
        }
	an_mutex_unlock(&jthread_creationLock);
}

/*
 * Walk all the live threads.  Takes the internal jthread_creationLock.  
 */
void
jthread_walk_jthreads(void *collector, void (*func)(void *c, jthread_t jtid))
{
        jthread_t tid;

	an_mutex_lock(&jthread_creationLock);
        for (tid = jthread_liveThreads; tid != NULL; tid = tid->nextlive) {
                func(collector, tid);
        }
	an_mutex_unlock(&jthread_creationLock);
}

