/*
 * Copyright (c) 1996, 1997, 1998, 1999
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 */

#ifndef __lock_impl_h
#define __lock_impl_h

struct _jthread;

/*
 * a mutex has pointer to the holder and a queue of threads waiting
 * on the mutex.
 */
typedef struct _jmutex
{
        struct _jthread *holder;
        struct _jthread *waiting;
} jmutex;

/*
 * a condition variable is simply expressed as a list of threads
 * waiting to be notified
 */
typedef struct _jthread *jcondvar;

#define JMUTEX_STATIC_INITIALIZER { NULL, NULL }
#define JCONDVAR_STATIC_INITIALIZER NULL

/* 
 * Locking API
 */
extern void jmutex_initialise(jmutex *lock);
extern void jmutex_lock(jmutex *lock);
extern void jmutex_unlock(jmutex *lock);
#define JMUTEX_BLOCKED
/* Return a list of threads blocked on a mutex.  Caller frees */
int  jmutex_blocked(jmutex *lock, struct _jthread ***list);
extern void jmutex_destroy(jmutex *lock);

extern void jcondvar_initialise(jcondvar *cv);
extern jboolean jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout);
extern void jcondvar_signal(jcondvar *cv, jmutex *lock);
extern void jcondvar_broadcast(jcondvar *cv, jmutex *lock);
#define JCONDVAR_WAITING
/* Return a list of threads waiting on a condvar.  Caller frees */
int  jcondvar_waiting(jcondvar *cv, struct _jthread ***list);
extern void jcondvar_destroy(jcondvar *lock);

#endif /* __lock_impl_h */
