/*
 * thread.h
 * Thread support.
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#ifndef __thread_h
#define __thread_h

#include "md.h"

#include "native.h"
#include "errors.h"
#include "java_lang_Throwable.h"
#include "java_lang_Thread.h"
#include "java_lang_ThreadGroup.h"

#define	THREADCLASS			"java/lang/Thread"
#define	THREADGROUPCLASS		"java/lang/ThreadGroup"
#define	THREADDEATHCLASS		"java/lang/ThreadDeath"

#define THREAD_MAXPRIO  		(java_lang_Thread_MAX_PRIORITY+1)

struct Hjava_lang_Runnable;

/*
 * Interface to the thread system.
 */
void	initThreads(void);
void    initNativeThreads(int nativestacksize);
void	yieldThread(void);
void	sleepThread(jlong);
void	exitThread(void) __NORETURN__;
bool	aliveThread(Hjava_lang_Thread*);
jint	framesThread(Hjava_lang_Thread*);
void	setDaemonThread(Hjava_lang_Thread*, jbool);
void	setPriorityThread(Hjava_lang_Thread*, jint);
void	startThread(Hjava_lang_Thread*);
void	startThreadOn(Hjava_lang_Thread *, struct Hjava_lang_Runnable*);
void	stopThread(Hjava_lang_Thread*, Hjava_lang_Object*);
void	destroyThread(Hjava_lang_Thread*);
void	interruptThread(Hjava_lang_Thread*);
void	finalizeThread(Hjava_lang_Thread*);

struct _Collector;
void	destructThread(struct _Collector* collector, void* obj);
char*	nameThread(Hjava_lang_Thread*);
char*	nameNativeThread(void*);
Hjava_lang_Thread* getCurrentThread(void);
Hjava_lang_Thread* createDaemon(void*, const char*, void *arg, int,
				size_t, struct _errorInfo *);
extern  Hjava_lang_Class* ThreadClass;

extern	jbool deadlockDetection;
extern int threadStackSize;
extern Hjava_lang_ThreadGroup* standardGroup;

#define THREAD_NATIVE()         ((void*)jthread_current())

#if !defined(KAFFEH)
struct _gc_collector;
#endif

extern void dumpThreads(void);

/*
 * AddModule: threadModule
 *   ModuleDependency: gcModule.md
 *   Comment: Initialize basic jthreads
 * Transition: jthread
 *   ModuleDependency: exceptionModule nameSpaceModule.initMetal
 *   Comment: Finish initializing the jthreads and create the main
 *   Comment: java.lang.Thread
 */

#endif
