/*
 * threadData-inline.h
 * Manage any thread specific data.
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _thread_data_inline
#define _thread_data_inline

#include "assert_pp.h"
#include <janosvm/jteam.h>

/* Get the frame pointer of the caller */
#define getCallerFrameAndPC(fp, pc) \
{ \
	exceptionFrame base; \
\
	FIRSTFRAME(base, base); \
	*(fp) = (void *)FPFRAME(&base); \
	*(pc) = (void *)PCFRAME(&base) + 1; \
}

/* Get the previous frame pointer from the given one */
KAFFE_INLINE
void *getPreviousFramePointer(void *fp)
{
	return( (void *)NEXTFRAME(fp) );
}

/* Get the PC the current function is supposed to return to */
KAFFE_INLINE
void *getFrameReturnPC(void *frame)
{
	/* XXX The +1 might just be for i386 */
	return( (void *)((uintp)PCFRAME((exceptionFrame *)frame) + 1) );
}

/* Take the iLock from the given thread */
KAFFE_INLINE
iLock *takeThreadLock(threadData *td)
{
	iLock *retval;

	require(td != 0);
	require(td->freeLock != 0);
	
	retval = td->freeLock;
	retval->link.owner = currentTeam();
	td->freeLock = 0;

	ensure(retval->mux == 0);
	ensure(retval->cv == 0);
	
	return( retval );
}

/* Give an iLock to the given thread */
KAFFE_INLINE
void giveThreadLock(threadData *td, iLock *il)
{
	require(td != 0);
	require(td->freeLock == 0);
	require(il != 0);
	require(il->mux == 0);
	require(il->cv == 0);

	td->freeLock = il;
	il->link.owner = 0;
}

#endif
