/*
 * feedbackSections.h
 * Definitions for feedback file sections
 *
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

#ifndef __feedback_sections_h
#define __feedback_sections_h

#include "sectionFile.h"

/*
 * A jitSectionData structure is used to store information about jitted
 * functions so that we can precompile them on the next run of the jvm.
 *
 * `flags' - Bitmask used to store some flags.
 * `size' - The size of the jitted code, not currently used.
 * `address' - The address where the code was placed, not current used.
 */

/* jitSectionData Flags */
enum {
	JSDB_PRECOMPILE,	/* Precompile the method */
};
enum {
	JSDF_PRECOMPILE = (1L << JSDB_PRECOMPILE),
};

typedef struct _jitSectionData {
	unsigned long flags;
	unsigned long size;
	char *address;
} jitSectionData;

/* The sectionType for the jit feedback section type */
extern sectionType jitSectionType;

/*
 * A libSectionData structure is used to store information about any shared
 * libraries that were loaded in previous runs.
 *
 * `flags' - Bitmask used to store some flags.
 */

/* libSectionData Flags */
enum {
	LSDB_PRELOAD,	/* Preload the library */
};
enum {
	LSDF_PRELOAD = (1L << LSDB_PRELOAD),
};

typedef struct _libSectionData {
	unsigned long flags;
} libSectionData;

/* The sectionType for the lib feedback section type */
extern sectionType libSectionType;

/*
 * AddModule: feedbackSectionsModule
 * Conditional: defined(KAFFE_FEEDBACK)
 */

#endif /* __feedback_sections_h */
