/*
 * journal.c
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <config.h>
#include <assert_pp.h>

#include "gtypes.h"
#include "support.h"
#include "gc.h"
#include "journal.h"
#include "journal-inline.h"

void initJournal(journal *j)
{
	int lpc;

	for( lpc = 0; lpc < JOURNAL_ENTRY_COUNT; lpc++ )
	{
		j->entries[lpc] = 0;
	}
}

void remJournalEntry(journal *j, void *id)
{
	int hash = hashJournalEntryID(id);
	journalEntry *curr, **prev;

	curr = j->entries[hash];
	prev = &j->entries[hash];
	while( curr && (curr->id != id) )
	{
		prev = &curr->next;
		curr = curr->next;
	}
	if( curr )
	{
		*prev = curr->next;
	}
}

journalEntry *findJournalEntry(journal *j, void *id)
{
	int hash = hashJournalEntryID(id);
	journalEntry *retval;

	retval = j->entries[hash];
	while( retval && (retval->id != id) )
	{
		retval = retval->next;
	}
	return( retval );
}
