/*
 * objectStack.h
 * Infrastructure for building potentially large objects.
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _object_stack_h
#define _object_stack_h

#include <janosvm/listNode.h>

typedef struct _objectSlice {
	struct lnMinNode link;
	char *buffer;
	int size;
	int max;
} objectSlice;

typedef struct _objectStack {
	struct lnMinList slices;
	struct {
		objectSlice *begin;
		objectSlice *end;
		int offset;
		int total;
	} current;
	int puddleSize;
} objectStack;

enum {
	OSA_TAG_DONE,
	OSA_Char,
	OSA_Bytes,
	OSA_String,
	OSA_IntegerString,
	OSA_PointerString,
};

objectStack *createObjectStack(int puddleSize);
void deleteObjectStack(objectStack *os);
void *objectStackFrame(objectStack *os);
void cutObjectStack(objectStack *os, void *frame);
int pushStackObject(objectStack *os, void *obj, int length);
int pushStackObjects(objectStack *os, int tag, ...);
void *topStackObject(objectStack *os, int *length);
void popStackObject(objectStack *os, void *obj, int length);

#endif
