/*
 * sectionFile-inline.h
 * Inline functions for the sectionFile/sectionType structures
 *
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

/* This only contains inline functions for calling sectionType functions */

KAFFE_INLINE
int sfInstall(sectionType *st, int tag, va_list *args)
{
	return( st->stf->install(st, tag, args) );
}

KAFFE_INLINE
int sfSuperInstall(sectionType *st, int tag, va_list *args)
{
	return( st->parent->stf->install(st->parent, tag, args) );
}

KAFFE_INLINE
void sfUninstall(sectionType *st)
{
	st->stf->uninstall(st);
}

KAFFE_INLINE
void sfSuperUninstall(sectionType *st)
{
	st->parent->stf->uninstall(st->parent);
}

KAFFE_INLINE
int sfInit(struct _sectionFileData *sfd)
{
	return( sfd->type->stf->init(sfd->type, sfd) );
}

KAFFE_INLINE
int sfSuperInit(struct _sectionType *st, struct _sectionFileData *sfd)
{
	return( st->parent->stf->init(st->parent, sfd) );
}

KAFFE_INLINE
void sfKill(struct _sectionFileData *sfd)
{
	sfd->type->stf->kill(sfd->type, sfd);
}

KAFFE_INLINE
void sfSuperKill(struct _sectionType *st, struct _sectionFileData *sfd)
{
	st->parent->stf->kill(st->parent, sfd);
}

KAFFE_INLINE
int sfCache(struct _sectionFileData *sfd, void *ps, char *line, int len)
{
	return( sfd->type->stf->
		cache(sfd->type, sfd, ps, line, len) );
}

KAFFE_INLINE
int sfSuperCache(struct _sectionType *st,
		 struct _sectionFileData *sfd, void *ps, char *line, int len)
{
	return( st->parent->stf->cache(st->parent, sfd, ps, line, len) );
}

KAFFE_INLINE
int sfFlush(struct _sectionFileData *sfd,
	    void *ps, char *line, int len, FILE *out_file)
{
	return( sfd->type->stf->
		flush(sfd->type, sfd, ps, line, len, out_file) );
}

KAFFE_INLINE
int sfSuperFlush(struct _sectionType *st, struct _sectionFileData *sfd,
		 void *ps, char *line, int len, FILE *out_file)
{
	return( st->parent->stf->
		flush(st->parent, sfd, ps, line, len, out_file) );
}

KAFFE_INLINE
int sfWalk(struct _sectionFileData *sfd, struct _sectionFile *sf)
{
	if( sfd->type->stf->walk )
		return( sfd->type->stf->walk(sfd->type, sfd, sf) );
	else
		return( 1 );
}

KAFFE_INLINE
int sfSuperWalk(struct _sectionType *st,
		struct _sectionFileData *sfd, struct _sectionFile *sf)
{
	if( st->parent->stf->walk )
		return( st->parent->stf->walk(st->parent, sfd, sf) );
	else
		return( 1 );
}
