/*
 * java.lang.Object.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "../../../kaffe/kaffevm/classMethod.h"
#include "../../../kaffe/kaffevm/gtypes.h"
#include "../../../kaffe/kaffevm/locks.h"
#include "../../../kaffe/kaffevm/object.h"
#include "../../../kaffe/kaffevm/constants.h"
#include "../../../kaffe/kaffevm/access.h"
#include "../../../kaffe/kaffevm/soft.h"
#include "../../../kaffe/kaffevm/baseClasses.h"
#include "../../../kaffe/kaffevm/support.h"
#include "../../../kaffe/janosvm/nameSpace.h"
#include "java_lang_Object.h"

extern Hjava_lang_Class *CrossTeamCloneableClass;

/*
 * Generate object hash code.
 */
jint
java_lang_Object_hashCode(struct Hjava_lang_Object* o)
{
	/* Hash code is object's address */
	return ((jint)(jword)o);
}

/*
 * Return class object for this object.
 */
struct Hjava_lang_Class*
java_lang_Object_getClass(struct Hjava_lang_Object* o)
{
	return OBJECT_CLASS(o);
}

/*
 * Notify threads waiting here.
 */
void
java_lang_Object_notifyAll(struct Hjava_lang_Object* o)
{
	broadcastCond(o);
}

/*
 * Notify a thread waiting here.
 */
void
java_lang_Object_notify(struct Hjava_lang_Object* o)
{
	signalCond(o);
}

/*
 * Clone me.
 */
struct Hjava_lang_Object*
java_lang_Object_clone(struct Hjava_lang_Object* o)
{
	Hjava_lang_Object* obj;
	Hjava_lang_Class* class;
	errorInfo einfo;
	int ctc = 0;
	int i;

	class = OBJECT_CLASS(o);
	if( CrossTeamCloneableClass && o )
		ctc = soft_instanceof(CrossTeamCloneableClass, o);
	if (!CLASS_IS_ARRAY(class)) {
		int *layout = class->gc_layout;
		int8* mem = (void *) o;
		int l;
		int nbits = CLASS_FSIZE(class)/ALIGNMENTOF_VOIDP;
		
		/* Check class is cloneable and throw exception if it isn't */
		if (soft_instanceof(CloneClass, o) == 0) {
			SignalError("java.lang.CloneNotSupportedException", getClassView(class)->name->data);
		}
		/* Clone an object */
		obj = newObject(class);
		/* bar writes here :) */
		while (nbits > 0) {
			/* get next integer from bitmap */
			l = *layout++;
			i = 0;
			while (i < BITMAP_BPI) {
				/* skip the rest if no refs left */
				if (l == 0) {
					mem += (BITMAP_BPI - i) * ALIGNMENTOF_VOIDP;
					break;
				}

				if (l < 0) {
                                /* we know this pointer points to gc'ed memory
                                 * there is no need to check - go ahead and
                                 * mark it.  Note that p may or may not point
                                 * to a "real" Java object.
                                 */
					if( GC_CHECK(obj,
						     *(void **)mem,
						     &einfo) )
					{
						if( ctc )
							(*(void **)mem) = NULL;
						else
							throwError(&einfo);
					}
				}
				i++;
				l <<= 1;
				mem += ALIGNMENTOF_VOIDP;
			}
			nbits -= BITMAP_BPI;
		}
		memcpy(OBJECT_DATA(obj), OBJECT_DATA(o),
		       CLASS_FSIZE(class) -
		       sizeof(Hjava_lang_Object));
		
	}
	else {
		/* Clone an array */
		obj = newArray(CLASS_ELEMENT_TYPE(class),
			       ARRAY_SIZE(o));
		
		/* here too!!! */
		if (!CLASS_IS_PRIMITIVE(CLASS_ELEMENT_TYPE(class))
		    && CLASS_ELEMENT_TYPE(class) != PtrClass) {
			HArrayOfObject *arr = (void *) o;

			for (i = 0; i < ARRAY_SIZE(arr); i++)
			{
				if( GC_CHECK(obj,
					     unhand_array(arr)->body[i],
					     &einfo) )
				{
					if( soft_instanceof(CrossTeamCloneableClass,
							    o) )
					{
						unhand_array(arr)->body[i] = NULL;
					}
					else
					{
						throwError(&einfo);
					}
				}
			}
		}
		memcpy(ARRAY_DATA(obj), ARRAY_DATA(o),
		       ARRAY_SIZE(o) * TYPE_SIZE(CLASS_ELEMENT_TYPE(class)));
	}
	return (obj);
}

/*
 * Wait for this object to be notified.
 */
void
java_lang_Object_wait0(struct Hjava_lang_Object* o, jlong timeout)
{
	waitCond(o, timeout);
}
