/*
 * ClassPathResource.java
 * Emulates the services provided by the real class.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

import java.util.Hashtable;
import java.util.Vector;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import edu.utah.janosvm.sys.DeadTeamException;

public final class ClassPathResource
{
    private static final Vector pathcomps = new Vector(8);
	
    /**
     * Initialize the pathcomps vector to the elements of
     * the global system classpath.
     */
    static { // XXX this is bogus, should be configured somewhere
	String cp = java.lang.System.getProperty("java.class.path");
		
	int i;
	while ((i = cp.indexOf(File.pathSeparatorChar)) != -1) {
	    if (i == 0)
				// XXX right thing to do?
		pathcomps.addElement(".");
	    else
		pathcomps.addElement(cp.substring(0, i));
	    cp = cp.substring(i+1);
	}
	pathcomps.addElement(cp);
    }

    public static byte[] loadFile(String cname)
	throws DeadTeamException,
	       IOException
    {
	// iterate through all components and look for .class file
	for (int i = 0; i < pathcomps.size(); i++)
	{
	    BufferedInputStream file = null;
	    int size = 0;
	    boolean foundit = false;
	    String compname = (String)pathcomps.elementAt(i);

	    foundit = false;

	    try
	    {
		// a zip file ?
		ZipFile  zip   = new ZipFile(compname);
		ZipEntry entry = zip.getEntry(cname);
				
		if (entry != null)
		{
		    file = new BufferedInputStream(zip.getInputStream(entry));
		    size = (int)entry.getSize();
		    foundit = true;
		}
	    }
	    catch (IOException ioe)
	    {
	    }
		
	    if (!foundit)
	    {
		try
		{
		    String fn  = compname.concat("/").concat(cname);
		    File f = new File(fn);
		    file = new BufferedInputStream(new FileInputStream(f));
		    size = (int)f.length(); 
		    foundit = true;
		}
		catch (FileNotFoundException fnfe)
		{
		}
	    }

	    // Try the next classpath entry
	    if (!foundit)
		continue;
			
	    // Try to read the file
	    byte[] result = new byte[size];
	    int off = 0;
			
	    while (off < result.length)
	    {
		int rc;
		
		rc = file.read(result, off, result.length - off);
		off += rc;
	    }
	    return result;
	}
	return null;
    }

    public synchronized static void addPath(String path)
	throws IOException, DeadTeamException
    {
	pathcomps.addElement(path);
    }
    
    public synchronized static void hidePath(String path)
	throws IOException, DeadTeamException
    {
	int lpc, size;

	size = pathcomps.size();
	for( lpc = 0; lpc < size; lpc++ )
	{
	    if( ((String)pathcomps.elementAt(lpc)).equals(path) )
	    {
		pathcomps.removeElementAt(lpc);
		size--;
	    }
	}
    }
    
    public synchronized static void removePath(String path)
	throws IOException, DeadTeamException
    {
	hidePath(path);
    }
}
