/*
 * GCPolicy.java
 * Base class for objects that implement a specific policy for the GC.
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

public class GCPolicy
{
    protected final long max;
    private static final GCPolicy currentPolicy = new GCPolicy();
    
    private GCPolicy()
    {
	this.max = 0;
    }
    
    public GCPolicy(long max)
    {
	this.max = max;
    }
    
    public boolean startThreads()
    {
	return true;
    }
    
    public boolean enable()
    {
	return this.startThreads();
    }
    
    public long initialTrigger()
    {
	return 250 * 1024;
    }
    
    public long newTrigger(long used, long freed)
    {
	return used + used / 3;
    }

    public static GCPolicy currentPolicy()
    {
        return currentPolicy;
    }
}
