/*
 * StdioSpec.java
 * Resource specification for standard I/O.
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Resource specification for standard I/O.
 */
public final class StdioSpec
    extends ResourceSpec
{

    /**
     * Constructs a standard I/O resource specification with the given
     * stream identifiers.  The stream identifier is a URI like string
     * that describes the type of stream and any additional options.
     * The format is `<protocol>://<path>' where protocol is one of:
     *   builtin - The VM's stdio streams.  Possible paths are `null' which
     *     specifies no stream, `stdin' for standard in, `stdout' for standard
     *     out, and `stderr' for standard error.
     *   jsi - A connected jsiChannel, currently, just the one that executed
     *     run of team x.
     *   file - A regular file where path should be the path of the file to
     *     create.
     *   fd - A file descriptor in the VM, the fd given in the path is dup'd
     *     and used to build a FILE object.
     *
     * @param inURI URI for the standard input.
     * @param outURI URI for the standard output.
     * @param errURI URI for the standard error.
     */
    public StdioSpec(String inURI, String outURI, String errURI)
    {
    }

    /**
     * @return The StdioSpec object for the current team.
     */
    public static StdioSpec current()
    {
	return new StdioSpec("", "", "");
    }

    /**
     * @return The StdioSpec object for the current team that has file
     * descriptor URI's.  This is useful for sharing input/output streams
     * with child teams.
     */
    public static StdioSpec currentFD()
    {
	return new StdioSpec("", "", "");
    }

    /**
     * @return The standard input URI.
     */
    public String getInput()
    {
	return "";
    }

    /**
     * @return The standard output URI.
     */
    public String getOutput()
    {
	return "";
    }

    /**
     * @return The standard error URI.
     */
    public String getError()
    {
	return "";
    }

}
