/*
 * TeamBackEnd.java
 * Emulates the services provided by the real class.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

import java.util.Hashtable;

import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.DuplicateResourceSpecException;

class TeamBackEnd
    extends Exportable
    implements PersistentExport
{
    private static TeamReservation reservation = new TeamReservation();
    
    private static TeamBackEnd kernelTeam = new TeamBackEnd();
    private static ExportManager kernelManager;

    static TeamListBackEnd teamList;

    public static final String kernelName = "kernel";
    
    static TeamBackEnd getKernelTeam()
    {
	return TeamBackEnd.kernelTeam;
    }

    static TeamBackEnd getCurrentTeam()
    {
	return TeamBackEnd.kernelTeam;
    }

    private static int nextID = 2;

    private int id;

    int getID()
    {
	return this.id;
    }
    
    private Team peer;

    Team getPeer()
    {
	return this.peer;
    }
    
    private String name;

    String getName()
    {
	return this.name;
    }

    void setDescription(String iDontCare)
    {
    }
    
    private Hashtable classLocals = new Hashtable();

    Object getClassLocal(Class cl)
    {
	return this.classLocals.get(cl);
    }
    
    static Object getCurrentClassLocal(Class cl)
    {
	return getCurrentTeam().classLocals.get(cl);
    }
    
    void setClassLocal(Class cl, Object obj)
    {
	this.classLocals.put(cl, obj);
    }

    static void setCurrentClassLocal(Class cl, Object obj)
    {
	getCurrentTeam().classLocals.put(cl, obj);
    }
    
    private TeamBackEnd()
    {
	super(0);
	try
	{
	    ExportManager em;
	    PortAuthority pa;

	    this.name = kernelName;
	    this.id = 1;
	    this.peer = new Team();

	    pa = this.peer.getPortAuthority();
	    em = pa.exportObject(kernelName, this);
	    em.addImporter(this.peer);

	    TeamBackEnd.kernelManager = em;
	    this.peer.setKernelHandle(em);

	    teamList = new TeamListBackEnd();
	    em = this.peer.exportObject(TeamList.exportID, teamList);
	    teamList.setExportManager(em);
	}
	catch(DuplicateExportException e)
	{
	}
    }
    
    TeamBackEnd(String name, ResourceSpec specs[], kaffe.util.Ptr nativeSpecs)
	    throws DuplicateResourceSpecException, IndexedResourceSpecException
    {
	this.name = name;
	this.peer = new Team(TeamBackEnd.kernelManager);
	this.id = nextID++;
    }

    boolean owns(Object obj)
    {
	return true;
    }

    static TeamBackEnd getObjectOwner(Object obj)
    {
	return TeamBackEnd.kernelTeam;
    }

    void terminate()
    {
    }

    boolean switchTo() throws TeamEjectionException
    {
	return true;
    }

    boolean switchTo(int w) throws TeamEjectionException
    {
	return true;
    }

    static void returnFrom()
    {
    }

    TeamReservation pushReservation()
	throws DeadTeamException
    {
	return TeamBackEnd.reservation;
    }

    TeamReservation pushReservation(int a)
	throws DeadTeamException
    {
	return TeamBackEnd.reservation;
    }

    static void popReservation()
    {
    }

    public boolean addDependency(TeamBackEnd be)
    {
	return true;
    }

    public void notifyOnFinish()
    {
    }
}
