/*
 * domainResource.h
 * Flow resource for teams, wraps an_domain object from Moab.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _domain_resource_h
#define _domain_resource_h

/*
 * This module exports a domain resource.  The domainResource
 * is an "external" resource.
 */

#include "assert_pp.h"

#include "janosvm/resource.h"
#include "thread.h"
#include <nodeos/an_domain.h>


/*
 * The domainResource is the global object which all resource clients
 * point to.  There isn't anything here other than the standard
 * resource information (in 'link').
 */
typedef struct _domainResource {
	resource link;
} domainResource;

/*
 * The domainResourceClient is the per-team structure that tracks
 * the specific domain used by that team, plus all the standard client
 * resource hooks.
 */
typedef struct _domainResourceClient {
	resourceClient client;
	an_domain_t *domain;
} domainResourceClient;

/*
 * There is nothing to a domain resource spec.  
 *
 * Eventually this could include all of the parameters to
 * an_domain_create(), and we could do all the domain create/destroy
 * through the resource API.
 */
typedef struct _domainResourceSpec {
	resourceSpec spec;
	/* XXX nothing */
} domainResourceSpec;

extern int domainResourceFixupCurrent(an_domain_t *domain);

/*
 * AddModule: domainResourceModule
 *   Comment: Install the domain
 */

#endif
