/*                                                                 -*-C-*-
 * jsiImpl-inline.h
 * Low level functions required by the JanosVM Scripting Interface layer
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _kaffe_jsi_impl_h
#define _kaffe_jsi_impl_h

/* This is a jsiImpl that just passes off to Kaffe syscalls */

#include <string.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <jmalloc.h>
#include <jsyscall.h>

#include "kaffevm/locks.h"

JSI_INLINE
void *jsiMalloc(int size)
{
	return( KMALLOC(size) );
}

JSI_INLINE
void *jsiRealloc(void *old, int size)
{
	return( KREALLOC(old, size) );
}

JSI_INLINE
void jsiFree(void *mem)
{
	KFREE(mem);
}

JSI_INLINE
void _jsiLock(JSI_LOCK_PARAM lock, int *iLockRoot)
{
	_lockMutex(lock, iLockRoot);
}

JSI_INLINE
void _jsiUnlock(JSI_LOCK_PARAM lock, int *iLockRoot)
{
	_unlockMutex(lock, iLockRoot);
}

JSI_INLINE
int _jsiWait(JSI_LOCK_PARAM lock, int ms_timeout)
{
	return( _waitCond(lock, ms_timeout) );
}

JSI_INLINE
void _jsiSignal(JSI_LOCK_PARAM lock)
{
	_signalCond(lock);
}

JSI_INLINE
int jsiOpen(char *filename, int flags, int mode, int *out_fd)
{
	return( KOPEN(filename, flags, mode, out_fd) );
}

JSI_INLINE
int jsiSocket(int domain, int type, int protocol, int *out_fd)
{
	return( KSOCKET(domain, type, protocol, out_fd) );
}

JSI_INLINE
int jsiConnect(int sockfd, struct sockaddr *serv, int addrlen)
{
	return( KCONNECT(sockfd, serv, addrlen, 0) );
}

JSI_INLINE
int jsiBind(int sockfd, struct sockaddr *my, int addrlen)
{
	return( KBIND(sockfd, my, addrlen) );
}

JSI_INLINE
int jsiListen(int sockfd, int backlog)
{
	return( KLISTEN(sockfd, backlog) );
}

JSI_INLINE
int jsiAccept(int sockfd, struct sockaddr *addr, int *addrlen, int *out_fd)
{
	return( KACCEPT(sockfd, addr, addrlen, 0, out_fd) );
}

JSI_INLINE
int jsiRecvFrom(int fd, void *buffer, int len, unsigned int flags,
		struct sockaddr *from, int *fromlen, int *out_read)
{
	return( KRECVFROM(fd, buffer, len, flags, from,
			  fromlen, 0, out_read) );
}

JSI_INLINE
int jsiSend(int fd, void *buffer, int len, unsigned int flags)
{
	int dummy;
	
	return( KSENDTO(fd, buffer, len, flags, 0, 0, &dummy) );
}

JSI_INLINE
int jsiSendTo(int fd, void *buffer, int len, unsigned int flags,
	      struct sockaddr *to, int tolen)
{
	int dummy;
	
	return( KSENDTO(fd, buffer, len, flags, to, tolen, &dummy) );
}

JSI_INLINE
int jsiSeek(int fd, off_t offset, int whence, off_t *out_offset)
{
	return( KLSEEK(fd, offset, whence, out_offset) );
}

JSI_INLINE
int jsiRead(int fd, void *buffer, size_t bufferSize, size_t *out_read)
{
	return( KREAD(fd, buffer, bufferSize, out_read) );
}

JSI_INLINE
int jsiReadAndWait(int fd, void *buffer, size_t bufferSize, int sample_time,
		   size_t *out_read)
{
	return( KREADANDWAIT(fd, buffer, bufferSize, sample_time, out_read) );
}

JSI_INLINE
int jsiWrite(int fd, void *buffer, size_t bufferSize, size_t *out_written)
{
	return( KWRITE(fd, buffer, bufferSize, out_written) );
}

JSI_INLINE
int jsiShutdown(int fd, int how)
{
	return( 1 );
}

JSI_INLINE
int jsiClose(int fd)
{
	return( KCLOSE(fd) );
}

JSI_INLINE
int jsiGetHostByName(char *name, struct hostent **he)
{
	return( KGETHOSTBYNAME(name, he) );
}

JSI_INLINE
int jsiGetHostByAddr(char *addr, int len, int type, struct hostent **he)
{
	return( KGETHOSTBYADDR(addr, len, type, he) );
}

JSI_INLINE
int jsiSetSockOpt(int sock, int level, int optname, void *optval, int optlen)
{
	return( KSETSOCKOPT(sock, level, optname, optval, optlen) );
}

#endif
