/*
 * moab_BufferHandle.c
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>

#include <nodeos/impl/an_impl.h>

#include "moabglue.h"
#include "edu_utah_janos_nodeos_moab_BufferHandle.h"

#define MAX_PACKET_LEN NODEOS_CHAN_BUFSIZE

jint edu_utah_janos_nodeos_moab_BufferHandle_getMaxPacketLen(void)
{
	return MAX_PACKET_LEN;
}

void edu_utah_janos_nodeos_moab_BufferHandle_init(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						  struct Hedu_utah_janos_nodeos_moab_BufferHandle* other,
						  jint offset,
						  jint length)
{
	an_pbuf_t* rc;
	an_error_t err;

	assert(this);
	assert(offset >= 0);
	assert(length >= 0);
	assert(offset <= MAX_PACKET_LEN);
	assert(length <= MAX_PACKET_LEN);
	assert(offset+length <= MAX_PACKET_LEN);

	if (other == NULL) 
	{
		/* Create a new buffer underneath */

		int pbuf_flags;
		pbuf_flags = AN_PBUF_ETH; // XXX 
		
		rc = an_pbuf_create(NATIVE_PBUF(this),
				    pbuf_flags,
				    offset, length);
		assert(rc == NATIVE_PBUF(this));
		assert(an_pbuf_offset(NATIVE_PBUF(this)) == offset);
	}
	else
	{
		int srcOffset;
		srcOffset = an_pbuf_offset(NATIVE_PBUF(other));

		/* Clone from other */
		rc = an_pbuf_clone(NATIVE_PBUF(other), NATIVE_PBUF(this));
		assert(rc == NATIVE_PBUF(this));

		/* Fixup offset/length for the new buffer. */
		err = an_pbuf_setsize(NATIVE_PBUF(this), length, NULL);
		assert(err == 0);
		
		err = an_pbuf_setoffset(NATIVE_PBUF(this), srcOffset + offset, NULL);
		assert(err == 0);
	}

	assert(an_pbuf_size(NATIVE_PBUF(this)) == length);
	assert(an_pbuf_bufsize(NATIVE_PBUF(this)) >= an_pbuf_size(NATIVE_PBUF(this)));
}

jint edu_utah_janos_nodeos_moab_BufferHandle_getLength(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this)
{
	return an_pbuf_size(NATIVE_PBUF(this));
}

void edu_utah_janos_nodeos_moab_BufferHandle_setLength(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this, jint size)
{
	an_pbuf_setsize(NATIVE_PBUF(this), size, 0);
	/* XXX Check return value */
}

jint edu_utah_janos_nodeos_moab_BufferHandle_getOffset(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this)
{
	return an_pbuf_offset(NATIVE_PBUF(this));
}

#if 0
void edu_utah_janos_nodeos_moab_BufferHandle_setLength(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						       jint length)
{
	int rc;
	assert(length <= an_pbuf_bufsize(NATIVE_PBUF(this)));
	rc = an_pbuf_setsize(NATIVE_PBUF(this), length, NULL);
	assert(rc == 0);
}
#endif

void edu_utah_janos_nodeos_moab_BufferHandle_reset(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this)
{
	int pbuf_flags = AN_PBUF_SOCK; // XXX
	an_pbuf_t *rc;
	
	an_pbuf_destroy(NATIVE_PBUF(this));
	rc = an_pbuf_create(NATIVE_PBUF(this), pbuf_flags, 0, 0);
	assert(rc == NATIVE_PBUF(this));
}

void edu_utah_janos_nodeos_moab_BufferHandle_destroy(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this)
{
	an_pbuf_destroy(NATIVE_PBUF(this));
}

void edu_utah_janos_nodeos_moab_BufferHandle_newBuffer(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						       jint offset,
						       jint length)
{
	int pbuf_flags = AN_PBUF_SOCK; // XXX
	an_pbuf_t* rc;

	assert(offset >= 0);
	assert(length >= 0);

	// XXX assert pbuf is destroyed
	rc = an_pbuf_create(NATIVE_PBUF(this), pbuf_flags, offset, length);
	assert(rc == NATIVE_PBUF(this));
}

void edu_utah_janos_nodeos_moab_BufferHandle_cloneHandle(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
							 struct Hedu_utah_janos_nodeos_moab_BufferHandle* other,
							 jint fromOffset,
							 jint fromLength)
{
	int srcOffset;
	an_error_t err;
	an_pbuf_t* rc;

	assert(this);
	assert(other);

	srcOffset = an_pbuf_offset(NATIVE_PBUF(other));

	/* Java code must do all the right range checks. */
	assert(fromOffset >= 0);
	assert(fromLength >= 0);
	assert(fromLength <= an_pbuf_size(NATIVE_PBUF(this)));
	assert(fromOffset <= an_pbuf_size(NATIVE_PBUF(other)));
	assert(fromLength <= an_pbuf_size(NATIVE_PBUF(other)));
	assert(fromOffset <= an_pbuf_size(NATIVE_PBUF(other)));
	assert(fromOffset + fromLength <= an_pbuf_size(NATIVE_PBUF(other)));

	/* Must clear out this (the destination pbuf) */
	an_pbuf_destroy(NATIVE_PBUF(this));

	/*
	 * Underlying an_pbuf_clone() just copies the offset and
	 * length parameters, so those have to be updated manually.
	 */
	rc = an_pbuf_clone(NATIVE_PBUF(other), NATIVE_PBUF(this));
	assert(rc == NATIVE_PBUF(this)); /* XXX */

	err = an_pbuf_setoffset(NATIVE_PBUF(this), srcOffset+fromOffset, NULL);
	assert(err == 0);
	err = an_pbuf_setsize(NATIVE_PBUF(this), fromLength, NULL);
	assert(err == 0);
}

void edu_utah_janos_nodeos_moab_BufferHandle_swapWith(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						      struct Hedu_utah_janos_nodeos_moab_BufferHandle* other,
						      jint offset,
						      jint length)
{
	assert(this);
	assert(other);
	assert(offset >= 0);
	assert(length >= 0);

	NOTIMPLEMENTED("MoabGlueFunction in moab_BufferHandle.c");
}

jbyte edu_utah_janos_nodeos_moab_BufferHandle_getByte(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						      jint offset)
{
	jbyte *data;

	assert(this);
	assert(offset >= 0);
	assert(offset < an_pbuf_size(NATIVE_PBUF(this)));

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	return data[offset];
}

void edu_utah_janos_nodeos_moab_BufferHandle_setByte(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						     jint offset,
						     jbyte val)
{
	jbyte *data;

	assert(this);
	assert(offset >= 0);
	assert(offset < an_pbuf_size(NATIVE_PBUF(this)));

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	data[offset] = val;
}

jshort edu_utah_janos_nodeos_moab_BufferHandle_getShort(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						    jint offset)
{
	jbyte *data;

	assert(this);
	assert(offset >= 0);
	assert(offset < an_pbuf_size(NATIVE_PBUF(this))-1);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	return ntohs(*((short*)(data+offset)));
}

void edu_utah_janos_nodeos_moab_BufferHandle_setShort(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						    jint offset,
						    jshort val)
{
	jbyte *data;

	assert(this);
	assert(offset >= 0);
	assert(offset < an_pbuf_size(NATIVE_PBUF(this))-1);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	*(short*)(data+offset) = htons(val);
}

jint edu_utah_janos_nodeos_moab_BufferHandle_getInt(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						    jint offset)
{
	jbyte *data;

	assert(this);
	assert(offset >= 0);
	assert(offset < an_pbuf_size(NATIVE_PBUF(this))-3);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	return ntohl(*((int*)(data+offset)));
}

void edu_utah_janos_nodeos_moab_BufferHandle_setInt(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						    jint offset,
						    jint val)
{
	jbyte *data;

	assert(this);
	assert(offset >= 0);
	assert(offset < an_pbuf_size(NATIVE_PBUF(this))-3);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	*(int*)(data+offset) = htonl(val);
}

void edu_utah_janos_nodeos_moab_BufferHandle_getBytes(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						      jint offset,
						      HArrayOfByte* dest,
						      jint destPosition,
						      jint length)
{
	jbyte *data;

	assert(this);
	assert(dest);
	assert(offset >= 0);
	assert(offset+length <= an_pbuf_size(NATIVE_PBUF(this)));
	assert(obj_length(dest) >= destPosition+length);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	memcpy(unhand_byte_array(dest) + destPosition,
	       data + offset,
	       length);
}

void edu_utah_janos_nodeos_moab_BufferHandle_setBytes(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						      jint offset,
						      HArrayOfByte* src,
						      jint srcPosition,
						      jint length)
{
	jbyte *data;

	assert(this);
	assert(src);
	assert(offset >= 0);
	assert(offset+length <= an_pbuf_size(NATIVE_PBUF(this)));
	assert(obj_length(src) >= srcPosition+length);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);

	memcpy(data + offset,
	       unhand_byte_array(src) + srcPosition,
	       length);
}

void edu_utah_janos_nodeos_moab_BufferHandle_getInts(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						     jint offset,
						     HArrayOfInt* dest,
						     jint destPosition,
						     jint length)
{
	int i;
	void *data;		/* raw nodeos buffer pointer */
	jint *idata;		/* nodeos buffer pointer, as jint */
	jint *jdata;		/* java array buffer pointer */

	assert(this);
	assert(dest);
	assert(offset >= 0);
	assert(offset+length*4 <= an_pbuf_size(NATIVE_PBUF(this)));
	assert(obj_length(dest) >= destPosition+length);
	
	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);
	data += offset;		/* offset is a byte offset */
	/* XXX fails on machines that can't handle unaligned accesses. */
	idata = (jint*)data;	/* cast result to a jint pointer */

	jdata = (jint*)(&(unhand_byte_array(dest)[destPosition]));

	for(i = 0; i < length; i++)
	{
		/* do byte-swapping on each int. */
		*jdata = ntohl(*idata);
		jdata++;
		idata++;
	}
}

void edu_utah_janos_nodeos_moab_BufferHandle_setInts(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
						     jint offset,
						     HArrayOfInt* src,
						     jint srcPosition,
						     jint length)
{
	int i;
	void *data;
	jint *idata;
	jint *jdata;

	assert(this);
	assert(src);
	assert(offset >= 0);
	assert(offset+(length*4) <= an_pbuf_size(NATIVE_PBUF(this)));
	assert(obj_length(src) >= srcPosition+length);

	data = an_pbuf_data(NATIVE_PBUF(this));
	assert(data);
	data += offset;		/* offset is a byte offset */
	/* XXX fails on machines that can't handle unaligned accesses. */
	idata = (jint*)data;	/* cast result to a jint pointer */

	jdata = (jint*)(&(unhand_byte_array(src)[srcPosition]));

	for(i = 0; i < length; i++)
	{
		/* do byte-swapping on each int. */
		*idata = htonl(*jdata);
		jdata++;
		idata++;
	}
}


jshort edu_utah_janos_nodeos_moab_BufferHandle_computeChecksum(
	struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
	jint offset,
	jint count)
{
	an_pbuf_t *pb = NATIVE_PBUF(this);
	char *addr = an_pbuf_data(pb) + offset;
	jshort retval;
	long sum = 0;

	while( count > 1 )
	{
		/* This is the inner loop */
		sum += *(unsigned short *)addr;
		addr += 2;
		count -= 2;
	}
	if( count > 0 )
		sum += *(unsigned char *)addr;
	while( sum >> 16 )
		sum = (sum & 0xffff) + (sum >> 16);
	retval = ~sum;
	/*
	 * XXX We swap here so that it will get swapped back later on...
	 * Otherwise we get a bogus number.
	 */
	return htons(retval);
}

jboolean edu_utah_janos_nodeos_moab_BufferHandle_compareBytes(
	struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
	jint offset,
	HArrayOfByte* javaSeq,
	jint seqOffset,
	jint seqLen)
{
	an_pbuf_t *pb;
	char *seqData;
	char *pbData;
	int i;

	require(this);
	require(javaSeq);

	pb = NATIVE_PBUF(this);
	require(pb);
	require(offset > 0);
	require(seqLen > 0);
	require(offset + seqLen < an_pbuf_size(pb));

	pbData = an_pbuf_data(pb) + offset;
	seqData = &unhand_byte_array(javaSeq)[seqOffset];
	
	for (i = 0; i < seqLen; i++)
	{
		if (*pbData != *seqData)
			return 0; // xxx 'return false'?
		pbData++;
		seqData++;
	}
	
	return 1; // xxx 'return true'?
}


jint
edu_utah_janos_nodeos_moab_BufferHandle_dbgPacketSummary(struct Hedu_utah_janos_nodeos_moab_BufferHandle* this,
							 HArrayOfChar* sbuf,
							 jint offset,
							 jint maxlen)
{
#define HACK_BUF_SIZE 1024
	char msgbuf[HACK_BUF_SIZE];
	int i;
	char *msgptr;
	jbyte *data;
	int bufsize;

	assert(this);
	assert(NATIVE_PBUF(this));
	assert(sbuf);
	assert(offset >= 0);
	assert(maxlen > offset);
	// assert(offset+maxlen < an_pbuf_size(NATIVE_PBUF(this)));
	
	msgptr = msgbuf;
	assert(msgptr);

	data = an_pbuf_data(NATIVE_PBUF(this));
	bufsize = an_pbuf_size(NATIVE_PBUF(this));
	assert(data);

	/* XXX similar to pj_BufferHandle.dbgPacketSummary() */

	for (i = offset; i < bufsize; i++)
	{
		if (i > maxlen)
		{
			msgptr += snprintf(msgptr,
					   HACK_BUF_SIZE - (msgptr - msgbuf),
					   "...");
			break;
		}
		
		// two hex digits per byte
		msgptr += snprintf(msgptr,
				   HACK_BUF_SIZE - (msgptr - msgbuf),
				   "%2.2x ",
				   data[offset+i]);
	}

	bufsize = (msgptr - msgbuf);
	assert(bufsize < obj_length(sbuf));
	for (i = 0; i < bufsize; i++)
	{
		jchar ch = 0;
		ch |= (msgbuf[i]);
		unhand_byte_array(sbuf)[i] = ch;
	}

	return bufsize;
}
