/*
 * moab_DemultiplexKey.c
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>

#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/udp.h>

#include "moabglue.h"

#include "edu_utah_janos_nodeos_moab_DemultiplexKey.h"

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_init(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
					       struct Hedu_utah_janos_nodeos_moab_Domain* domain)
{
	void *rc;
	assert(this);
	assert(domain);

	rc = an_demuxkey_create(NATIVE_DEMUXKEY(this), NATIVE_DOMAIN(domain));
	assert(rc == NATIVE_DEMUXKEY(this));
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_add(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
					      jint length,
					      HArrayOfByte* sequence,
					      HArrayOfByte* mask)
{
	int rc;

	assert(this);
	assert(length >= 0);
	assert(sequence);

	assert(length <= obj_length(sequence));
	//assert(offset + length <= obj_length(sequence));

	rc = an_demuxkey_add(NATIVE_DEMUXKEY(this),
			     length,
			     unhand_array(sequence),
			     mask == NULL ? 0 : unhand_byte_array(mask));

	/* Caller must have done all the checks. */
	assert(rc == 0);
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addByte(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						  jbyte b)
{
	int rc;

	assert(this);

	rc = an_demuxkey_add(NATIVE_DEMUXKEY(this),
			     1,
			     &b, NULL);

	/* Caller must have done all the checks. */
	assert(rc == 0);
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addShort(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						   jshort js)
{
	int rc;
	jshort s = htons(js);

	assert(this);

	rc = an_demuxkey_add(NATIVE_DEMUXKEY(this),
			     2,
			     &s, NULL);

	/* Caller must have done all the checks. */
	assert(rc == 0);
}
void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addInt(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						 jint ji)
{
	int rc;
	jint i = htonl(ji);
	
	assert(this);

	rc = an_demuxkey_add(NATIVE_DEMUXKEY(this),
			     4,
			     &i, NULL);

	/* Caller must have done all the checks. */
	assert(rc == 0);
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addLong(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						  jlong jl)
{
	int rc;
	int low = (jl & (jlong)0x0FFFFFFFF);
	int high = ((jl >> 32) & (jlong)0x0FFFFFFFF);
	jlong l = (((jlong)htonl(low)) << 32) || ((jlong)htonl(high));

	assert(this);

	rc = an_demuxkey_add(NATIVE_DEMUXKEY(this),
			     8,
			     &l, NULL);

	/* Caller must have done all the checks. */
	assert(rc == 0);
}

void
edu_utah_janos_nodeos_moab_DemultiplexKey_addETH(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey *this,
						 jint flags,
						 HArrayOfByte *src,
						 HArrayOfByte *dst,
						 jshort type)
{
	an_error_t err;

	err = an_demuxkey_addeth(NATIVE_DEMUXKEY(this),
				 flags,
				 (u_int8_t *)unhand_array(src),
				 (u_int8_t *)unhand_array(dst),
				 htons(type));
	assert(err == 0);
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addIPv4(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						  jint flags,
						  jint src, jint srcMask,
						  jint dst, jint dstMask,
						  jbyte proto)
{
	an_error_t err;

	err = an_demuxkey_addipv4(NATIVE_DEMUXKEY(this),
				  flags,
				  htonl(src), htonl(srcMask),
				  htonl(dst), htonl(dstMask),
				  proto);
	assert(err == 0);
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addUDP(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						 jint srcPort,
						 jint destPort)
{
	an_error_t error;
	int flags = 0;

	/* XXX XXX */
	if (srcPort == 0)
		flags |= AN_DEMUXKEY_UDP_ANYSPORT;
	if (destPort == 0)
		flags |= AN_DEMUXKEY_UDP_ANYDPORT;

	error = an_demuxkey_addudp(NATIVE_DEMUXKEY(this),
				   flags,
				   srcPort, destPort);
	assert(error == 0);
}

void 
edu_utah_janos_nodeos_moab_DemultiplexKey_addTCP(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						 jint srcPort,
						 jint destPort)
{
	an_error_t error;
	int flags = 0;

	/* XXX XXX */
	if (srcPort == 0)
		flags |= AN_DEMUXKEY_TCP_ANYSPORT;
	if (destPort == 0)
		flags |= AN_DEMUXKEY_TCP_ANYDPORT;

	error = an_demuxkey_addtcp(NATIVE_DEMUXKEY(this),
				   flags, srcPort, destPort);
	assert(error == 0);
}

void
edu_utah_janos_nodeos_moab_DemultiplexKey_addANEP(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						  jbyte version,
						  jboolean allVersions,
						  jshort protocolID)
{
	an_error_t error;
	
	error = an_demuxkey_addanep(NATIVE_DEMUXKEY(this),
				    version, allVersions, htons(protocolID));
	assert(error == 0);
}

jint
edu_utah_janos_nodeos_moab_DemultiplexKey_matches(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						  HArrayOfByte* buf,
						  jint bufstart,
						  jint buflength)
{
	int offset;

	assert(this);
	assert(buf);
	assert(bufstart >= 0);
	assert(buflength >= 0);
	assert(bufstart <= buflength);
	assert(bufstart <= obj_length(buf));
	assert(buflength <= obj_length(buf));

	/*
	 * XXX buflength has odd semantics.  See the Javadoc comments.
	 */

	offset = an_demuxkey_matches(NATIVE_DEMUXKEY(this),
				     ((jbyte *)unhand_array(buf)) + bufstart,
				     buflength - bufstart);
	return offset;
}

jint
edu_utah_janos_nodeos_moab_DemultiplexKey_length(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this)
{
	an_error_t rc;
	jint retval;
	
	rc = an_demuxkey_len(NATIVE_DEMUXKEY(this), &retval);
	assert(rc == 0);
	return( retval );
}

/**
 * Dump string representation of given key into the given sbuf.  Return number
 * of characters inserted.
 */
jint
edu_utah_janos_nodeos_moab_DemultiplexKey_toString(struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* this,
						   HArrayOfChar *sbuf)
{
	an_error_t rc;
	char cbuf[512];

	require(this);
	require(NATIVE_DEMUXKEY(this));
	require(sbuf);
	require(obj_length(sbuf) > 0);
	require(obj_length(sbuf) <= sizeof(cbuf));
	
	rc = an_demuxkey_tostring(NATIVE_DEMUXKEY(this), cbuf, sizeof(cbuf));
	if (rc == 0)
	{
		int i;

		/* copy the cbuf into the sbuf, widening each char */
		for (i = 0; i < obj_length(sbuf)-1; i++)
		{
			jchar ch = 0;
			ch |= cbuf[i];

			unhand_array(sbuf)->body[i] = ch;

			if (cbuf[i] == '\0')
				break;
		}

		return i;
	}
	else 
	{
		return 0;
	}
	
	return rc;
}
