/*
 * moab_Domain.c
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>

#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/thread.h"

#include "moabglue.h"
#include "moabthread.h"
#include "domainResource.h"

static void
moab_domain_termfunc(an_domain_t* dyingdomain)
{
	extern JNIEnv Kaffe_JNIEnv;
	JNIEnv *env = &Kaffe_JNIEnv;
	struct Hedu_utah_janos_nodeos_moab_Domain* jdomain;
	jmethodID termHandlerMethod;
	jthrowable eobj;

	assert(dyingdomain);
	jdomain = KAFFE_DOMAIN(dyingdomain);
	
	termHandlerMethod = (*env)->GetMethodID(env, (*env)->GetObjectClass(env, jdomain),
						"native_termHandler", "()V");
	assert(termHandlerMethod);
	(*env)->CallVoidMethodV(env, jdomain, termHandlerMethod, NULL);
	eobj = (*env)->ExceptionOccurred(env);
	(*env)->ExceptionClear(env);	
	assert(eobj == NULL); /* XXX Just ignore exceptions in here?? */
}

void edu_utah_janos_nodeos_moab_Domain_init(struct Hedu_utah_janos_nodeos_moab_Domain* this,
					  struct Hedu_utah_janos_nodeos_moab_Credentials* cred,
					  struct Hedu_utah_janos_nodeos_moab_ThreadPool* tps,
					  struct Hedu_utah_janos_nodeos_moab_MemSpec* ms)
{
	an_domain_t* rc;

	/* XXX note that the ThreadPool passed in contains a native threadpool*spec* */
	/* XXX that's cause mike hasn't implemented the Moab threadpool. waaaaa. */

	rc = an_domain_create(NATIVE_DOMAIN(this),
			    //NATIVE_CRED(cred),
			    an_cred_current(),
			    NATIVE_THREADPOOLSPEC(tps), /* cpuspec */
			    NATIVE_MEMSPEC(ms), /* memspec */
			    NULL, /* init func, start first thread explicitly at Java level */
			    NULL, /* initarg */
			    moab_domain_termfunc /* termfunc */);

	if (!rc)
		panic("Error: an_domain_create() -> %s\n",  an_error_message());
	
	assert(rc == (void*)NATIVE_DOMAIN(this));

	/*
	 * Fixup the Domain Resource client
	 */
	domainResourceFixupCurrent(NATIVE_DOMAIN(this));

}

/**
 * For starting an explicit thread with an explicit callback.
 *
 * Currently only used for starting the first thread in a domain.
 */
void edu_utah_janos_nodeos_moab_Domain_threadStart(struct Hedu_utah_janos_nodeos_moab_Domain* this,
						 struct Hjava_lang_Runnable* runnable,
						 struct Hjava_lang_String* name)
{
	assert(this);
	assert(runnable);
	assert(name); /* XXX */

	moabglue_startExplicitThread(NATIVE_DOMAIN(this),
				     runnable);
}

void edu_utah_janos_nodeos_moab_Domain_destroy0(struct Hedu_utah_janos_nodeos_moab_Domain* this)
{
	/* XXX nothing to do... */
}

struct Hjava_lang_Object* edu_utah_janos_nodeos_moab_Domain_getObject(struct Hedu_utah_janos_nodeos_moab_Domain* this)
{
	NOTIMPLEMENTED("MoabGlueFunction in moab_Domain.c");
	return NULL;
}

void edu_utah_janos_nodeos_moab_Domain_setObject(struct Hedu_utah_janos_nodeos_moab_Domain* this,
					       struct Hjava_lang_Object* obj)
{
	NOTIMPLEMENTED("MoabGlueFunction in moab_Domain.c");
}

struct Hedu_utah_janos_nodeos_moab_Domain*
edu_utah_janos_nodeos_moab_Domain_native_currentDomain(void)
{
	extern an_domain_t rootDomain;

	an_domain_t* currndomain;

	currndomain = an_domain_current();

	/* XXX there is no java wrapper for the real root domain. */
	if (currndomain == &rootDomain)
		return NULL;
	
	return KAFFE_DOMAIN(currndomain);
}

void edu_utah_janos_nodeos_moab_Domain_setName(struct Hedu_utah_janos_nodeos_moab_Domain* this,
					     struct Hjava_lang_String* name)
{
	NOTIMPLEMENTED("MoabGlueFunction in moab_Domain.c");
}

void edu_utah_janos_nodeos_moab_Domain_getStats(struct Hedu_utah_janos_nodeos_moab_Domain* this,
					      struct Hedu_utah_janos_nodeos_moab_DomainStats *fs)
{
	an_error_t err;

	err = an_domain_stats(NATIVE_DOMAIN(this), NATIVE_DOMAINSTATS(fs));
	NATIVE_DOMAINSTATS(fs)->memused += gc_get_resource(this)->used;
	assert(err == 0);
}

struct Hjava_lang_String*
edu_utah_janos_nodeos_moab_Domain_getName(struct Hedu_utah_janos_nodeos_moab_Domain* this)
{
	NOTIMPLEMENTED("MoabGlueFunction in moab_Domain.c");
	return NULL;
}

