/*
 * moab_EthernetAddressSpec.c
 *
 * Copyright (c) 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>

#include <nodeos/impl/an_impl.h>

#include "moabglue.h"
#include "edu_utah_janos_nodeos_moab_EthernetAddressSpec.h"

#include <misc/stringParsing.h>

void edu_utah_janos_nodeos_moab_EthernetAddressSpec_init(struct Hedu_utah_janos_nodeos_moab_EthernetAddressSpec *this,
							 struct Hjava_lang_String *jAddr)
{
	long long srcEthAddr = 0, dstEthAddr = 0;
	char *addr = checkPtr(stringJava2C(jAddr));
	an_chanaddr_t *ca;
	int type = 0;

	ca = NATIVE_ADDRSPEC(
		(struct Hedu_utah_janos_nodeos_moab_ChannelAddressSpec *)this);
	/* Parse out the MAC address and optional type value */
	if( parseString(addr,
			SPO_HexLongInteger, &dstEthAddr,
			SPO_Cond, ":",
			  SPO_HexInteger, &type,
			  SPO_End,
			SPO_Cond, ":",
			  SPO_HexLongInteger, &srcEthAddr,
			  SPO_End,
			SPO_End) )
	{
		ca->eth.daddr[0] = (u_char)((dstEthAddr >> 40) & 0xff);
		ca->eth.daddr[1] = (u_char)((dstEthAddr >> 32) & 0xff);
		ca->eth.daddr[2] = (u_char)((dstEthAddr >> 24) & 0xff);
		ca->eth.daddr[3] = (u_char)((dstEthAddr >> 16) & 0xff);
		ca->eth.daddr[4] = (u_char)((dstEthAddr >>  8) & 0xff);
		ca->eth.daddr[5] = (u_char)((dstEthAddr      ) & 0xff);
		
		ca->eth.saddr[0] = (u_char)((srcEthAddr >> 40) & 0xff);
		ca->eth.saddr[1] = (u_char)((srcEthAddr >> 32) & 0xff);
		ca->eth.saddr[2] = (u_char)((srcEthAddr >> 24) & 0xff);
		ca->eth.saddr[3] = (u_char)((srcEthAddr >> 16) & 0xff);
		ca->eth.saddr[4] = (u_char)((srcEthAddr >>  8) & 0xff);
		ca->eth.saddr[5] = (u_char)((srcEthAddr      ) & 0xff);
		
		ca->eth.type = (u_short)htons(type);
	}
	else
	{
		printf("Bogus address spec %s\n", addr);
		assert(0);
	}
}

Hjava_lang_String *
edu_utah_janos_nodeos_moab_EthernetAddressSpec_getSource(struct Hedu_utah_janos_nodeos_moab_EthernetAddressSpec *this)
{
	Hjava_lang_String *retval = 0;
	an_chanaddr_t *ca;
	char tmp[16];

	ca = NATIVE_ADDRSPEC(
		(struct Hedu_utah_janos_nodeos_moab_ChannelAddressSpec *)this);
	sprintf(tmp,
		"%x%x%x%x%x%x",
		ca->eth.saddr[0],
		ca->eth.saddr[1],
		ca->eth.saddr[2],
		ca->eth.saddr[3],
		ca->eth.saddr[4],
		ca->eth.saddr[5]);
	retval = stringC2Java(tmp);
	return( retval );
}

Hjava_lang_String *
edu_utah_janos_nodeos_moab_EthernetAddressSpec_getDestination(struct Hedu_utah_janos_nodeos_moab_EthernetAddressSpec *this)
{
	Hjava_lang_String *retval = 0;
	an_chanaddr_t *ca;
	char tmp[16];

	ca = NATIVE_ADDRSPEC(
		(struct Hedu_utah_janos_nodeos_moab_ChannelAddressSpec *)this);
	sprintf(tmp,
		"%x%x%x%x%x%x",
		ca->eth.daddr[0],
		ca->eth.daddr[1],
		ca->eth.daddr[2],
		ca->eth.daddr[3],
		ca->eth.daddr[4],
		ca->eth.daddr[5]);
	retval = stringC2Java(tmp);
	return( retval );
}
