/*
 * moab_InChannel.c
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>

#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/stringSupport.h"
#include "../../../../kaffe/kaffevm/thread.h"

#include "moabglue.h"
#include "moabthread.h"

struct Hedu_utah_janos_nodeos_DeliverFunc; // XXX
struct Hedu_utah_janos_nodeos_moab_DemultiplexKey;
struct Hedu_utah_janos_nodeos_moab_Domain;
struct Hedu_utah_janos_nodeos_moab_BufferHandle;
#include "edu_utah_janos_nodeos_moab_InChannel.h"

jint 
edu_utah_janos_nodeos_moab_InChannel_init(struct Hedu_utah_janos_nodeos_moab_InChannel* this,
					  struct Hedu_utah_janos_nodeos_moab_Domain* domain,
					  jint threadCount,
					  struct Hedu_utah_janos_nodeos_moab_DemultiplexKey* key,
					  struct Hjava_lang_String* protoSpec,
					  struct Hedu_utah_janos_nodeos_moab_ChannelAddressSpec *cas,
					  struct Hedu_utah_janos_nodeos_DeliverFunc* javaCallback)
{
	extern JNIEnv Kaffe_JNIEnv;
	JNIEnv *env = &Kaffe_JNIEnv;
	moabInChannelData *ic = NATIVE_INCHAN(this);
	an_chan_t *rc;
	an_chanproto_t proto;
	an_chanaddr_t *addr;
	an_netspec_t netspec;

	/* Inform the GC that the NodeOS will be holding on to some
	 * objects.  These references are only cleaned up if the
	 * corresponding Java object is destroyed.  Or, they're made
	 * irrelevant if the owning domain is terminated.
	 *
	 * Note: `this' is anchored in the moab_Domain.
	 */
	gc_add_ref(key);
	gc_add_ref(javaCallback);
	
	ic->javaCallBack = javaCallback;
	ic->runmethod = (*env)->GetMethodID(
		env,
		(*env)->GetObjectClass(env, javaCallback),
		"receive",
		"(Ledu/utah/janos/nodeos/BufferHandle;I)V");
	assert(sizeof(proto.data) > STRING_SIZE(protoSpec));
	proto.len = STRING_SIZE(protoSpec);
	stringJava2CBuf(protoSpec, proto.data, sizeof(proto.data));

	/* Setup cpu spec.  XXX currently just the MAX_THREADS is used by Moab. */
	/* XXX uses threads out of the Domain's threadpool. */
	netspec.maxthreads = threadCount;
	netspec.buffers.npbufs = 0;

	addr = NATIVE_ADDRSPEC(cas);

	/*
	 * XXX initial pbufs are null for now because the Java code does
	 * a couple explicit an_pbuf_adds.
	 */
	rc = an_inchan_create(&ic->chan,
			      NATIVE_DOMAIN(domain),
			      NATIVE_DEMUXKEY(key),
			      &proto, addr,
			      &netspec,
			      moabglue_inChanCallback,
			      (void *)ic);
	if (rc != &ic->chan)
	{
		/* NodeOS didn't keep refs to any of these. */
		gc_rm_ref(key);
		gc_rm_ref(javaCallback);

		return AN_ERR_INVALIDARG; // XXX XXX
	}
	
	return 0;
}

void edu_utah_janos_nodeos_moab_InChannel_native_addBuffer(struct Hedu_utah_janos_nodeos_moab_InChannel* this,
							   struct Hedu_utah_janos_nodeos_moab_BufferHandle* buf)
{
	int rc;

	gc_add_ref(buf);
	rc = an_inchan_addpbuf(&NATIVE_INCHAN(this)->chan, NATIVE_PBUF(buf));
	assert(rc == 0);
}

struct Hedu_utah_janos_nodeos_BufferHandle*
edu_utah_janos_nodeos_moab_InChannel_native_remBuffer(struct Hedu_utah_janos_nodeos_moab_InChannel* this)
{
	NOTIMPLEMENTED("MoabGlueFunction in moab_InChannel.c");
	return NULL;
}

void
edu_utah_janos_nodeos_moab_InChannel_getStats(struct Hedu_utah_janos_nodeos_moab_InChannel* this,
					      struct Hedu_utah_janos_nodeos_moab_ChanStats *cs)
{
	an_error_t err;
	
	err = an_chan_stats(&NATIVE_INCHAN(this)->chan, NATIVE_CHANSTATS(cs));
	assert(err == 0);
}

jbool
edu_utah_janos_nodeos_moab_InChannel_handoff(struct Hedu_utah_janos_nodeos_moab_InChannel* this,
					     struct Hedu_utah_janos_nodeos_moab_BufferHandle* buf)
{
	jbool retval = false;

	if( an_inchan_handoff(&NATIVE_INCHAN(this)->chan,
			      NATIVE_PBUF(buf)) == 0 )
	{
		retval = true;
	}
	return( retval );
}

void edu_utah_janos_nodeos_moab_InChannel_native_destroy(struct Hedu_utah_janos_nodeos_moab_InChannel* this)
{
	NOTIMPLEMENTED("MoabGlueFunction in moab_InChannel.c");
}

