/*
 * moab_Setup.c
 *
 * Copyright (c) 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>

#include <nodeos/an_moab_setup.h>
#include <nodeos/an_domain.h>
#include <nodeos/an_cred.h>
#include <nodeos/an_thread.h>

#include "moabglue.h"
#include "jtypes.h"

struct Hjava_lang_ClassLoader;

/* XXX FROM kaffe/kaffevm/support.h: */
extern jvalue
do_execute_java_class_method(const char*, struct Hjava_lang_ClassLoader*, const char*, const char*, ...);

struct rootDomainSetupInfo {
	const char* classname;
	struct Hjava_lang_ClassLoader* classloader;
	const char* methodname;
	const char* methodsignature;
	struct Hjava_lang_Object* eeDomain;
	struct Hjava_lang_String* initClassStr;
	HArrayOfObject* initArgs;
};

static void
setupInitialEE(void *arg);

static void
initialEETerm(an_domain_t *domain);


void
edu_utah_janos_nodeos_moab_Setup_startEEDomain(struct Hjava_lang_String* initClass,
					     HArrayOfObject* initArgs,
					     struct Hjava_lang_Class* domainClass,
					     struct Hjava_lang_Class* credentialClass,
					     struct Hjava_lang_Class* setupClass,
					     struct Hjava_lang_String* methodName)
{
	extern an_cred_t eeCred; /* nodeos/md.c */

	struct Hjava_lang_Object* eeDomain;
	struct rootDomainSetupInfo* eeArg = malloc(sizeof(struct rootDomainSetupInfo));
	/* an_thread_t *threads[5]; */
	errorInfo einfo;
	an_cpuspec_t eeCpu;
	an_memspec_t eeMem;

	assert(initClass);
	assert(setupClass);
	assert(domainClass);
	assert(credentialClass);
	assert(methodName);

	panic("THIS FUNCTION IS ALREADY DEPRECATED.");

	/*
	 * XXX setup the new domain's Cpu and Mem specs.
	 */
	eeCpu.ttype = AN_TH_EXPLICIT;
	eeCpu.threads.ex.nthreads = 5;
	eeCpu.threads.ex.threads  = 0;
	eeCpu.rate = 0;
	eeCpu.slice.tv_sec = 1;
	eeCpu.slice.tv_nsec = 0;
	eeCpu.sched = NULL;
	eeMem.nchunks = 1000;
	eeMem.pool = an_mempool_current();

	/*
	 * Create an instance of a domainClass object
	 */
	eeDomain = newObjectChecked(domainClass, &einfo);
	assert(eeDomain != NULL);

	eeArg->classname = "edu.utah.janos.nodeos.moab_Setup";
	eeArg->classloader = NULL;
	eeArg->methodname = "startInitClass";
	eeArg->methodsignature = "(Ledu/utah/janos/nodeos/Domain;Ljava/lang/String;[Ljava/lang/String;)V";
	eeArg->eeDomain = eeDomain;
	eeArg->initClassStr = initClass;
	eeArg->initArgs = initArgs;

	an_domain_create(NATIVE_DOMAIN((struct Hedu_utah_janos_nodeos_moab_Domain*)eeDomain),
		       &eeCred,
		       &eeCpu, &eeMem,
		       setupInitialEE, eeArg,
		       initialEETerm);
	
}

static void
initialEETerm(an_domain_t *domain)
{
	printf("initial EE termination");
	an_moab_shutdown();
}


static void
setupInitialEE(void *arg)
{
	struct rootDomainSetupInfo* inf = (struct rootDomainSetupInfo*)arg;

	/*
	 * Invoke "methodName" on setupClass.
	 */
	do_execute_java_class_method(inf->classname,
				     inf->classloader,
				     inf->methodname,
				     inf->methodsignature,
				     inf->eeDomain,
				     inf->initClassStr,
				     inf->initArgs
				     );

	free(arg);
}
