/*
 * moabglue.c
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "moabglue.h"
#include "domainResource.h"

#include <native.h> /* XXX kaffe */
#include <assert.h>

#include <kaffevm/classMethod.h>
#include <kaffevm/module.h>

#include <janosvm/nativeClass.h>
#include <janosvm/logResource.h>

static int initMoabModule(int stage);
static void killMoabModule(int stage);

module_t moabModule = {
	"moab",
	0,
	initMoabModule,
	killMoabModule,
	0
};

typedef struct _moabNativeClass {
	char *name;
	nativeClass nClass;
} moabNativeClass;

static
moabNativeClass moabNativeClassList[] = {
	{ "edu/utah/janos/nodeos/moab_MemSpec",
	  { 0, 0, sizeof(an_memspec_t) } },
	{ "edu/utah/janos/nodeos/moab_BufferHandle",
	  { 0, 0, sizeof(an_pbuf_t) } },
	{ "edu/utah/janos/nodeos/moab_DemultiplexKey",
	  { 0, 0, sizeof(an_demuxkey_t) } },
	{ "edu/utah/janos/nodeos/moab_Credentials",
	  { 0, 0, sizeof(an_cred_t) } },
	{ "edu/utah/janos/nodeos/moab_ThreadPool",
	  { 0, 0, sizeof(an_cpuspec_t) } },
	{ "edu/utah/janos/nodeos/moab_ThreadPoolSpec",
	  { 0, 0, sizeof(an_cpuspec_t) } },
	{ "edu/utah/janos/nodeos/moab_OutChannel",
	  { 0, 0, sizeof(an_chan_t) } },
	{ "edu/utah/janos/nodeos/moab_InChannel",
	  { 0, 0, sizeof(moabInChannelData) } },
	{ "edu/utah/janos/nodeos/moab_CutThroughChannel",
	  { 0, 0, sizeof(an_chan_t) } },
	{ "edu/utah/janos/nodeos/moab_Thread",
	  { 0, 0, sizeof(struct _jthread) } },
	{ "edu/utah/janos/nodeos/moab_Domain",
	  { 0, 0, sizeof(an_domain_t) } },
	{ "edu/utah/janos/nodeos/moab_DomainStats",
	  { 0, 0, sizeof(an_domainstats_t) } },
	{ "edu/utah/janos/nodeos/moab_ChanStats",
	  { 0, 0, sizeof(an_chanstats_t) } },
	{ "edu/utah/janos/nodeos/moab_ChannelAddressSpec",
	  { 0, 0, sizeof(an_chanaddr_t) } },
	{ "edu/utah/janos/nodeos/moab_Transports",
	  { 0, 0, sizeof(moabInfoArray) } },
	{ 0, { 0 } }
};

extern Hjava_lang_Class *TransportInterfaceClass;
extern Hjava_lang_Class *EthernetAddressSpecClass;

static iLock *initLock;
static jteam *userTeam = 0;

int initMoabModule(int stage)
{
	int lpc, retval = 1;
	errorInfo einfo;
	int iLockRoot;
	
	lockStaticMutex(&initLock);
	if( userTeam )
	{
		teamLogEntry("moab",
			     LEA_String, "Error", "Library already in use.", -1,
			     LEA_String, "userTeam", userTeam->name, -1,
			     LEA_TAG_DONE);
		unlockStaticMutex(&initLock);
		return 0;
	}
	else
	{
		userTeam = currentTeam();
	}
	unlockStaticMutex(&initLock);
	for( lpc = 0; retval && moabNativeClassList[lpc].name; lpc++ )
	{
		struct Hjava_lang_Class *cl;

		if( (cl = lookupClass(moabNativeClassList[lpc].name,
				      NULL,
				      &einfo)) )
		{
			retval = bindNativeClass(&moabNativeClassList[lpc].
						 nClass,
						 cl,
						 &einfo);
		}
		else
		{
			teamLogEntry("moab",
				     LEA_String, "Error", "Couldn't find moab class to bind native data too...", -1,
				     LEA_String, "class", moabNativeClassList[lpc].name, -1,
				     LEA_TAG_DONE);
			retval = 0;
		}
	}
	if( !(TransportInterfaceClass =
	      lookupClass("edu/utah/janos/nodeos/moab_TransportInterface",
			  NULL,
			  &einfo)) )
	{
		retval = 0;
	}
	if( !(EthernetAddressSpecClass =
	      lookupClass("edu/utah/janos/nodeos/moab_EthernetAddressSpec",
			  NULL,
			  &einfo)) )
	{
		retval = 0;
	}
	return( retval );
}

void killMoabModule(int stage)
{
	int iLockRoot;
	
	lockStaticMutex(&initLock);
	if( currentTeam() == userTeam )
	{
		int lpc;
		
		for( lpc = 0; moabNativeClassList[lpc].name; lpc++ )
		{
			unbindNativeClass(&moabNativeClassList[lpc].nClass);
		}
		userTeam = 0;
	}
	unlockStaticMutex(&initLock);
}
