/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.net;

final public class DatagramPacket {

private byte[] buf;
private final int offset;
private int length;
private InetAddress address;
private int port;

/**
 * Receive buffer.
 */
public DatagramPacket(byte ibuf[], int ioffset, int ilength) {
	this.setData(ibuf);
	if ((ioffset >= 0) && (ioffset < buf.length)) {
		offset = ioffset;
	}
	else {
		throw new IllegalArgumentException();
	}
	this.setLength(ilength);
	port = -1;
}

public DatagramPacket(byte ibuf[], int ilength)
{
	this(ibuf, 0, ilength);
}

/**
 * Send buffer.
 */
public DatagramPacket(byte ibuf[],
		      int ioffset,
		      int ilength,
		      InetAddress iaddr,
		      int iport) {
	this(ibuf, ioffset, ilength);

	this.setAddress(iaddr);
	this.setPort(iport);
}

public DatagramPacket(byte ibuf[], int ilength, InetAddress iaddr, int iport)
{
	this(ibuf, 0, ilength, iaddr, iport);
}

synchronized void prime() {
	this.address = new InetAddress();
}

public synchronized InetAddress getAddress() {
	return (address);
}

public synchronized byte[] getData() {
	return (buf);
}

public synchronized int getOffset() {
	return (offset);
}

public synchronized int getLength() {
	return (length);
}

public synchronized int getPort() {
	return (port);
}

public synchronized void setAddress(InetAddress addr) {
	if( addr == null )
		throw new NullPointerException("null address");
	address = addr;
}

public synchronized void setData(byte[] newbuf) {
	if( newbuf == null )
		throw new NullPointerException("null packet buffer");
	buf = newbuf;
}

public synchronized void setLength(int ilength) {
	if ((offset + ilength <= buf.length) && (offset + ilength >= offset)) {
		length = ilength;
	}
	else {
		throw new IllegalArgumentException();
	}
	length = ilength;
}

public synchronized void setPort(int newport) {
	if (newport < 0 || newport > 65535) {
		throw new IllegalArgumentException();
	}
	port = newport;
}

}
