/*
 * StdOutputStream.java
 * Output stream for a team's standard output.
 *
 * Copyright (c) 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.io;

import java.io.OutputStream;

/**
 * Output stream for a team's standard output.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class StdOutputStream
    extends OutputStream
{

    static
    {
	/* We depend on the IO library */
	System.loadLibrary("io");
    }

    /**
     * The standard output kind.
     *
     * @see #kind
     */
    public static final int STDOUT = 1;
    
    /**
     * The standard error kind.
     *
     * @see #kind
     */
    public static final int STDERR = 2;

    /**
     * The output stream this object is bound too.
     *
     * @see #STDOUT
     * @see #STDERR
     */
    private int kind;

    /**
     * Construct a StdOutputStream of the given kind.
     *
     * @param kind The kind of output stream, stdout or stderr.
     */
    public StdOutputStream(int kind)
    {
	this.kind = kind;
    }
    
    public native void close();

    public native void flush();

    public native void write(byte b[], int off, int len);

    private native void writeInt(int x);
    
    public void write(int x)
    {
	this.writeInt(x);
    }

    public String toString()
    {
	return "StdOutputStream["
	    + super.toString()
	    + "; kind="
	    + this.kind
	    + "]";
    }
}
