/*
 * ClassPathSpec.java
 * Resource specification for a team's class path.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Specification for the class path resource.
 *
 * @see ClassPathResource
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class ClassPathSpec
    extends ResourceSpec
    implements Cloneable
{
    /**
     * Constructs the native part of the class.
     */
    private native void nativeConstructor(String entries[]);

    /**
     * Finalizes the native part of the class.
     */
    private native void nativeFinalizer();
    
    /**
     * Construct a class path specification with the given paths.
     *
     * @param entries An array of strings specifying each part of the
     * classpath or null if the ClassPath is empty.  Any null elements
     * are ignored.
     */
    public ClassPathSpec(String entries[])
    {
	this.nativeConstructor(entries);
    }
    
    public void finalize()
    {
	this.nativeFinalizer();
    }

    /**
     * @return The number of paths in the spec.
     */
    public native int getPathCount();

    /**
     * @param index The index of the class path to get.
     * @return The class path at the given index.
     */
    public native String getPath(int index);

    public String toString()
    {
	int lpc, count = this.getPathCount();
	String retval =
	    "ClassPathSpec[paths["
	    + count
	    + "]={ ";

	count = this.getPathCount();
	for( lpc = 0; lpc < count; lpc++ )
	{
	    if( lpc > 0 )
		retval += ", ";
	    retval += this.getPath(lpc);
	}
	retval += " }]";
	return retval;
    }
    
    public native Object clone();

    /**
     * @return The ClassPathSpec associated with the current team.
     */
    public static native ClassPathSpec current();
}
