/*
 * ResourceSpec.java
 * Base class for resource specifications
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Base class for resource specifications.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class ResourceSpec
{
    /**
     * Indicates whether this spec was copied from the configuration file,
     * or if it was created by java code.
     */
    private boolean found = false;
	
    /**
     * Find and construct a ResourceSpec with the given name.
     *
     * @param name Name that corresponds to a resource specification
     * defined in the config file.
     */
    private static native ResourceSpec nativeFind(String name);
    
    /**
     * Find and create a resource specification with the given name.
     *
     * @param name Name that corresponds to a resource specification
     * defined in the config file.
     * @return The ResourceSpec as defined in the config file or NULL if
     * it couldn't be found.
     */
    public static ResourceSpec find(String name)
    {
	return ResourceSpec.nativeFind(name);
    }
    
}
