/*
 * ResourceSpecException.java
 * Exception indicating that the system couldn't satisfy a resource
 * specification.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Exception indicating that the system couldn't satisfy a resource
 * specification.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class ResourceSpecException
    extends Exception
{
    /**
     * The ResourceSpec that couldn't be satisfied.
     */
    private ResourceSpec unsatisfiedSpec;

    /**
     * Construct an empty ResourceSpecException.
     */
    public ResourceSpecException()
    {
    }

    /**
     * Construct a ResourceSpecException with a message.
     *
     * @param msg A description of the problem.
     */
    public ResourceSpecException(String msg)
    {
	super(msg);
    }

    /**
     * Construct a ResourceSpecException with an unsatisfied ResourceSpec.
     *
     * @param unsatisfiedSpec The ResourceSpec that couldn't be satisfied by
     * the system.
     */
    public ResourceSpecException(ResourceSpec unsatisfiedSpec)
    {
	this.unsatisfiedSpec = unsatisfiedSpec;
    }

    /**
     * @return The ResourceSpec that couldn't be satisfied by the system, or
     * null if the resource was the default or internal.
     */
    public ResourceSpec getUnsatisfiedSpec()
    {
	return this.unsatisfiedSpec;
    }

    public String toString()
    {
	return "ResourceSpecException["
	    + super.toString()
	    + "; unsatisfiedSpec="
	    + this.unsatisfiedSpec
	    + "]";
    }
}
