/*
 * DuplicateExportException.java
 * Exception thrown when trying to export a object with a name that has
 * already been used.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Exception thrown when trying to export a object with an 'id' that has
 * already been used.  Ids for exportable objects are guaranteed to be unique.
 * This is enforced when Exportables are actually exported from their team.
 * The name must be unique in the 'equals' sense, not the '==' sense.  (Think
 * Hashtable key.)
 *
 * @see Exportable
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a> */
public class DuplicateExportException
    extends Exception
{
    public DuplicateExportException(String s)
    {
	super(s);
    }
}
