/*
 * IndexedResourceSpecException.java
 * Internal exception thrown when an resource spec couldn't be satisfied.
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Internal exception thrown when an resource spec couldn't be satisfied.
 */
class IndexedResourceSpecException
    extends Exception
{
    /**
     * The index in the ResourceSpec array passed to TeamBackEnd() that
     * couldn't be satisfied.
     */
    private final int index;
    
    /**
     * Construct an IndexedResourceSpecException with the given index.
     *
     * @param index The index in the ResourceSpec array passed to TeamBackEnd()
     * that couldn't be satisfied.
     */
    IndexedResourceSpecException(int index)
    {
	this.index = index;
    }

    /**
     * @return The index in the ResourceSpec array passed to TeamBackEnd() that
     * couldn't be satisfied.
     */
    int getIndex()
    {
	return this.index;
    }

    public String toString()
    {
	return "IndexedResourceSpecException["
	    + super.toString()
	    + "; index="
	    + this.index
	    + "]";
    }
}
