/*
 * List.java
 * Root for a doubly linked list.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Root for a doubly linked list.
 *
 * @see ListNode
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class List
{
    /**
     * The header for the list.
     */
    public ListNode head;
    /**
     * The tail node for the list.
     */
    public ListNode tail;

    /**
     * Construct an empty list object.
     */
    public List()
    {
	this.head = new ListNode();
	this.tail = new ListNode();
	this.head.succ = this.tail;
	this.tail.pred = this.head;
    }

    /**
     * @return The number of nodes in the list.
     */
    public int count()
    {
	int retval = 0;
	ListNode curr;

	curr = this.head.succ;
	while( curr.succ != null )
	{
	    retval++;
	    curr = curr.succ;
	}
	return retval;
    }

    public ListNode nodeAt(int index)
    {
	ListNode curr, retval = null;

	curr = this.head;
	while( (curr != null) && (index > 0) )
	{
	    curr = curr.succ;
	    index--;
	}
	return retval;
    }

    /**
     * Add a node to the head of the list.
     *
     * @param ln The node to add.
     */
    public void addHead(ListNode ln)
    {
	ln.succ = this.head.succ;
	ln.succ.pred = ln;
	ln.pred = this.head;
	this.head.succ = ln;
    }

    /**
     * Add a node to the tail of the list.
     *
     * @param ln The node to add.
     */
    public void addTail(ListNode ln)
    {
	ln.succ = this.tail;
	ln.pred = this.tail.pred;
	ln.pred.succ = ln;
	this.tail.pred = ln;
    }

    /**
     * Remove and return the node at the head of the list, or null if its
     * empty.
     *
     * @return The node to removed.
     */
    public ListNode remHead()
    {
	ListNode retval = null;

	if( this.head.succ != this.tail )
	{
	    retval = this.head.succ;
	    retval.remove();
	}
	return retval;
    }

    /**
     * Remove and return the node at the tail of the list, or null if its
     * empty.
     *
     * @return The node to removed.
     */
    public ListNode remTail()
    {
	ListNode retval = null;

	if( this.tail.pred != this.head )
	{
	    retval = this.tail.pred;
	    retval.remove();
	}
	return retval;
    }

    /**
     * @return True if the list is empty, false otherwise.
     */
    public boolean isEmpty()
    {
	return this.head.succ == this.tail;
    }

    public String toString()
    {
	return "List["
	    + "]";
    }
}
