/*
 * ListNode.java
 * Doubly linked list node.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Doubly linked list node.
 * Only to be used via a 'List'
 *
 * @see List
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class ListNode
{
    /**
     * The successor node in the list.
     * <p>
     * XXX use an accessor, this should be read-only outside this class.
     */
    public ListNode succ;

    /**
     * The predecessor node in the list.
     * <p>
     * XXX use an accessor, this should be read-only outside this class.
     */
    public ListNode pred;

    public ListNode()
    {
    }

    /**
     * Remove the node from whatever list it is currently on.  If its
     * not on a list, that's okay.
     */
    public void remove()
    {
	if( this.succ != null )
	{
	    this.pred.succ = this.succ;
	    this.succ.pred = this.pred;
	    this.succ = null;
	    this.pred = null;
	}
    }

    /**
     * Insert the given node directly after `this'.
     *
     * @param ln The node to append.
     */
    public void append(ListNode ln)
    {
	ln.pred = this;
	ln.succ = this.succ;
	ln.succ.pred = ln; 
	this.succ = ln;
    }

    /**
     * Insert the given node directly before `this'.
     *
     * @param ln The node to prepend.
     */
    public void prepend(ListNode ln)
    {
	ln.succ = this;
	ln.pred = this.pred;
	ln.pred.succ = ln;
	this.pred = ln;
    }
    
    /**
     * Clone a ListNode.  The node is cloned in isolation;
     * the clone is never on a list, regardless of the state
     * of the original.
     */
    protected Object clone()
	throws CloneNotSupportedException
    {
	Object retval = super.clone();

	if( this.succ != null )
	{
	    ListNode ln = (ListNode)retval;
	    
	    /* Clear the links */
	    ln.succ = null;
	    ln.pred = null;
	}
	return retval;
    }

    public String toString()
    {
	return "ListNode["
	    + "]";
    }
}
