/*
 * TeamEvent.java
 * Simple class that describes a team related event.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Simple class that describes a team related event.
 */
public class TeamEvent
{
    /**
     * The name of the team, currently only valid for the CREATED event.
     */
    private String name;

    /**
     * The unique identifier of the team.
     */
    private int id;

    private static final int MIN       = 0;
    
    /**
     * The team was created.
     */
    public static final int CREATED    = 1;

    /**
     * There are only daemon threads left in the team.
     */
    public static final int FINISHED   = 2;
    
    /**
     * The team was marked for termination.
     */
    public static final int TERMINATED = 3;

    /**
     * The team is dead and all of its resources have been reclaimed.
     */
    public static final int BURIED     = 4;
    
    private static final int MAX       = 5;

    /**
     * Mapping from event id to a string.
     */
    private static final String eventNames[] = {
	null,
	"created",
	"finished",
	"terminated",
	"buried",
    };
    
    /**
     * One of the above event number values.
     */
    private int event;

    public TeamEvent()
    {
    }

    /**
     * @param name The name of the team that the event relates to.
     */
    public void setName(String name)
    {
	this.name = name;
    }

    /**
     * @return The name of the team that the event relates to.
     */
    public String getName()
    {
	return this.name;
    }

    /**
     * @param id The unique identifier of the team.
     */
    public void setID(int id)
    {
	this.id = id;
    }

    /**
     * @return The unique identifier of the team.
     */
    public int getID()
    {
	return this.id;
    }
    
    /**
     * @param value The event number.
     */
    public void setEvent(int value)
    {
	this.event = value;
    }

    /**
     * @return The event number.
     */
    public int getEvent()
    {
	return this.event;
    }

    public String toString()
    {
	return "TeamEvent[name="
	    + this.name
	    + "; id="
	    + this.id
	    + "; event="
	    + eventNames[this.event]
	    + "]";
    }
}
