/*
 * TeamList.java
 * An Importable for TeamListBackEnd that just provides a path for callbacks.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * An Importable for TeamListBackEnd that just provides a path for callbacks.
 * NOTE: This is final since we don't want to require the library writer to
 * export their classes from the kernel.  And, otherwise, the kernel would be
 * calling non-kernel code, ick.
 */
public final class TeamList
    extends Importable
{
    /**
     * The name of the TeamListBackEnd object exported by the kernel.
     */
    public static String exportID = "TeamList";

    /**
     * Pointer to the back end object...  XXX Is this needed?
     */
    private TeamListBackEnd be;

    /**
     * The local object that will handle the event.
     */
    private TeamListHandler tlh;

    protected final void bindTo(Exportable ex)
    {
	this.be = (TeamListBackEnd)ex;
    }

    protected final void unbindFrom()
    {
	this.be = null;
    }

    /**
     * @param tlh The call back that will handle any team events.
     */
    public synchronized void setTeamListHandler(TeamListHandler tlh)
    {
	this.tlh = tlh;
    }

    /**
     * @return The call back that will handle any team events.
     */
    public synchronized TeamListHandler getTeamListHandler()
    {
	return this.tlh;
    }

    /**
     * Use this object to import the kernel's TeamListBackEnd.
     */
    public synchronized void listen()
    {
	if( this.be == null )
	{
	    TeamHandle kernel = Team.current().getKernelTeam();

	    try
	    {
		kernel.importObject(this, exportID);
	    }
	    catch(DeadTeamException e)
	    {
	    }
	    catch(TeamEjectionException e)
	    {
	    }
	    catch(NoSuchExportException e)
	    {
	    }
	}
    }
    
    /**
     * Callback for handling events related to teams.  NOTE: This is called in
     * the context of the kernel so you must use the provided TeamReservation
     * if you want to do anything in the Importables team.
     *
     * @param tr A TeamReservation thats reserved the team that `this' is in.
     * @param ev The TeamEvent object that describes the event that occurred.
     */
    public synchronized void event(TeamReservation tr, TeamEvent ev)
    {
	if( this.tlh != null )
	{
	    this.tlh.event(tr, ev);
	}
    }

    public String toString()
    {
	return "TeamList["
	    + super.toString()
	    + "; be="
	    + this.be
	    + "; tlh="
	    + this.tlh
	    + "]";
    }
}
