/*
 * TeamListBackEnd.java
 * A kernel export that sends out notifications of team related events.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * A kernel export that sends out notifications of team related events.
 */
class TeamListBackEnd
    extends Exportable
{
    /**
     * A cached TeamEvent object that will be passed to the call backs.
     */
    private TeamEvent ev = new TeamEvent();

    /**
     * The ImportEnumerator for our ExportManager.
     */
    private ExportManager.ImportEnumerator ie;

    /**
     * The ExportManager for our object.
     */
    private ExportManager em;

    /**
     * Construct an empty TeamListBackEnd object.
     */
    public TeamListBackEnd()
    {
    }

    /**
     * @param em The object thats managing this export.
     */
    public synchronized void setExportManager(ExportManager em)
    {
	this.em = em;
	this.ie = em.getEnumerator();
    }

    /**
     * Trigger an event notification for the given team.
     *
     * @param name The name of the team that this event relates too.
     * @param id The unique identifier of the team this event relates too.
     * @param event The TeamEvent number that indicates what event this is.
     */
    public synchronized void event(String name, int id, int event)
    {
	if( this.em != null )
	{
	    /* Setup the TeamEvent object. */
	    this.ev.setName(name);
	    this.ev.setID(id);
	    this.ev.setEvent(event);

	    /* Put the Enumerator on the imports list */
	    this.em.imports(this.ie);
	    /* Traverse the list of Importable's */
	    while( this.ie.nextImport() )
	    {
		try
		{
		    TeamList tl = (TeamList)this.ie.getImportable();

		    /* Do the call back */
		    tl.event(this.ie.getReservation(), this.ev);
		}
		catch(Throwable th)
		{
		    /*
		     * XXX Its a call back, i don't care, well log it i guess
		     */
		    System.err.println("Got odd error");
		    th.printStackTrace();
		}
	    }
	}
    }

    /**
     * Trigger an event notification for the given team.
     *
     * @param team The Team object that refers to the team this event relates
     * too.
     * @param event The TeamEvent number that indicates what event this is.
     */
    public synchronized void event(Team team, int event)
    {
	this.event(team.getName(), team.getID(), event);
    }

    public String toString()
    {
	return "TeamListBackEnd["
	    + super.toString()
	    + "; ev="
	    + this.ev
	    + "; ie="
	    + this.ie
	    + "; em="
	    + this.em
	    + "]";
    }
}
