/*
 * TemporaryExport.java
 * Indicates that an exported object is only alive as long as it is imported.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Indicate that an exported object is only alive as long as it is
 * imported by someone.  Only useful when implemented by a subclass
 * of Exportable.
 *
 * @see PersistentExport
 * @see Exportable
 * @see ExportManager#remImporter
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public interface TemporaryExport
{
    /**
     * This method is called when there are no more imports for the export.
     * If it returns true than the export is revoked otherwise it will
     * continue to be available for importing.
     *
     * @return True if the export has expired and should be revoked.
     */
    boolean expired();
}
