/*
 * VMInstance.java
 * Functions specific to the VM as a whole
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Functions specific to the VM as a whole.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class VMInstance
{
    /**
     * Crash into the team boundary, this is basically a violent exit from
     * a visit and results in a DeadTeamException being thrown in the
     * previous team context.  NOTE: This is only used for testing, its
     * possible for the VM to do something similar so we needed a way to
     * explicitly cause it.
     */
    public static native void crash();
    
    public static void panic(String msg)
    {
	System.out.println("Panic: " + msg);
	shutdown();
    }

    public static void panic(String msg, Throwable th)
    {
	System.err.println("Panic: " + msg);
	th.printStackTrace(System.err);
	shutdown();
    }
    
    /**
     * Shutdown the VM.
     */
    public static native void shutdown();

    /**
     * Get the name of this VM instance.
     */
    public static native String getName();

    /**
     * This is a static only object.
     */
    private VMInstance()
    {
    }
}
