/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.io;

public class PipedReader extends Reader {

  final PipedInputStream rawInput = new PipedInputStream();

  public PipedReader() {
  }

  public PipedReader(PipedWriter writer) throws IOException {
    connect(writer);
  }

  public void connect(PipedWriter writer) throws IOException {
    rawInput.connect(writer.rawOutput);
  }

  public int read(char cbuf[], int off, int len) throws IOException {

    // Read each character as two bytes big endian
    len = Math.min(len, rawInput.PIPE_SIZE >> 1) << 1;
    byte[] buf = new byte[len];
    int offset = 0, num;

    do {
      num = rawInput.read(buf, offset, len - offset);
      offset += num;
    } while((num != -1) && ((num & 0x1) != 0));

    num = offset;
    
    if (num == -1) {
      return(-1);
    }

    // Convert pairs of bytes to characters
    for (int i = 0; i < num; i += 2) {
      cbuf[off++] = (char) ((buf[i] << 8) | (buf[i + 1] & 0xff));
    }
    return (num >> 1);
  }

  public void close() throws IOException {
    rawInput.close();
  }

}
