/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.lang;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.CodeSource;
import java.security.ProtectionDomain;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.Vector;
import kaffe.lang.SystemClassLoader;

import java.io.InputStream;

import java.io.File;
import java.util.StringTokenizer;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.util.zip.*;			// why zip and not jar?

public abstract class ClassLoader {

private Object getResource(String name) {
	try{ return getResources(name).nextElement(); }
	catch( IOException ioe ) { ioe.printStackTrace(); return null; }
	catch( NoSuchElementException e ) { return null; }
	} /* end getResource() */

public InputStream getResourceAsStream(String name) {
	// stolen from kaffe/lang/SystemClassLoader.java

        String fileSep = System.getProperties().getProperty("file.separator");
        String pathSep = System.getProperties().getProperty("path.separator");
        String classpath = System.getProperties().getProperty("java.class.path");
        StringTokenizer t = new StringTokenizer(classpath, pathSep);
	String fileName = null;

        while (t.hasMoreTokens()) {
                File file = new File(t.nextToken());
                if (!file.exists()) { continue; }

                if (file.isDirectory()) {
                        file = new File(file, name); 
                        if (file.isFile()) {
				try { 
					return new FileInputStream( file.getCanonicalPath() );
					} catch ( IOException ioe ) { continue; }
				} /* end if file found if directory */
			} /* end if classpath element is directory */

		if(file.isFile()) {
			ZipFile zFile = null;
			try {
				zFile = new ZipFile(file);
				ZipEntry zEntry = zFile.getEntry( name );
				if( zEntry != null && ! zEntry.isDirectory() ) {
					/* slurp this into a stream... */
					return new ZipInputStream( zFile.getInputStream( zEntry ) );
					}
			} catch( IOException ioe ) {
			} finally {
				if( zFile != null ) {
					try{ zFile.close(); } catch( IOException ioe ) {}
					}
				} /* end zipfile finally */
			} /* end zipfile if */
		
		} /* end classpath loop */

	return null;
	}

/**
 * To prevent any classes from being gc'd before we are gc'd, we keep them
 * in this hashtable.  The contents of this table correspond to entries
 * in the VM-internal class entry pool.  This table only contains classes
 * for which we are the defining loader (ie, not classes for which we are
 * merely the initiating loader).
 */
private final Hashtable loadedClasses = new Hashtable();

/**
 * Similarly, we keep a reference to all native libraries loaded by this
 * loader.  When this class loader is GC'd, the native libraries become
 * candidates for finalization, which unlinks the shared library.
 */
private final Set loadedLibraries = new HashSet();

private ProtectionDomain defaultProtectionDomain;
private final ClassLoader parent;

protected ClassLoader() {
	this(getSystemClassLoader());
}

protected ClassLoader(ClassLoader parent) {
	System.getSecurityManager().checkCreateClassLoader();
	this.parent = parent;
}

public Class loadClass(String name) throws ClassNotFoundException {
	return (loadClass(name, false));
}

protected Class loadClass(String name, boolean resolve)
		throws ClassNotFoundException {
	Class c;

	// Search for class...
	search: {
		// First, see if already loaded by this class
		if ((c = findLoadedClass(name)) != null) {
			break search;
		}

		// Second, try the parent class loader
		try {
			if (parent != null) {
				c = parent.loadClass(name, resolve);
				break search;
			} else if (this != getSystemClassLoader()) {
				c = getSystemClassLoader()
					.loadClass(name, resolve);
				break search;
			}
		} catch (ClassNotFoundException e) {
		}

		// Third, try findClass()
		if ((c = findClass(name)) == null) {
			throw new ClassNotFoundException(name);
		}
	}

	// Now, optionally resolve the class
	if (resolve) {
		resolveClass(c);
	}
	return (c);
}

protected Class findClass(String name) throws ClassNotFoundException {
	throw new ClassNotFoundException(name);
}

/**
 * @deprecated
 */
protected final Class defineClass(byte data[], int off, int len)
		throws ClassFormatError {
	return (defineClass(null, data, off, len));
}

protected final Class defineClass(String name, byte data[], int off, int len)
		throws ClassFormatError {
	if (defaultProtectionDomain == null) {
		// XXX FIXME..
		defaultProtectionDomain = new ProtectionDomain(null, null);
	}
	return defineClass(name, data, off, len, defaultProtectionDomain);
}

private final Class defineClass(String name, byte data[], int off,
		int len, ProtectionDomain pd) throws ClassFormatError {
	if (off < 0 || len < 0 || off + len > data.length) {
		throw new IndexOutOfBoundsException();
	}
	Class clazz = defineClass0(name, data, off, len);
	if (name != null) {
		loadedClasses.put(name, clazz);
	}
	else {
		loadedClasses.put(clazz.getName(), clazz);
	}
	return (clazz);
}

protected final void resolveClass(Class c) {
	resolveClass0(c);
}

protected final Class findSystemClass(String name)
		throws ClassNotFoundException {
	return getSystemClassLoader().findClass(name);
}

public final ClassLoader getParent() {
	return parent;
}

protected final Class findLoadedClass(String name) {
	return findLoadedClass0(name);
}

protected Enumeration findResources(String name) throws IOException {
        return new Vector().elements();         // ie, an empty Enumeration
}

public final Enumeration getResources(String name) throws IOException {
        Vector v = new Vector();
        ClassLoader p;
        
        if (parent != null) {
                p = parent;
        } else if (this != getSystemClassLoader()) {
                p = getSystemClassLoader();
        } else {
                p = null;
        }
        if (p != null) {
                for (Enumeration e = p.getResources(name);
                    e.hasMoreElements(); )
                        v.addElement(e.nextElement());
        }
        for (Enumeration e = findResources(name); e.hasMoreElements(); )
                v.addElement(e.nextElement());
        return v.elements();
}

public static ClassLoader getSystemClassLoader() {
	return SystemClassLoader.getClassLoader();
}

protected String findLibrary(String libname) {
	return null;
}

synchronized void addNativeLibrary(NativeLibrary lib) {
	loadedLibraries.add(lib);
}

private native Class defineClass0(String name, byte data[], int off, int len);
private native Class findLoadedClass0(String name);
private native void resolveClass0(Class cls);

}
