/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.util;

import java.io.Serializable;

public class Date implements Serializable, Cloneable, Comparable {

	private static final long serialVersionUID = 7523967970034938905L;
	private long time;

public Date() {
	this(System.currentTimeMillis());
}

public Date(long date) {
	time = date;
}

public boolean after(Date when) {
	return (getTime() > when.getTime());
}

public boolean before(Date when) {
	return (getTime() < when.getTime());
}

public int compareTo(Object o) {
	return compareTo((Date)o);
}

public int compareTo(Date that) {
	final long thisTime = this.getTime();
	final long thatTime = that.getTime();

	return (thisTime == thatTime) ? 0 : (thisTime < thatTime) ? -1 : 1;
}

public boolean equals(Object obj) {
	return (obj instanceof Date) && getTime() == ((Date)obj).getTime();
}

public long getTime() {
	return (time);
}

public int hashCode() {
	return (int)getTime();
}

public void setTime(long time) {
	this.time = time;
}

public Object clone() {
	try {
		return super.clone();
	} catch (CloneNotSupportedException e) {
		return null;		// can't happen
	}
}
}
