package kaffe.io;

import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.lang.String;
import java.util.Hashtable;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;

import edu.utah.janosvm.resources.NameSpaceResource;

/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */
abstract public class CharToByteConverter
{
	private static Class thisClass = CharToByteConverter.class;
	protected char[] buf;
	protected int blen;
	private static String encodingRoot;
	private static String encodingDefault;
	private static final Class noConverter = Object.class;

	private static Hashtable getCache()
	{
		Team t = Team.current();
		Hashtable retval;

		if( (retval = (Hashtable)t.getClassLocal(thisClass)) == null )
		{
			retval = new Hashtable();
			t.setClassLocal(thisClass, retval);
		}
		return retval;
	}

static {
	// see explanation in ByteToCharConverter
	encodingRoot = "kaffe.io";
	encodingDefault = System.getProperty("file.encoding");
}

public CharToByteConverter() {
}

public Object clone() {
	try {
		return (super.clone());
	}
	catch (CloneNotSupportedException _) {
		return (null);
	}
}

public void carry ( char[] from, int fpos, int flen ) {
	int n;
	int m = blen + flen;

	if ( (from == buf) && (fpos == 0) ) // avoid recursive carry by flush()
		return;

	if ( buf == null ){
		n = (flen < 128) ? 128 : flen;
		buf = new char[n];
	}
	else {
		if ( m > buf.length ) {
			for ( n=buf.length*2; n<m; n *= 2);
			char[] newBuf = new char[n];
			System.arraycopy( buf, 0, newBuf, 0, blen);
			buf = newBuf;
		}
	}

	System.arraycopy( from, fpos, buf, blen, flen);
	blen = m;
}

abstract public int convert(char[] from, int fpos, int flen, byte[] to, int tpos, int tlen);

public int flush ( byte[] to, int tpos, int tlen ) {
	if ( blen == 0 ){
		return 0;
	}
	else {
		int oblen = blen;
		blen = 0;
		return (convert( buf, 0, oblen, to, tpos, tlen));
	}
}

private static CharToByteConverter getConverterInternal(final String enc)
{
	Hashtable cache = getCache();
	Class cls = (Class)cache.get(enc);
	if (cls == noConverter) {
		return (null);
	}
	try {
		if (cls == null) {
			cls = NameSpaceResource.loadTeamClass(
				encodingRoot +
				".CharToByte" +
				ConverterAlias.alias(enc));
			cache.put(enc, cls);
		}
		return ((CharToByteConverter)cls.newInstance());
	}
	catch (ClassNotFoundException _) {
	}
	catch (ClassCastException _) {
	}
	catch (InstantiationException _) {
	}
	catch (IllegalAccessException _) {
	}
	cache.put(enc, noConverter);
	return (null);
}

public static CharToByteConverter getConverter(String enc) throws UnsupportedEncodingException
{
	CharToByteConverter conv = getConverterInternal(enc);
	if (conv == null) {
		throw new UnsupportedEncodingException(enc);
	}
	return (conv);
}

public static CharToByteConverter getDefault() {
	CharToByteConverter conv = getConverterInternal(encodingDefault);
	if (conv == null) {
		conv = new CharToByteDefault();
	}
	return (conv);
}

abstract public int getNumberOfBytes ( char[] from, int fpos, int flen );
}
