/*
 * Message.java
 * Base class for simple messages that can be sent between threads.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.comm.amiga;

import edu.utah.janosvm.sys.ListNode;

/**
 * Base class for simple messages that can be sent between threads.
 */
public class Message
    extends ListNode
    implements Cloneable
{
    /**
     * The MsgPort that is waiting to receive a reply.
     */
    private MsgPort replyPort;

    /**
     * Indicates whether or not this is a rely message.
     */
    private boolean reply = false;
    
    public Message()
    {
    }

    /**
     * Set the reply port for this message.
     *
     * @param mp The port to expect replies on.
     */
    public void setReplyPort(MsgPort mp)
    {
	this.replyPort = mp;
    }

    /**
     * @return The reply port for this message.
     */
    public MsgPort getReplyPort()
    {
	return this.replyPort;
    }

    /**
     * @return Whether or not this message is a reply to another.
     */
    public boolean isReply()
    {
	return this.reply;
    }
    
    /**
     * Send this message back as a reply.
     */
    public void reply()
    {
	if( this.replyPort != null )
	{
	    this.reply = true;
	    this.replyPort.putMessage(this);
	}
	else
	{
	    throw new NoReplyPortError();
	}
    }
    
    public Object clone()
    {
	try
	{
	    Object retval = super.clone();
	    
	    return retval;
	}
	catch(CloneNotSupportedException e)
	{
	    throw new InternalError();
	}
    }
}
