/*
 * SignalHandler.java
 * Base class for handlers that can be added to a SignalDispatcher.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.comm.amiga;

import edu.utah.janosvm.sys.ListNode;

/**
 * A SignalHandler is a base class for handlers that will be called by a
 * SignalDispatcher.
 */
public abstract class SignalHandler
    extends ListNode
    implements Runnable
{
    /**
     * The signal number that this object will handle.
     */
    private int signalNumber = -1;

    /**
     * @return The signal number that this object will handle.
     */
    public int getSignalNumber()
    {
	return this.signalNumber;
    }

    /**
     * @param sn The signal number that this object will handle.
     */
    public void setSignalNumber(int sn)
    {
	this.signalNumber = sn;
    }

    public String toString()
    {
	return "SignalHandler[signalNumber="
	    + this.signalNumber
	    + "]";
    }
}
