/*
 * CommHashtable.java
 *
 * Copyright (c) 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package edu.utah.janosvm.kit.comm.commspace;

import java.util.Hashtable;

import edu.utah.janosvm.sys.Exportable;
import edu.utah.janosvm.sys.TemporaryExport;

/**
 * The CommHashtable wraps a regular Hashtable and makes it available for
 * export to other classes.
 */
class CommHashtable
    extends Exportable
    implements TemporaryExport
{
    /**
     * Hashtable to store pairs of CommSpaceElements.
     */
    private final Hashtable table = new Hashtable();

    /**
     * Construct an empty CommHashtable.
     */
    CommHashtable()
    {
    }

    /**
     * @param id The identifier to be copied and stored in the hash table.
     * @param value The value to be copied and stored in the hash table.
     * @return Null or the previous value paired with the given identifier.
     */
    CommSpaceElement put(CommSpaceElement id,
			 CommSpaceElement value)
    {
	CommSpaceElement retval;
		
	retval = (CommSpaceElement)this.table.
	    put(CommSpaceElement.copy(id),
		CommSpaceElement.copy(value));
	return retval;
    }

    /**
     * @param id The identifier to lookup in the table.
     * @return Null or the value paired with the given identifier.
     */
    CommSpaceElement lookup(CommSpaceElement id)
    {
	return (CommSpaceElement)this.table.get(id);
    }

    /**
     * @param id The identifier to remove from the hash table.
     */
    void remove(CommSpaceElement id)
    {
	this.table.remove(id);
    }

    /**
     * @param id The identifier to check for in the table.
     * @return True if the identifier was found in the table, false
     * otherwise.
     */
    boolean containsKey(CommSpaceElement id)
    {
	return this.table.containsKey(id);
    }

    public boolean expired()
    {
	return true;
    }
}
