/*
 * CommHashtableHandle.java
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package edu.utah.janosvm.kit.comm.commspace;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.Importable;
import edu.utah.janosvm.sys.Exportable;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;

/**
 * A handle for a CommHashtable that may be in another Flow.
 */
public class CommHashtableHandle
	extends Importable
{
	private CommHashtable ch;

	/**
	 * Construct a new CommHashtable and attach this handle to it.
	 */
	public CommHashtableHandle()
	{
		Team.current().
			exportAnonymousObject(new CommHashtable()).
			addImporter(this);
	}

	/**
	 * Copy constructor.
	 *
	 * @param chh The handle to copy.
	 */
	public CommHashtableHandle(CommHashtableHandle chh)
	{
		this.copy(chh);
	}

	protected void bindTo(Exportable ex)
	{
		this.ch = (CommHashtable)ex;
	}

	protected void unbindFrom()
	{
		this.ch = null;
	}
	
	/**
	 * @param id Identifier to be paired with a value
	 * @param val Value to pair with an identifier
	 * @return The element that was previously paired with the identifier.
	 */
	public CommSpaceElement put(final CommSpaceElement id,
				    final CommSpaceElement val)
	{
		CommSpaceElement retval = null;
		TeamReservation ta;
		
		try
		{
			ta = this.ch.pushOwnerReservation();
			ta.visit();
			{
				retval = this.ch.put(id, val);
			}
			ta.leave();
			retval = (CommSpaceElement)retval.clone();
			ta.pop();
		}
		catch(TeamEjectionException dte)
		{
		}
		catch(DeadTeamException dte)
		{
		}
		return retval;
	}

	/**
	 * @param id The identifier to lookup in the hash table.
	 * @return Null or the element that is paired with the given identifier
	 */
	public CommSpaceElement lookup(final CommSpaceElement id)
	{
		CommSpaceElement retval = null;
		TeamReservation ta;
		
		try
		{
			ta = this.ch.pushOwnerReservation();
			ta.visit();
			{
				retval = this.ch.lookup(id);
			}
			ta.leave();
			retval = (CommSpaceElement)retval.clone();
			ta.pop();
		}
		catch(TeamEjectionException dte)
		{
		}
		catch(DeadTeamException dte)
		{
		}
		return retval;
	}

	/**
	 * @param id The identifier to remove from the hash table.
	 */
	public void remove(final CommSpaceElement id)
	{
		try
		{
			this.ch.switchToOwner();
			{
				this.ch.remove(id);
			}
			this.ch.returnFromOwner();
		}
		catch(TeamEjectionException dte)
		{
		}
		catch(DeadTeamException dte)
		{
		}
	}

	/**
	 * @param id The identifier to check for in the hash table.
	 * @return True if the identifier was found in the table, false
	 * otherwise.
	 */
	public boolean containsKey(final CommSpaceElement id)
	{
		boolean retval = false;
		TeamReservation ta;
		
		try
		{
			ta = this.ch.pushOwnerReservation();
			ta.visit();
			{
				retval = this.ch.containsKey(id);
			}
			ta.leave();
			ta.pop();
		}
		catch(TeamEjectionException dte)
		{
		}
		catch(DeadTeamException dte)
		{
		}
		return retval;
	}
}
