/*
 * CommQueue.java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package edu.utah.janosvm.kit.comm.commspace;

import java.util.Vector; // XXX

import edu.utah.janosvm.sys.Exportable;
import edu.utah.janosvm.sys.TemporaryExport;

/**
 * A shareable FIFO queue of CommSpaceElements.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
final class CommQueue
    extends Exportable
    implements TemporaryExport
{
    // Sorry about the inefficient implementation:
    private final Vector elements = new Vector();

    /**
     * @param elem The element to copy and enqueue.
     */
    public void enqueue(CommSpaceElement elem)
    {
	synchronized(this.elements)
	{
	    this.elements.addElement(elem.clone());
	    this.elements.notify();
	}
    }

    /**
     * @param elems The array of elements to copy and atomically add to the
     * queue.
     */
    public void enqueue(CommSpaceElement elems[])
    {
	synchronized(this.elements)
	{
	    int lpc;

	    for( lpc = 0; lpc < elems.length; lpc++ )
	    {
		this.elements.addElement(elems[lpc].clone());
	    }
	    this.elements.notify();
	}
    }

    /**
     * Wait to take an element from the queue.
     * <p>
     * May throw InterruptedException if the wait() is
     * interrupted.  The caller will have to restart the
     * waitToTake().
     *
     * @return The next element on the queue.
     */
    public CommSpaceElement waitToTake()
	throws InterruptedException
    {
	synchronized(elements)
	{
	    while (elements.size() == 0)
		elements.wait();
	    CommSpaceElement elem = (CommSpaceElement)
		elements.firstElement();
	    elements.removeElementAt(0);

	    return elem;
	}
    }

    public boolean expired()
    {
	return true;
    }
}
