/*
 * CommQueuePuller.java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package edu.utah.janosvm.kit.comm.commspace;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.Importable;
import edu.utah.janosvm.sys.Exportable;

/**
 * A handle for the owner of the CommQueue to pull elements out of it.
 */
public final class CommQueuePuller
	extends Importable
{

	private CommQueue cq;
	
	/**
	 * Construct CommQueuePuller handle and a CommQueue for it to pull
	 * CommSpaceElements from.
	 */
	public CommQueuePuller()
	{
		Team.current().
			exportAnonymousObject(new CommQueue()).
			addImporter(this);
	}

	protected void bindTo(Exportable ex)
	{
		this.cq = (CommQueue)ex;
	}

	protected void unbindFrom()
	{
		this.cq = null;
	}

	/**
	 * Take an element from the queue, or wait for one to be added and
	 * return that.
	 *
	 * @return The next element on the queue.
	 */
	public CommSpaceElement waitToTake()
		throws InterruptedException
	{
		CommSpaceElement retval = null;

		if( this.cq != null )
		{
			retval = this.cq.waitToTake();
		}
		return retval;
	}
	
}
