/*
 * CommQueuePusher.java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package edu.utah.janosvm.kit.comm.commspace;

import edu.utah.janosvm.sys.Importable;
import edu.utah.janosvm.sys.Exportable;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;

/**
 * A handle for other flows to push elements onto a CommQueue.
 */
public final class CommQueuePusher
	extends Importable
{

	private CommQueue cq;
	
	/**
	 * Construct an empty pusher, it can be bound to a queue later.
	 */
	public CommQueuePusher()
	{
	}

	/**
	 * Construct a handle that will push elements onto the queue managed
	 * by the puller.
	 *
	 * @param puller The handle for the CommQueue we want to push elements
	 * onto.
	 */
	public CommQueuePusher(CommQueuePuller puller)
	{
		this.copy(puller);
	}

	/**
	 * @param pusher The handle for the CommQueue we want to push elements
	 * onto.
	 */
	public CommQueuePusher(CommQueuePusher pusher)
	{
		this.copy(pusher);
	}

	protected void bindTo(Exportable ex)
	{
		this.cq = (CommQueue)ex;
	}

	protected void unbindFrom()
	{
		this.cq = null;
	}

	/**
	 * Bind this pusher to the CommQueue referred to by the argument.
	 *
	 * @param pusher The handle to the CommQueue we should bind ourselves
	 * too.
	 */
	public void bind(CommQueuePusher pusher)
	{
		this.copy(pusher);
	}

	/**
	 * @param cse The element to enqueue.
	 */
	public void enqueue(final CommSpaceElement cse)
	{
		try
		{
			this.cq.switchToOwner();
			{
				this.cq.enqueue(cse);
			}
			this.cq.returnFromOwner();
		}
		catch(TeamEjectionException dte)
		{
		}
		catch(DeadTeamException dte)
		{
		}
	}

	/**
	 * @param cse The array of elements to atomically add to the queue.
	 */
	public void enqueue(final CommSpaceElement cse[])
	{
		try
		{
			this.cq.switchToOwner();
			{
				this.cq.enqueue(cse);
			}
			this.cq.returnFromOwner();
		}
		catch(TeamEjectionException dte)
		{
		}
		catch(DeadTeamException dte)
		{
		}
	}
}
