/*
 * JSIChannel.java
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.comm.jsi;

import java.io.IOException;

/**
 * A communication channel for sending/receiving JSI messages.
 */
public interface JSIChannel
{
    /**
     * Disconnect the channel from its peer.
     */
    void disconnect();

    /**
     * Send a message to the peer.
     *
     * @param msg A flattened JSI message.
     * @exception Throws an IOException if something went wrong.
     */
    void sendMessage(byte msg[])
	throws IOException;

    /**
     * Send a reply message to the peer.
     *
     * @param msg A flattened JSI message.
     * @exception Throws an IOException if something went wrong.
     */
    void replyMessage(byte msg[])
	throws IOException;

    /**
     * Receive a flattened message from the peer.
     *
     * @return The flattened message or null if the channel is no longer
     * connected.
     * @exception Throws an IOException if something went wrong.
     */
    byte[] recvMessage()
	throws IOException;
}
