/*
 * JSIHandler.java
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.comm.jsi;

/**
 * Interface for objects that can handle JSI messages.
 */
public interface JSIHandler
{
    /**
     * Resolve the next specifier on the stack.
     *
     * @param jm The JSI message to route.
     * @param jp The specifier pair currently being resolved.
     * @return The Object that can handle the given message or null if there
     * is none.
     */
    JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
	throws Throwable;

    /**
     * Process the given message.
     *
     * @param jhc The context in which to send/process interim messages.
     * @param jm The received message.  When processing is done the message
     * should be cleared and any reply data placed inside it.
     * @exception The handler can throw any of a number of exceptions/erros.
     */
    void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
	throws Throwable;

    /**
     * Add any suites for this handler to the given message.
     *
     * @param jm The message containing all supported suites.
     */
    void getSuites(JSIMessage jm);
}
