/*
 * TCPProvider.java
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.comm.jsi;

import java.io.IOException;

import java.net.Socket;
import java.net.ServerSocket;

/**
 * A straight forward implementation of a JSIProvider running over TCP.
 */
public final class TCPProvider
    implements JSIProvider
{
    private ServerSocket ss;

    /**
     * Construct a TCPProvider thats listening on the given port.
     *
     * @param port The port to listen for connections.
     */
    public TCPProvider(int port)
	throws IOException
    {
	this.ss = new ServerSocket(port);
    }

    public void detach()
    {
	try
	{
	    this.ss.close();
	}
	catch(IOException e)
	{
	}
	this.ss = null;
    }

    public void handle(JSIConnectionHandler jch)
    {
	try
	{
	    boolean done = false;
	    Socket socket;
	    
	    while( !done )
	    {
		socket = this.ss.accept();
		jch.handle(this, new TCPChannel(socket));
	    }
	}
	catch(IOException e)
	{
	}
    }

    public String toString()
    {
	return "TCPProvider[ss="
	    + this.ss
	    + "]";
    }
}
