/*
 * IsolateManager.java
 * An implementation of the JSR-121 straw man classes.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.isolate;

import java.util.Hashtable;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;

/**
 * The IsolateManager class manages the global table for mapping the Isolate's
 * unique identifier to their TeamHandle.
 */
class IsolateManager
{
    /**
     * Reference to the java Class object to be used with
     * Team.setClassLocal/getClassLocal.
     */
    private static Class thisClass = IsolateManager.class;

    /**
     * Mapping from the Isolate's unique Identifier to the TeamHandle object.
     */
    private static Hashtable uniqueIDtoTeam = new Hashtable();

    /**
     * Pointer to the team that actually holds the mapping data.
     */
    private TeamHandle managerTeam;

    /**
     * @return The IsolateManager object for the current team.
     */
    public static IsolateManager getCurrentIsolateManager()
    {
	return (IsolateManager)Team.getCurrentClassLocal(thisClass);
    }

    /**
     * Initialize and install an IsolateManager in the current team.
     *
     * @param manager The manager TeamHandle.
     * @return The IsolateManager associated with the current team.
     */
    public static IsolateManager initIsolateManager(TeamHandle manager)
    {
	IsolateManager retval = new IsolateManager(manager);

	Team.setCurrentClassLocal(thisClass, retval);
	return retval;
    }

    /**
     * @param managerTeam Reference to the manager team.
     */
    public IsolateManager(TeamHandle managerTeam)
    {
	this.managerTeam = managerTeam;
    }

    /**
     * @return The reference to the manager team.
     */
    public TeamHandle getTeamHandle()
    {
	return this.managerTeam;
    }

    /**
     * Add a mapping to the table.
     *
     * @param uniqueID The unique identifier associated with the Isolate.
     * @param th The reference to the team associated with the Isolate.
     */
    public void addMapping(String uniqueID, TeamHandle th)
    {
	uniqueIDtoTeam.put(uniqueID, th);
    }

    /**
     * @return The Hashtable containing the mappings.
     */
    public Hashtable getMappings()
    {
	return uniqueIDtoTeam;
    }
}
