/*
 * IsolateTeamList.java
 * A TeamList listener for Isolate objects.
 * 
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.isolate;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamList;
import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.TeamListHandler;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.NoSuchExportException;

class IsolateTeamList
    implements TeamListHandler
{
    /**
     * Our connection to the global team list.
     */
    private TeamList tl;
    
    /**
     * The Isolate object paired with this object.
     */
    private Isolate isolate;

    /**
     * Construct a listener for the given Isolate object.
     *
     * @param isolate The Isolate object to pair with this object.
     */
    public IsolateTeamList(Isolate isolate)
    {
	TeamHandle kernel = Team.current().getKernelTeam();

	this.isolate = isolate;
	this.tl = new TeamList();
	this.tl.setTeamListHandler(this);
	this.tl.listen();
    }

    public synchronized void event(TeamReservation tr, TeamEvent ev)
    {
	try
	{
	    tr.visit();
	    {
		/*
		 * Make sure the event is for the Isolate we're paired with
		 * and then update its state accordingly.
		 */
		switch( ev.getEvent() )
		{
		case TeamEvent.FINISHED:
		    if( ev.getID() == this.isolate.getID() )
			this.isolate.setState(Isolate.STATE_FINISHED);
		    break;
		case TeamEvent.TERMINATED:
		    if( ev.getID() == this.isolate.getID() )
			this.isolate.setState(Isolate.STATE_DYING);
		    break;
		case TeamEvent.BURIED:
		    if( ev.getID() == this.isolate.getID() )
			this.isolate.setState(Isolate.STATE_DEAD);
		    break;
		}
	    }
	    tr.leave();
	}
	catch(TeamEjectionException e)
	{
	}
    }

    public String toString()
    {
	return "IsolateTeamList["
	    + super.toString()
	    + "; isolate="
	    + this.isolate
	    + "]";
    }
}
