/*
 * TeamListMessage.java
 * A message class that stores the values from a TeamEvent.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.sys.TeamEvent;

import edu.utah.janosvm.kit.comm.amiga.Message;

/**
 * A message class that stores the values from a TeamEvent.
 */
public class TeamListMessage
    extends Message
{
    /**
     * The name of the team this event relates too.
     */
    private String name;

    /**
     * The unique identifier of the team this event relates too.
     */
    private int id;

    /**
     * The TeamEvent event number.
     */
    private int event;

    /**
     * Construct a TeamListMessage with the same values as the given TeamEvent.
     *
     * @param ev The TeamEvent object to copy values from.
     */
    TeamListMessage(TeamEvent ev)
    {
	if( ev.getName() != null )
	    this.name = CloneFactory.clone(ev.getName());
	this.id = ev.getID();
	this.event = ev.getEvent();
    }

    /**
     * @return The name of the team this event relates too.
     */
    public String getName()
    {
	return this.name;
    }

    /**
     * @return The unique identifier this event relates too.
     */
    public int getID()
    {
	return this.id;
    }

    /**
     * @return The TeamEvent event number.
     */
    public int getEvent()
    {
	return this.event;
    }

    public String toString()
    {
	return "TeamListMessage[name="
	    + this.name
	    + "; id="
	    + this.id
	    + "; event="
	    + this.event
	    + "]";
    }
}
