/*
 * TeamListMessenger.java
 * A TeamList implementation that sends TeamListMessages to a MsgPort.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamList;
import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.TeamListHandler;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.NoSuchExportException;

import edu.utah.janosvm.kit.comm.amiga.MsgPort;

/**
 * A TeamList implementation that sends TeamListMessages to a MsgPort.
 * Since the `event' call back happens on kernel time we don't want to waste
 * time so we just visit the other team, construct a message and send it off
 * to someone who can handle it better.
 */
public class TeamListMessenger
    implements TeamListHandler
{
    /**
     * Our connection to the global team list.
     */
    private TeamList tl;
    
    /**
     * The port we should send TeamListMessage's too.
     */
    private MsgPort mp;

    /**
     * Construct a messenger that will send TeamListMessage's to the given
     * MsgPort.
     *
     * @param mp A MsgPort that is willing to receive and process
     * TeamListMessage's
     */
    public TeamListMessenger(MsgPort mp)
    {
	TeamHandle kernel = Team.current().getKernelTeam();

	this.mp = mp;
	this.tl = new TeamList();
	this.tl.setTeamListHandler(this);
	this.tl.listen();
    }
    
    public void event(TeamReservation tr, TeamEvent ev)
    {
	try
	{
	    /*
	     * We're in the kernel so visit our team to create and send the
	     * message.
	     */
	    tr.visit();
	    {
		TeamListMessage tlm = new TeamListMessage(ev);

		this.mp.putMessage(tlm);
	    }
	    tr.leave();
	}
	catch(TeamEjectionException e)
	{
	}
    }
}
