/*
 * MemoryFile.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.server.memfs;

import java.io.IOException;
import java.io.FileNotFoundException;

import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;

/**
 * The MemoryFileSystem version of the java.io.File class.
 */
public final class MemoryFile
{
    /**
     * The handle on the memory block in the server team.
     */
    private final MemoryBlock mb;

    /**
     * Construct a MemoryFile object that refers to the given file.
     *
     * @param fileName The name of the file in the MemoryFileSystem.
     * @exception Throws a FileNotFoundException if the file couldn't be
     * found or the server team was unreachable.
     */
    public MemoryFile(String fileName)
	throws FileNotFoundException
    {
	this.mb = new MemoryBlock(fileName);
	this.mb.connect(false);
    }

    /**
     * Delete the file.
     *
     * @return True if the file was deleted.
     * @exception Throws an IOException if the file couldn't be found or the
     * server team was unreachable.
     */
    public boolean delete()
	throws IOException
    {
	boolean retval = false;

	do {
	    try
	    {
		retval = this.mb.delete();
		return retval;
	    }
	    catch(DeadTeamException e)
	    {
	    }
	    catch(TeamEjectionException e)
	    {
	    }
	    try
	    {
		this.mb.connect(false);
	    }
	    catch(FileNotFoundException fnfe)
	    {
		throw new IOException();
	    }
	} while( true );
    }
    
    public String toString()
    {
	return "MemoryFile[mb="
	    + this.mb
	    + "]";
    }
}
