/*
 * MemoryFileInputStream.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.server.memfs;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileNotFoundException;

import edu.utah.janosvm.sys.DeadTeamException;

/**
 * An InputStream for a MemoryFileSystem file.
 */
public class MemoryFileInputStream
    extends InputStream
{
    /**
     * The handle on the memory block in the server team.
     */
    private final MemoryBlock mb;

    /**
     * The current location in the file.
     */
    private int cursor;

    /**
     * Construct a MemoryFileInputStream that reads from the given file.
     *
     * @param fileName The name of the file in the MemoryFileSystem.
     * @exception Throws a FileNotFoundException if the file couldn't be
     * found or the server team was unreachable.
     */
    public MemoryFileInputStream(String fileName)
	throws FileNotFoundException
    {
	/* Create and connect the import */
	this.mb = new MemoryBlock(fileName);
	this.mb.connect(false);
    }

    public synchronized int read(byte buffer[], int offset, int length)
	throws IOException
    {
	do {
	    try
	    {
		int retval = 0;
		
		retval = this.mb.read(this.cursor, buffer, offset, length);
		this.cursor += retval;
		return( retval );
	    }
	    catch(DeadTeamException e)
	    {
		/* The server is unreachable, try to reconnect */
		try
		{
		    this.mb.connect(false);
		}
		catch(FileNotFoundException fnfe)
		{
		    throw new IOException();
		}
	    }
	} while( true );
    }

    public int read()
	throws IOException
    {
	byte buffer[] = new byte[1];
	int retval;

	if( this.read(buffer) == 1 )
	    retval = buffer[0] & 0xff;
	else
	    retval = -1;
	return retval;
    }

    public String toString()
    {
	return "MemoryFileInputStream[mb="
	    + this.mb
	    + "; cursor="
	    + this.cursor
	    + "]";
    }
}
