/*
 * Association.java
 * Associate an object with a unique identifier.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.util;

class Association
{
    /**
     * The identifier to associate with an object.
     */
    private Object name;

    /**
     * The object to associate with the above identifier.
     */
    private Object obj;

    /**
     * Construct an empty Association to be filled in later.
     */
    public Association()
    {
    }
    
    /**
     * Construct an Association with the given values.
     *
     * @param name The identifier to associate with an object.
     * @param obj The object to associate with the above identifier.
     */
    public Association(Object name, Object obj)
    {
	this.name = name;
	this.obj = obj;
    }

    /**
     * @param name The value for this.name.
     */
    public void setName(String name)
    {
	this.name = name;
    }

    /**
     * @return The value of this.name.
     */
    public Object getName()
    {
	return this.name;
    }

    /**
     * @param obj The value for this.obj.
     */
    public void setObject(Object obj)
    {
	this.obj = obj;
    }
	
    /**
     * @return The value of this.obj.
     */
    public Object getObject()
    {
	return this.obj;
    }

    /**
     * @return The hashCode of this.name.
     */
    public int hashCode()
    {
	return this.name.hashCode();
    }

    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof Association )
	{
	    Association e = (Association)obj;

	    retval = this.name.equals(e.name);
	}
	return retval;
    }
	
    public String toString()
    {
	return "Association[name="
	    + this.name
	    + "; obj="
	    + this.obj
	    + "]";
    }
}
