/*
 * SharedTableReader.java
 * Provides read-only access to a shared table.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.util;

import java.util.Vector;
import java.util.Enumeration;

/**
 * Provides read-only access to a shared table.
 */
public class SharedTableReader
{
    private SharedTable table;

    /**
     * Construct a SharedTableReader.  Restricted to package access since
     * it is only constructed by SharedTableWriters.
     */
    SharedTableReader()
    {
    }

    void setTable(SharedTable table)
    {
	this.table = table;
    }
    
    /**
     * Get an object out of the table.
     *
     * @param name The identifier associated with the object.
     * @return The object associated with the given identifier.
     */
    public Object get(Object name)
    {
	return this.table.get(name);
    }

    private static class KeyWalker
	implements SharedTableWalker
    {
	private final Vector keys = new Vector();
	
	public boolean walk(Association as)
	{
	    this.keys.addElement(as.getName());
	    return false;
	}

	public Vector getKeys()
	{
	    return this.keys;
	}
    }
    
    /**
     * @return An enumeration containing the key values in the table.
     */
    public Enumeration keys()
    {
	KeyWalker kw = new KeyWalker();
	Enumeration retval;

	this.table.walk(null, kw);
	retval = kw.getKeys().elements();
	return retval;
    }
    
    private static class ElementWalker
	implements SharedTableWalker
    {
	private final Vector keys = new Vector();
	
	public boolean walk(Association as)
	{
	    this.keys.addElement(as.getObject());
	    return false;
	}

	public Vector getElements()
	{
	    return this.keys;
	}
    }
    
    /**
     * @return An enumeration containing the key values in the table.
     */
    public Enumeration elements()
    {
	ElementWalker kw = new ElementWalker();
	Enumeration retval;

	this.table.walk(null, kw);
	retval = kw.getElements().elements();
	return retval;
    }
}
