/*
 * SparseArrayElement.java
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.util;

/**
 * The elements that make up the sparse array.
 */
public class SparseArrayElement
{
    private static final SparseArrayElement IS_ALLOCATED =
	new SparseArrayElement();
    
    /**
     * The next element in the list.
     */
    private SparseArrayElement next;

    /**
     * The elements unique id.
     */
    private long id;

    protected SparseArrayElement()
    {
    }

    /**
     * @param next The next element in the list.
     */
    void setNext(SparseArrayElement next)
    {
	this.next = next;
    }

    /**
     * @return The value of next.
     */
    SparseArrayElement getNext()
    {
	return this.next;
    }

    /**
     * @param id The unique identifier.
     */
    void setID(long id)
    {
	this.id = id;
    }

    /**
     * @return The unique identifier.
     */
    public long getID()
    {
	return this.id;
    }

    /**
     * Obtain this element for use.
     */
    void obtain()
    {
	this.next = IS_ALLOCATED;
	this.id &= ~(1L << (SparseArray.DEFAULT_ID_BITS - 1));
    }

    /**
     * Release this element from use.
     *
     * @param timeOffset The offset of the time bits in the unique identifier.
     */
    void release(int timeOffset)
    {
	this.next = null;
	this.id |= (1L << (SparseArray.DEFAULT_ID_BITS - 1));
	this.id += (1L << timeOffset);
	this.dropped();
    }

    /**
     * Method called when the element is returned to the array.
     */
    protected void dropped()
    {
    }

    /**
     * @return True if the element is allocated.
     */
    boolean isAllocated()
    {
	return (this.next == IS_ALLOCATED);
    }

    public int hashCode()
    {
	return (int)this.id;
    }

    public boolean equals(Object obj)
    {
	return (obj == this);
    }

    public String toString()
    {
	return "SparseArrayElement[next="
	    + ((this.next != null) ? this.next.id : 0)
	    + "; id="
	    + this.id
	    + "]";
    }
}
