#
# Copyright (c) 2001 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

import string

from jsi.message import *
from jsi.channel import *
from jsi.messageError import *
from socket import *
from teams import *
from team import *
from xprofiler import *
from classpath import *
from nativepath import *
from jvmfiles import *
from jvmfile import *
from proxies import *
from proxy import *

class JanosVM:
    def __init__(self, chan):
        self.channel = chan
        self.msg = Message()

    def disconnect(self):
        self.channel.disconnect()

    def send(self):
        self.msg.reverseSpecifiers()
        what = self.msg.getWhat()
        buf = self.msg.flatten()
        self.msg.clear()
        self.channel.send(buf)
        if what == JSI_SHUTDOWN:
            return
        else:
            buf = self.channel.recv()
            reply = Message()
            reply.unflatten(buf)
            if reply.getWhat() == JSI_ERROR:
                raise MessageError(reply)
            else:
                return reply

    def sendMessage(self, req):
        what = req.getWhat()
        buf = req.flatten()
        self.channel.send(buf)
        if what == JSI_SHUTDOWN:
            return
        else:
            buf = self.channel.recv()
            reply = Message()
            reply.unflatten(buf)
            if reply.getWhat() == JSI_ERROR:
                raise MessageError(reply)
            else:
                return reply

    def unstall(self):
        self.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.msg.pushSpecifier("continue")
        self.send()

    def getIdentity(self):
        self.msg.setWhat(JSI_GET_PROPERTY)
        self.msg.pushSpecifier("identity");
        reply = self.send()
        lpc = 0
        count = reply.identifierCount("result")
        retval = ""
        while lpc < count:
            retval = retval + "\n" + reply.find("result", lpc)
            lpc = lpc + 1
        return retval
        
    def shutdown(self):
        self.msg.setWhat(JSI_SHUTDOWN)
        self.send()
        
    def xprofiler(self):
        return XProfiler(self)

    def classpath(self):
        return ClassPath(self)

    def nativepath(self):
        return NativePath(self)

    def teams(self):
        return Teams(self)

    def team(self, tid):
        return Team(self, tid)

    def files(self):
        return JVMFiles(self)

    def file(self, name):
        return JVMFile(self, name)

    def proxies(self):
        return Proxies(self)

    def proxy(self, pid):
        return Proxy(self, pid)

    def out(self, data):
        self.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.msg.pushSpecifier("out")
        self.msg.pushSpecifier("write")
        req = copy.deepcopy(self.msg)
        req.reverseSpecifiers()
        self.msg.clear()
        offset = 0
        stride = 2048
        size = len(data)
        while offset < size:
            req.add("data", data[offset:min(size, offset + stride)])
            reply = self.sendMessage(req)
            offset = offset + string.atoi(reply.find("actual"))
            req.remove("data")

    def err(self, data):
        self.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.msg.pushSpecifier("err")
        self.msg.pushSpecifier("write")
        req = copy.deepcopy(self.msg)
        req.reverseSpecifiers()
        self.msg.clear()
        offset = 0
        stride = 2048
        size = len(data)
        while offset < size:
            req.add("data", data[offset:min(size, offset + stride)])
            reply = self.sendMessage(req)
            offset = offset + string.atoi(reply.find("actual"))
            req.remove("data")
