#
# Copyright (c) 2001 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

from janosvm import *
from math import *
import copy

class JVMFile:
    def __init__(self, jvm, name):
        self.jvm = jvm
        self.jvm.msg.pushSpecifier("file", name)

    def size(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("size")
        reply = self.jvm.send()
        return string.atoi(reply.find("result"))

    def read(self):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("read")
        req = copy.deepcopy(self.jvm.msg)
        req.reverseSpecifiers()
        self.jvm.msg.clear()
        retval = ""
        offset = 0
        size = 2048
        actual = 2048
        req.add("size", size)
        while size == actual:
            req.add("offset", offset)
            try:
                reply = self.jvm.sendMessage(req)
                actual = string.atoi(reply.find("actual"))
                retval = retval + reply.find("data")[0:actual]
                req.remove("offset")
                offset = offset + actual
            except MessageError, e:
                if e.msg.find("why") == "Seek error":
                    actual = 0
                else:
                    raise
        return retval

    def write(self, data):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("write")
        req = copy.deepcopy(self.jvm.msg)
        req.reverseSpecifiers()
        self.jvm.msg.clear()
        offset = 0
        stride = 2048
        size = len(data)
        while offset < size:
            req.add("offset", offset)
            req.add("data", data[offset:min(size, offset + stride)])
            reply = self.jvm.sendMessage(req)
            offset = offset + string.atoi(reply.find("actual"))
            req.remove("data")
            req.remove("offset")

    def delete(self):
        self.jvm.msg.setWhat(JSI_DELETE_PROPERTY)
        reply = self.jvm.send()
        return reply.find("result")
