#
# Copyright (c) 2001 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#
from janosvm import *

class Team:
    def __init__(self, jvm, tid):
        self.jvm = jvm
        self.jvm.msg.pushSpecifier("team", tid)

    def getName(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("name")
        reply = self.jvm.send()
        return reply.find("result")
        
    def setDescription(self, desc):
        self.jvm.msg.setWhat(JSI_SET_PROPERTY)
        self.jvm.msg.pushSpecifier("description")
        self.jvm.msg.add("data", desc)
        self.jvm.send()

    def getDescription(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("description")
        reply = self.jvm.send()
        return reply.find("result")

    def gc(self):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("gc");
        self.jvm.send()
        
    def getGCStats(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("gc");
        reply = self.jvm.send()
        retval = {}
        ids = reply.getIdentifiers();
        for i in ids:
            retval[i[0]] = i[1];
        return retval
    
    def run(self, className, args = ""):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("run")
        self.jvm.msg.add("class", className)
        if args != "":
            self.jvm.msg.add("args", args)
        reply = self.jvm.send()
        return reply.find("result")

    def addDependency(self, tid):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("addDependency")
        self.jvm.msg.add("team", tid)
        self.jvm.send()
        
    def delete(self):
        self.jvm.msg.setWhat(JSI_DELETE_PROPERTY)
        try:
            reply = self.jvm.send()
        except MessageError:
            pass

    def waitForThreads(self):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("waitForThreads")
        self.jvm.send()

    def wait(self):
        self.jvm.msg.setWhat(JSI_EXECUTE_PROPERTY)
        self.jvm.msg.pushSpecifier("wait")
        self.jvm.send()
