#
# Copyright (c) 2001 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

from janosvm import *

class Teams:
    def __init__(self, jvm):
        self.jvm = jvm
        self.jvm.msg.pushSpecifier("teams")

    def create(self,
               name = "",
               memory = 0,
               stdin = "",
               stdout = "",
               stderr = "",
               configs = [],
               namespace = [],
               classpath = [],
               nativepath = []):
        "Create a team with the given name"
        self.jvm.msg.setWhat(JSI_CREATE_PROPERTY)
        
        if name != "":
            self.jvm.msg.add("name", name)

        if stdin != "":
            self.jvm.msg.add("stdin", stdin)
        if stdout != "":
            self.jvm.msg.add("stdout", stdout)
        if stderr != "":
            self.jvm.msg.add("stderr", stderr)

        if memory != 0:
            self.jvm.msg.add("memory", memory)

        for cfg in configs:
            self.jvm.msg.add("config", cfg)
        
        lpc = 0
        while lpc < len(namespace):
            self.jvm.msg.add("namespace", namespace[lpc])
            lpc = lpc + 1
        
        lpc = 0
        while lpc < len(classpath):
            self.jvm.msg.add("classpath", classpath[lpc])
            lpc = lpc + 1
        
        lpc = 0
        while lpc < len(nativepath):
            self.jvm.msg.add("nativepath", nativepath[lpc])
            lpc = lpc + 1
        
        reply = self.jvm.send()
        return string.atoi(reply.find("result"))

    def list(self):
        self.jvm.msg.setWhat(JSI_LIST_PROPERTIES)
        reply = self.jvm.send()
        count = reply.identifierCount("id")
        lpc = 0
        retval = []
        while lpc < count:
            retval.append((reply.find("id", lpc),
                           reply.find("name", lpc),
                           reply.find("description", lpc)))
            lpc = lpc + 1
        return retval

    def find(self, name):
        for team in self.list():
            if team[1] == name:
                return string.atoi(team[0])
        return 0
