#
# Copyright (c) 2001 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

from socket import *
from struct import *
from string import *

from registry import *

class Channel:
    def __init__(self, host, port, protocol = "tcp"):
        self.connected = 0
        self.myHost = host
        self.myPort = port
        if protocol == "tcp":
            self.mySock = socket(AF_INET, SOCK_STREAM)
        else:
            self.mySock = socket(AF_INET, SOCK_DGRAM)

    def connect(self):
        if self.connected == 0:
            self.mySock.connect((self.myHost, self.myPort))
            self.connected = 1

    def disconnect(self):
        if self.connected == 1:
            self.mySock.close()
            self.connected = 0

    def recv(self):
        buf = self.mySock.recv(4)
        if len(buf) != 4:
            raise IOError()
        else:
            msgSize, = unpack("!i", buf)
            msgSize = msgSize - 4
            payload = ""
            while msgSize > 0:
                payload = payload + self.mySock.recv(msgSize)
                msgSize = msgSize - len(payload)
            buf = buf + payload
            return buf

    def send(self, buf):
        self.mySock.send(buf)

def jsiConnectChannelTo(dest, defaultProtocol = "tcp"):
    try:
        dest = Registry().findExport(dest, "jsi.id")
    except IOError, e:
        pass
    protoPath = split(dest, ":", 1)
    if len(protoPath) == 1:
        proto = defaultProtocol
    else:
        proto = protoPath[0]
    path = split(protoPath[1], "/")
    if proto == "tcp":
        retval = Channel(path[0], atoi(path[1]), proto)
        retval.connect()
        return retval
    else:
        return
