#
# Copyright (c) 2001 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

from struct import *
from socket import htonl
from socket import ntohl

JSI_WHAT_NONE = 0
JSI_ERROR = 1
JSI_SHUTDOWN = 2
JSI_CARRIER = 3
JSI_REPLY = 4
JSI_GET_SUITES = 5
JSI_LIST_PROPERTIES = 6
JSI_GET_PROPERTY = 7
JSI_SET_PROPERTY = 8
JSI_CREATE_PROPERTY = 9
JSI_DELETE_PROPERTY = 10
JSI_EXECUTE_PROPERTY = 11

headerFormat = "!4i"

class Message:
    def __init__(self):
        self.clear()

    def clear(self):
        self.what = JSI_ERROR
        self.specifiers = []
        self.specifierCount = 0
        self.identifiers = []
        
    def setWhat(self, value):
        self.what = value

    def getWhat(self):
        return self.what
    
    def add(self, name, data):
        self.identifiers.append((name, str(data)))

    def remove(self, name, instance = 0):
        lpc = 0
        while lpc < len(self.identifiers):
            if self.identifiers[lpc][0] == name:
                if instance == 0:
                    del self.identifiers[lpc]
                    return
                else:
                    instance = instance - 1
            lpc = lpc + 1
        raise AttributeError
    
    def find(self, name, instance = 0):
        lpc = 0
        while lpc < len(self.identifiers):
            if self.identifiers[lpc][0] == name:
                if instance == 0:
                    return self.identifiers[lpc][1]
                else:
                    instance = instance - 1
            lpc = lpc + 1
        raise AttributeError

    def identifierCount(self, name):
        lpc = 0
        retval = 0
        while lpc < len(self.identifiers):
            if self.identifiers[lpc][0] == name:
                retval = retval + 1
            lpc = lpc + 1
        return retval

    def getIdentifiers(self):
        return self.identifiers
    
    def pushSpecifier(self, name, data = ""):
        self.specifiers.append((name, str(data)))
        self.specifierCount = self.specifierCount + 1

    def popSpecifier(self):
        self.specifierCount = self.specifierCount - 1
        retval = self.specifiers[self.specifierCount]
        return retval
    
    def restoreSpecifier(self):
        self.specifierCount = self.specifierCount + 1

    def reverseSpecifiers(self):
        lpc = 0
        while lpc < (len(self.specifiers) / 2):
            swapIndex = len(self.specifiers) - lpc - 1
            spec = self.specifiers[lpc]
            self.specifiers[lpc] = self.specifiers[swapIndex]
            self.specifiers[swapIndex] = spec
            lpc = lpc + 1

    def flatten(self):
        msgSize = calcsize(headerFormat)
        contents = ""
        for id in self.identifiers:
            packFormat = ("!i" + `(len(id[0]) + 1 + 3) & ~3` + "s" +
                          "i" + `(len(id[1]) + 1 + 3) & ~3` + "s")
            msgSize = msgSize + calcsize(packFormat)
            contents = contents + pack(packFormat,
                                       len(id[0]) + 1,
                                       id[0],
                                       len(id[1]) + 1,
                                       id[1])
        offsets = []
        for spec in self.specifiers:
            offsets.append(msgSize + calcsize("!i"))
            packFormat = ("!i" + `(len(spec[0]) + 1 + 3) & ~3` + "s" +
                          "i" + `(len(spec[1]) + 1 + 3) & ~3` + "s")
            msgSize = msgSize + calcsize(packFormat)
            contents = contents + pack(packFormat,
                                       len(spec[0]) + 1,
                                       spec[0],
                                       len(spec[1]) + 1,
                                       spec[1])
        msgSize = msgSize + (calcsize("!i") * len(offsets))
        for off in offsets:
            contents = contents + pack("!i", off)
        contents = (pack(headerFormat,
                         msgSize,
                         self.what,
                         len(self.specifiers),
                         len(self.identifiers)) +
                    contents)
        return contents

    def unflatten(self, buf):
        self.specifiersCount = 0
        headerSize = calcsize(headerFormat)
        intSize = calcsize("!i")
        msgSize, self.what, specCount, idCount = unpack(headerFormat,
                                                        buf[:headerSize])
        specIndexes = buf[msgSize - (specCount * intSize):]
        lpc = 0
        specOffsets = []
        while lpc < specCount:
            offset, = unpack("!i", specIndexes[:intSize])
            specOffsets.append(offset)
            specIndexes = specIndexes[intSize:]
            lpc = lpc + 1
        buf = buf[headerSize:]
        if len(specOffsets) == 0:
            currSpec = -1
        else:
            currSpec = 0
        lpc = 0
        while lpc < (specCount + idCount):
            slen, = unpack("!i", buf[:intSize])
            buf = buf[intSize:]
            offset = msgSize - len(buf)
            padCount = ((slen + 3) & ~3) - (slen - 1)
            packFormat = "!" + `slen - 1` + "s" + `padCount` + "xi"
            packSize = calcsize(packFormat)
            id, slen = unpack(packFormat, buf[:packSize])
            buf = buf[packSize:]
            padCount = ((slen + 3) & ~3) - (slen - 1)
            packFormat = "!" + `slen - 1` + "s" + `padCount` + "x"
            packSize = calcsize(packFormat)
            val, = unpack(packFormat, buf[:packSize])
            buf = buf[packSize:]
            if currSpec >= 0 and offset == specOffsets[currSpec]:
                self.specifiers.append((id, val))
                self.specifiersCount = self.specifiersCount + 1
                currSpec = currSpec + 1
            else:
                self.identifiers.append((id, val))
            lpc = lpc + 1

    def describe(self):
        print self.what
        print self.specifiers
        print self.identifiers

