#! /usr/local/bin/python
#
# Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

from jsi.channel import *
from jsi.message import *
from jsi.messageError import *
from janosvm.janosvm import *

jvm = JanosVM(jsiConnectChannelTo("tcp:localhost/14000"))

from string import *
from whrandom import *
from math import *

# Used for tracking min/max values for gc stats
gcMin = {}
gcMax = {}

def printDiff(tag, value1, value2):
    # Use broke* to bracket the min/max values for a tag.  If the previous
    # values still hold then they are just the following brackets.  If there
    # was no previous value then we use curly braces and if the value broke
    # the min or max then we use an angle bracket pointing the appropriate
    # direction
    brokeMin = "["
    brokeMax = "]"
    if gcMin.get(tag, -1) == -1:
        # First value seen
        gcMin[tag] = min(value1, value2)
        brokeMin = "{"
    else:
        newMin = min(value1, value2, int(gcMin[tag]))
        if newMin != gcMin[tag]:
            brokeMin = "<"
        gcMin[tag] = newMin
    if gcMax.get(tag, -1) == -1:
        # First value seen
        gcMax[tag] = max(value1, value2)
        brokeMax = "}"
    else:
        newMax = max(value1, value2, int(gcMax[tag]))
        if newMax != gcMax[tag]:
            brokeMax = ">"
        gcMax[tag] = newMax
    print ("Difference of "
           + `value2 - value1`
           + " in "
           + `tag`
           + " Before: "
           + `value1`
           + " After: "
           + `value2`
           + " "
           + brokeMin
           + `gcMin[tag]`
           + " "
           + `gcMax[tag]`
           + brokeMax)

def gcDiff(gc1, gc2):
    items = gc1.items();
    # Sort the list by value names
    items.sort(lambda x, y: cmp(x[0], y[0]))
    for i in items:
        tag = i[0];
        # Get a scoped name list
        scopedTag = split(tag, ".")
        if (len(scopedTag) == 2) and ((scopedTag[1] == "allocCount") or (scopedTag[1] == "freeCount")):
            # Count the objects
            if scopedTag[1] == "allocCount":
                allocTag = tag;
                freeTag = scopedTag[0] + ".freeCount"
                activeObjects1 = int(gc1[allocTag]) - int(gc1[freeTag])
                activeObjects2 = int(gc2[allocTag]) - int(gc2[freeTag])
                if activeObjects1 != activeObjects2:
                    printDiff((scopedTag[0] + ".liveObjects"),
                              activeObjects1, activeObjects2)
        else:
            # A usage/peakUsage value
            value1 = int(i[1])
            value2 = int(gc2[tag])
            if value1 != value2:
                printDiff(tag, value1, value2)


# Tests to run
jvmtests = ["TestIntLong",
            "TestFloatDouble",
            "DoubleNeg",
            "DoublePrint",
            "DoubleComp",
            "ModuloTest",
            "LongNeg",
            "FPUStack",
            "Str",
            "Str2",
            # XXX "InternHog",
            "IndexTest",
            #"StackDump",
            "tname",
            "ttest",
            # XXX "ThreadState",
            "IllegalWait",
            "ReflectInterfaces",
            "TestCasts",
            #"NullPointerTest",
            #"NullInvoke",
            "TableSwitch",
            #"LostFrame",
            "ConstructorTest",
            "burford",
            "IllegalInterface",
            "GetInterfaces",
            "IntfTest",
            "SignedShort",
            "CharCvt",
            "BadFloatTest",
            # "wc",
            "FileTest",
            "finaltest",
            "forNameTest",
            "ArrayForName",
            # XXX "KaffeVerifyBug",
            "Schtum",
            "MethodBug",
            "SortTest",
            "MapTest",
            "ReflectInvoke",
            "InvTarExcTest",
            #"CLTest",
            #"CLTestConc",
            #"CLTestLie",
            #"CLTestFindLoaded",
            "ClassDeadLock",
            "ExceptionTest",
            #"ExceptionTestClassLoader",
            #"ExceptionTestClassLoader2",
            "TestClassRef",
            "ClassGC",
            # XXX "GCTest",
            "ThreadLocalTest",
            #"ExceptionInInitializerTest",
            "InitTest",
            #"ProcessClassTest",
            "FindSystemClass",
            "DoubleBug",
            "MarkResetTest",
            "HashtableTest1",
            "ReflectMultiArray",
            "DeadThread",
            "tthrd1",

            "HelloWorld",
            "NonExistentClass",
            "BadClass",
            "UncaughtException",
            "SharedTableTest",
            "JavaJSITest",
            "ConfigTest",
            "SignalTest",
            "HandlerTest",
            "MessageTest",
            "FaxTest",
            "TeamListTest",
            "Crash",
            "SuicideGeno",
            "SuicideGeno2",
            "TeamTest",
            "Visit",
            "Visit2",
            "VisitException",
            "NativeTest",
            "WriteBarrier",
            "StdioTest",
            "StalledGeno",
            "ImportExport",
            "RevokeExport",
            "RevokeImport",
            "MemoryLimit",
            "MemoryPolicy",
            "IllegalClassPath",
            "ClassPathLimit",
            "NameSpaceBoundary",
            "IllegalNameSpace",
            "TeamLocal",
            "ReservationTest",
            "IsolateTest",
            # XXX "CommQueueTester",
            # XXX "CommSpaceElementTester",
            # XXX "CommSpaceHandleTester"
            ]

# Record the number of times we run a test
testruns = [0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0]

lpc = 0
# Get the initial gc stats
jvm.team(1).gc()
jvm.team(1).gc()
# gc1 = jvm.team(1).getGCStats()
while lpc < 10000:
    try:
        testIndex = int(random() * len(jvmtests))
        #testruns[testIndex] = testruns[testIndex] + 1
        tid = jvm.teams().create(name=jvmtests[testIndex],
                                 namespace=["1:kernel:identity",
                                            "1:janosvm:identity",
                                            "1:fakeJava:identity"])
        if tid != 0:
            print `lpc` + " Running " + jvmtests[testIndex] + " in " + `tid`
            jvm.team(tid).run(jvmtests[testIndex])
            jvm.team(tid).delete()
            jvm.team(tid).wait()
        tl = jvm.teams().list()
        if len(tl) != 1:
            print jvmtests[testIndex] + " left " + `len(tl) - 1` + " stale team(s)"
            for i in tl:
                if i[0] != "1":
                    jvm.team(string.atoi(i[0])).delete()
        # if (lpc % 10) == 0:
            # Run the kernel gc
            # jvm.team(1).gc()
            # jvm.team(1).gc()
            # jvm.team(1).gc()
            # Get its stats
            # gc2 = jvm.team(1).getGCStats()
            # Print out any differences between the last gc and the current
            # gcDiff(gc1, gc2)
            # Get the current stats
            # gc1 = jvm.team(1).getGCStats()
    except MessageError:
        pass
    lpc = lpc + 1

jvm.shutdown()

#lpc = 0
#while lpc < len(jvmtests):
#    print jvmtests[lpc] + ": " + `testruns[lpc]`
#    lpc = lpc + 1
