
import java.io.IOException;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.DeadTeamException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.ClassPathSpec;
import edu.utah.janosvm.resources.ClassPathResource;
import edu.utah.janosvm.resources.NameSpaceResource;
import edu.utah.janosvm.resources.ResourceSpecException;

public class ClassPathLimit
{

    public static void main(String args[])
	throws Exception
    {
	Team lt = Team.current();
	TeamHandle child = null;
	ClassPathSpec cps;
	int lpc;

	{
	    String oldPaths[] = ClassPathResource.getPaths();

	    for( lpc = 0; lpc < oldPaths.length; lpc++ )
	    {
		ClassPathResource.hidePath(oldPaths[lpc]);
	    }

	    if( ClassPathResource.getPaths().length != 0 )
	    {
		String paths[] = ClassPathResource.getPaths();
		
		System.out.println("There are still paths in there?");
		for( lpc = 0; lpc < paths.length; lpc++ )
		{
		    System.out.println("path["
				       + lpc
				       + "] = "
				       + paths[lpc]);
		}
	    }

	    for( lpc = 0; lpc < oldPaths.length; lpc++ )
	    {
		ClassPathResource.addPath(oldPaths[lpc]);
	    }

	    if( ClassPathResource.getPaths().length != oldPaths.length )
		System.out.println("The paths weren't restored correctly?");
	}

	try
	{
	    /* Create a new team that doesn't have a class path */
	    ResourceSpec spec[] = {
		cps = new ClassPathSpec(null),
		StdioSpec.currentFD()
	    };

	    if( cps.getPathCount() != 0 )
		System.out.println("Theres something in an empty spec?");
	    if( cps.getPath(0) != null )
		System.out.println("Theres something in an empty spec?");
	    child = Team.create("Child 1 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				spec);

	    /* Visit the team and try to load some classes */
	    child.switchTo();
	    {
		try
		{
		    /*
		     * Need to load a class that isn't imported from the kernel
		     */
		    NameSpaceResource.loadTeamClass("ClassPathLimit");
		    System.out.println("Found class without classpath?");
		}
		catch(ClassNotFoundException cnfe)
		{
		    System.out.println("Didn't find class");
		}
	    }
	    child.returnFrom();
	    child.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Couldn't create team with empty class path?");
	}

	cps = new ClassPathSpec(new String[0]);
	
	if( cps.getPathCount() != 0 )
	    System.out.println("Theres something in an empty spec?");
	if( cps.getPath(0) != null )
	    System.out.println("Theres something in an empty spec?");

	cps = new ClassPathSpec(new String[] { null });
	
	if( cps.getPathCount() != 0 )
	    System.out.println("Theres something in an empty spec?");
	if( cps.getPath(0) != null )
	    System.out.println("Theres something in an empty spec?");

	try
	{
	    /*
	     * Create a team whose class path is just the current directory
	     * XXX Well this isn't portable now is it?
	     */
	    String cp[] = { "." };
	    ResourceSpec spec[] = {
		cps = new ClassPathSpec(cp),
		StdioSpec.currentFD()
	    };

	    if( cps.getPathCount() != cp.length )
		System.out.println("Bogus path count?");
	    for( lpc = 0; lpc < cp.length; lpc++ )
	    {
		if( !cps.getPath(lpc).equals(cp[lpc]) )
		    System.out.println("Invalid entry in class path?");
	    }
	    child = Team.create("Child 2 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				spec);
	    
	    child.switchTo();
	    {
		try
		{
		    /* Load something in the current directory */
		    NameSpaceResource.loadTeamClass("ClassPathLimit");
		    System.out.println("Found class");
		}
		catch(ClassNotFoundException cnfe)
		{
		    System.out.println("Didn't find class with classpath?");
		}
		try
		{
		    /*
		     * Need to load a class that isn't imported from the
		     * kernel or in the current directory.
		     */
		    NameSpaceResource.loadTeamClass("kaffe.lang.ClassPathReader");
		    System.out.println("Found class without classpath?");
		}
		catch(ClassNotFoundException cnfe)
		{
		    System.out.println("Didn't find class");
		}
	    }
	    child.returnFrom();
	    child.terminate();
	}
	catch(ResourceSpecException e)
	{
	}

	try
	{
	    ClassPathResource.addPath("/tmp");
	    child = Team.create();
	    ClassPathResource.hidePath("/tmp");
	    ClassPathResource.removePath("/tmp");
	    child.terminate();
	}
	catch(ResourceSpecException e)
	{
	    ClassPathResource.removePath("/tmp");
	    System.out.println("Bah, couldn't create team?");
	}
	catch(IOException e)
	{
	    System.out.println("Bah, couldn't test with /tmp class path.");
	}
    }
    
}

/* Expected Output:
Didn't find class
Found class
Didn't find class
*/
