
import java.util.Vector;

import java.lang.reflect.Field;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.ConfigFile;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ConfigSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;

public class ConfigTest
{
    private static final String sectionName = "ConfigTest";
    
    public static class MyData
    {
	public byte byteSlot;
	public int intSlot;
	public float floatSlot;
	public String stringSlot;
	public Vector child;

	public String toString()
	{
	    String retval = "MyData[byteSlot="
		+ this.byteSlot
		+ "; intSlot="
		+ this.intSlot
		+ "; floatSlot="
		+ this.floatSlot
		+ "; stringSlot="
		+ this.stringSlot
		+ "; child[]={";
	    int lpc;

	    if( this.child != null )
	    {
		for( lpc = 0; lpc < this.child.size(); lpc++ )
		{
		    if( lpc > 0 )
			retval += ", ";
		    retval += this.child.elementAt(lpc).toString();
		}
	    }
	    retval += "}]";
	    return retval;
	}
    }

    private static Class StringClass;
    private static Class VectorClass;

    static {
	try
	{
	    StringClass = Class.forName("java.lang.String");
	    VectorClass = Class.forName("java.util.Vector");
	}
	catch(ClassNotFoundException e)
	{
	    System.out.println("Couldn't load class...");
	    e.printStackTrace();
	}
    }
    
    private static void storeValue(MyData dest, String objName,
				   Field field, String value)
	throws IllegalAccessException
    {
	Class fieldType = field.getType();

	if( fieldType == Byte.TYPE )
	{
	    field.setByte(dest, Byte.parseByte(value));
	}
	else if( fieldType == Character.TYPE )
	{
	    field.setChar(dest, value.charAt(0));
	}
	else if( fieldType == Integer.TYPE )
	{
	    field.setInt(dest, Integer.parseInt(value));
	}
	else if( fieldType == Float.TYPE )
	{
	    field.setFloat(dest, Float.parseFloat(value));
	}
	else if( fieldType == Double.TYPE )
	{
	    field.setDouble(dest, Double.parseDouble(value));
	}
	else if( fieldType == StringClass )
	{
	    field.set(dest, value);
	}
	else if( fieldType == VectorClass )
	{
	    Vector vec;
	    int lpc;

	    if( (vec = (Vector)field.get(dest)) == null )
	    {
		vec = new Vector();
		field.set(dest, vec);
	    }
	    for( lpc = 0;
		 (value = ConfigFile.lookup(sectionName + ":" + objName,
					    field.getName(),
					    lpc)) != null;
		 lpc++ )
	    {
		vec.addElement(makeObject(value));
	    }
	}
	else
	{
	    System.out.println("Unsupported field type "
			       + fieldType.getName());
	}
    }

    private static MyData makeObject(String name)
	throws IllegalAccessException
    {
	Class cl = MyData.class;
	Field f[] = cl.getDeclaredFields();
	MyData retval = new MyData();
	int lpc;

	for( lpc = 0; lpc < f.length; lpc++ )
	{
	    String fieldName = f[lpc].getName();
	    String value;
			
	    if( (value = ConfigFile.lookup(sectionName + ":" + name,
					   fieldName))
		!= null )
	    {
		storeValue(retval, name, f[lpc], value);
	    }
	}
	return retval;
    }
	
    public static void main(String args[])
	throws Exception
    {
	MyData md = makeObject("root");
	int lpc, count;
	TeamHandle th;
	
	System.out.println("md = " + md);

	th = Team.create("Child of "
			 + Team.current().getID(),
			 new ResourceSpec[] {
				 new ConfigSpec(new String[] {
					 "custom.config"
				 }),
				 NameSpaceSpec.current(),
				 StdioSpec.currentFD()
			 });
	th.switchTo();
	{
	    System.out.println(ConfigFile.lookup("ConfigTestPartDeux", "foo"));
	}
	th.returnFrom();
	th.terminate();
	
	th = Team.create("Child of "
			 + Team.current().getID(),
			 new ResourceSpec[] {
				 new ConfigSpec(new String[0]),
				 NameSpaceSpec.current(),
				 StdioSpec.currentFD()
			 });
	th.terminate();
	
	th = Team.create("Child of "
			 + Team.current().getID(),
			 new ResourceSpec[] {
				 new ConfigSpec(new String[] {
					 null
				 }),
				 NameSpaceSpec.current(),
				 StdioSpec.currentFD()
			 });
	th.terminate();

	ConfigSpec cs = new ConfigSpec(new String[] {
		"rory",
		"dean",
		"tristan"
	});

	cs = (ConfigSpec)cs.clone();
	count = cs.getFileCount();
	for( lpc = 0; lpc < count; lpc++ )
	{
	    System.out.println(cs.getFile(lpc));
	}
    }
}

/* Expected Output:
md = MyData[byteSlot=64; intSlot=123456; floatSlot=1.23; stringSlot=Hello World!; child[]={MyData[byteSlot=32; intSlot=0; floatSlot=0.0; stringSlot=null; child[]={}], MyData[byteSlot=100; intSlot=0; floatSlot=0.0; stringSlot=null; child[]={}]}]
bar
rory
dean
tristan
*/
