
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.NoSuchExportException;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.kit.comm.amiga.FaxPort;
import edu.utah.janosvm.kit.comm.amiga.Facsimile;
import edu.utah.janosvm.kit.comm.amiga.SignalSet;
import edu.utah.janosvm.kit.comm.amiga.FaxPortHandle;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;
import edu.utah.janosvm.resources.NameSpaceResource;

public class FaxTest
    extends Thread
{
    /**
     * Simple fax that contains a string that the server thread should
     * print out.
     */
    public static class EchoFacsimile
	extends Facsimile
    {
	private String msg;

	public EchoFacsimile(String msg)
	{
	    this.msg = msg;
	}

	public String getString()
	{
	    return this.msg;
	}

	public Object clone()
	{
	    Object retval = super.clone();
	    EchoFacsimile ef = (EchoFacsimile)retval;

	    /* Do a deep clone */
	    ef.msg = CloneFactory.clone(this.msg);
	    return retval;
	}
    }
    
    /**
     * Another simple fax type, this one will ask the server thread for
     * some statistics.  Currently, its just the number of echo messages its
     * processed.
     */
    public static class QueryFacsimile
	extends Facsimile
    {
	private int echoCount;

	public QueryFacsimile()
	{
	}

	public void setEchoCount(int count)
	{
	    this.echoCount = count;
	}
	
	public int getEchoCount()
	{
	    return this.echoCount;
	}

	public void reply()
	    throws DeadTeamException, TeamEjectionException
	{
	    super.reply();
	}
    }

    public FaxTest()
    {
    }

    public void run()
    {
	try
	{
	    SignalSet ss = new SignalSet();
	    FaxPort fp = new FaxPort("Echo-Server", ss);
	    int fpSig = fp.getSignalNumber();
	    long waitMask = (1L << fpSig);
	    long recvMask;
	    int echoCount = 0;

	    /*
	     * Just loop forever, the team we're running will get killed when
	     * the test is done.
	     */
	    while( true )
	    {
		try
		{
		    /* Wait for our FaxPort to signal us */
		    recvMask = ss.waitFor(waitMask);
		    if( (recvMask & (1L << fpSig)) != 0 )
		    {
			/* It was a fax, pull it off the queue */
			Facsimile fax = fp.getFacsimile();

			while( fax != null )
			{
			    /* Figure out what kind of fax */
			    if( fax instanceof EchoFacsimile )
			    {
				EchoFacsimile ef = (EchoFacsimile)fax;

				echoCount++;
				System.out.println("Fax #"
						   + echoCount
						   + ":");
				System.out.println(ef.getString());
			    }
			    else if( fax instanceof QueryFacsimile )
			    {
				QueryFacsimile qf = (QueryFacsimile)fax;
				
				qf.setEchoCount(echoCount);
				qf.reply();
			    }
			    else
			    {
				System.out.println("Can't handle facsimile: "
						   + fax);
			    }
			    /*
			     * Keep dequeueing faxes until there aren't any
			     * more (returns null).
			     */
			    fax = fp.getFacsimile();
			}
		    }
		}
		catch(Throwable th)
		{
		    /* doh! */
		    System.out.println("Got exception: " + th);
		    th.printStackTrace();
		}
	    }
	}
	catch(DuplicateExportException e)
	{
	}
    }

    public static void main(String args[])
	throws Exception
    {
	/* Test messages */
	String echoMessages[] = {
	    "\'Things involving the computer fill me with a childlike terror."
	    + "  Now, if it were a nice ogre or some such, I'd be more in my"
	    + " element.\' -- Giles",
	    "\'You were looking at my neck.\'\n"
	    + "\'What?\'\n"
	    + "\'You were checking out my neck, I saw that.\'\n"
	    + "\'No, I wasn't.\'\n"
	    + "\'Just keep your distance, pal.\'\n"
	    + "\'I wasn't looking at your neck.\'\n"
	    + "\'I told you to eat before we left.\' -- Xander and Angel",
	    "\'We like to talk big, vampires do. \'I'm going to destroy the "
	    + "world.\' That's just tough guy talk. Strutting around with "
	    + "your friends over a pint of blood. The truth is, I like "
	    + "this world. You've got dog racing. Manchester United. And "
	    + "you've got people. Billions of people walking around like "
	    + "Happy Meals with legs. It's all right here.\' -- Spike",
	    "'How can I thank you, you mysterious black-clad hunk of a "
	    + "night-thing.' 'No need, little lady.  Your tears of gratitude "
	    + "are enough for me. You see, I was once a bad-ass vampire. But "
	    + "love and a pesky curse defanged me and now I'm just a big "
	    + "fluffy puppy with bad teeth. No, not the hair. Never the hair.'"
	    + "'But there must be some way I can show my appreciation.' 'No. "
	    + "Helping those in need is my job and working up a load of "
	    + "sexual tension and prancing away like a magnificent poof is "
	    + "truly thanks enough.' 'I understand, I have a nephew who's "
	    + "gay.' 'Say no more, evil's afoot and I'm almost out of that "
	    + "nancy-boy hair gel I like so much. Quickly! To the "
	    + "Angel-mobile.  Away!' - Spike"
	};

	Team curr = Team.current();
	TeamHandle kernel = curr.getKernelTeam();
	SignalSet ss = new SignalSet();
	FaxPort replyPort = new FaxPort(ss);
	QueryFacsimile qf = new QueryFacsimile();
	QueryFacsimile qfReply;
	FaxPortHandle fp = null;
	int lpc;

	qf.setReplyPort(replyPort);
	/* Create a new team that shares the same facsimile classes */
	NameSpaceResource.exportClassGroup("faxServer");
	NameSpaceGroup nsg[] = {
	    new NameSpaceGroup(kernel, "kernel", "identity"),
	    new NameSpaceGroup(kernel, "janosvm", "identity"),
	    new NameSpaceGroup(curr, "faxServer", "identity")
	};
	TeamHandle child = Team.create("Child of "
				       + curr.getName()
				       + curr.getID(),
				       new ResourceSpec[] {
					       new NameSpaceSpec(nsg),
					       StdioSpec.currentFD()
				       });
	
	/* Visit the child and start the server thread inside of it */
	child.switchTo();
	{
	    Class cl = NameSpaceResource.loadTeamClass("FaxTest");
	    Thread th = (Thread)cl.newInstance();
	    
	    th.start();
	    /*
	     * XXX Too lazy to do some signal thing, just delay a bit...
	     */
	    Thread.sleep(50);
	}
	child.returnFrom();
	try
	{
	    /* Import the server message port from the child team */
	    fp = new FaxPortHandle(child, "Echo-Server");
	    /* Send some messages to it */
	    for( lpc = 0; lpc < echoMessages.length; lpc++ )
	    {
		fp.putFacsimile(new EchoFacsimile(echoMessages[lpc]));
		fp.putFacsimile(qf);
		qfReply = (QueryFacsimile)replyPort.waitForFacsimile();
		System.out.println("Echo count = " + qfReply.getEchoCount());
	    }
	}
	catch(NoSuchExportException e)
	{
	    System.out.println("The port wasn't exported?");
	    e.printStackTrace();
	}
	finally
	{
	    child.terminate();
	}

	try
	{
	    /*
	     * We killed the team, so it should've been revoked and this
	     * shouldn't work.
	     */
	    if( fp != null )
		fp.putFacsimile(new EchoFacsimile("This shouldn't work"));
	}
	catch(DeadTeamException dte)
	{
	    System.out.println("FaxPort shutdown correctly");
	}
    }
}

/* Expected Output:
Fax #1:
'Things involving the computer fill me with a childlike terror.  Now, if it were a nice ogre or some such, I'd be more in my element.' -- Giles
Echo count = 1
Fax #2:
'You were looking at my neck.'
'What?'
'You were checking out my neck, I saw that.'
'No, I wasn't.'
'Just keep your distance, pal.'
'I wasn't looking at your neck.'
'I told you to eat before we left.' -- Xander and Angel
Echo count = 2
Fax #3:
'We like to talk big, vampires do. 'I'm going to destroy the world.' That's just tough guy talk. Strutting around with your friends over a pint of blood. The truth is, I like this world. You've got dog racing. Manchester United. And you've got people. Billions of people walking around like Happy Meals with legs. It's all right here.' -- Spike
Echo count = 3
Fax #4:
'How can I thank you, you mysterious black-clad hunk of a night-thing.' 'No need, little lady.  Your tears of gratitude are enough for me. You see, I was once a bad-ass vampire. But love and a pesky curse defanged me and now I'm just a big fluffy puppy with bad teeth. No, not the hair. Never the hair.''But there must be some way I can show my appreciation.' 'No. Helping those in need is my job and working up a load of sexual tension and prancing away like a magnificent poof is truly thanks enough.' 'I understand, I have a nephew who's gay.' 'Say no more, evil's afoot and I'm almost out of that nancy-boy hair gel I like so much. Quickly! To the Angel-mobile.  Away!' - Spike
Echo count = 4
FaxPort shutdown correctly
*/
