
import edu.utah.janosvm.sys.List;
import edu.utah.janosvm.sys.ListNode;

import edu.utah.janosvm.kit.comm.amiga.MsgPort;
import edu.utah.janosvm.kit.comm.amiga.Message;
import edu.utah.janosvm.kit.comm.amiga.SignalSet;
import edu.utah.janosvm.kit.comm.amiga.SignalHandler;
import edu.utah.janosvm.kit.comm.amiga.SignalDispatcher;

public class HandlerTest
{
    private static final int SIG_CONTINUE = 0;
    
    private static final SignalSet mainSignals = new SignalSet();
    
    private static class Sig1Handler
	extends SignalHandler
    {
	public Sig1Handler(int sn)
	{
	    this.setSignalNumber(sn);
	}
	
	public void run()
	{
	    System.out.println("This is Sig1Handler");
	    HandlerTest.mainSignals.signal(SIG_CONTINUE);
	}
    }

    private static class Sig2Handler
	extends SignalHandler
    {
	public Sig2Handler(int sn)
	{
	    this.setSignalNumber(sn);
	}
	
	public void run()
	{
	    System.out.println("This is Sig2Handler");
	    HandlerTest.mainSignals.signal(SIG_CONTINUE);
	}
    }

    private static class DummyMessage
	extends Message
    {
	public int demuxValue;

	public DummyMessage(int dv)
	{
	    this.demuxValue = dv;
	}
    }

    private static class DummyHandler
	extends ListNode
    {
	private int id;

	public DummyHandler(int id)
	{
	    this.id = id;
	}

	public int getID()
	{
	    return this.id;
	}

	public void handle(DummyMessage dm)
	{
	    System.out.println("Hello, I'm " + this.id);
	}
    }
    
    private static class DemuxHandler
	extends SignalHandler
    {
	private final List handlers = new List();
	private final MsgPort port;
	
	public DemuxHandler(MsgPort port)
	{
	    this.port = port;
	    this.setSignalNumber(port.getSignalNumber());
	}

	public void addHandler(DummyHandler dh)
	{
	    this.handlers.addHead(dh);
	}

	public void run()
	{
	    DummyHandler dh;
	    DummyMessage dm;
	    ListNode ln;
	    
	    System.out.println("This is DemuxHandler");
	    while( (dm = (DummyMessage)this.port.getMessage()) != null )
	    {
		ln = this.handlers.head.succ;
		while( ln.succ != null )
		{
		    dh = (DummyHandler)ln;
		    if( dh.getID() == dm.demuxValue )
		    {
			dh.handle(dm);
		    }
		    ln = ln.succ;
		}
	    }
	    HandlerTest.mainSignals.signal(SIG_CONTINUE);
	}
    }
    
    public static void main(String args[])
	throws Exception
    {
	final int DUMMY_COUNT = 3;
	
	SignalDispatcher ht = SignalDispatcher.getTeamDispatcher();
	int sig1, sig2, lpc;
	DemuxHandler dh;
	SignalSet ss;
	MsgPort mp;
	Thread th;

	if( ht != SignalDispatcher.getTeamDispatcher() )
	{
	    System.out.println("Different result from getTeamDispatcher()");
	}
	mainSignals.allocate(SIG_CONTINUE);
	th = new Thread(ht);
	th.start();
	ss = ht.getSignalSet();
	sig1 = ss.allocate();
	ht.addHandler(new Sig1Handler(sig1));
	sig2 = ss.allocate();
	ht.addHandler(new Sig2Handler(sig2));
	mp = new MsgPort(ss);
	dh = new DemuxHandler(mp);
	for( lpc = 0; lpc < DUMMY_COUNT; lpc++ )
	{
	    dh.addHandler(new DummyHandler(lpc));
	}
	ht.addHandler(dh);
	ss.signal(sig1);
	mainSignals.waitFor(1L << SIG_CONTINUE);
	ss.signal(sig2);
	mainSignals.waitFor(1L << SIG_CONTINUE);
	for( lpc = 0; lpc < DUMMY_COUNT; lpc++ )
	{
	    mp.putMessage(new DummyMessage(lpc));
	    mainSignals.waitFor(1L << SIG_CONTINUE);
	}
	ht.exit();
	th.join();
    }
}

/* Expected Output:
This is Sig1Handler
This is Sig2Handler
This is DemuxHandler
Hello, I'm 0
This is DemuxHandler
Hello, I'm 1
This is DemuxHandler
Hello, I'm 2
*/
