
import java.io.IOException;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.DeadTeamException;

import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.ClassPathSpec;
import edu.utah.janosvm.resources.ClassPathResource;
import edu.utah.janosvm.resources.ResourceSpecException;

public class IllegalClassPath
{
    public static void main(String args[])
	throws Exception
    {
	ClassPathSpec cps = null;
	Team lt = Team.current();
	TeamHandle child;
	
	try
	{
	    ClassPathResource.loadFile(null);
	    System.out.println("Able to load null file?");
	}
	catch(NullPointerException e)
	{
	    System.out.println("Caught null on loadFile");
	}
	
	try
	{
	    ClassPathResource.loadFile("XXXbogus fileXXX");
	    System.out.println("Able to load XXXbogus fileXXX file?");
	}
	catch(IOException e)
	{
	    System.out.println("Caught IOException on bogus loadFile");
	}
	
	try
	{
	    ClassPathResource.loadFile("");
	    System.out.println("Able to load empty file name?");
	}
	catch(IOException e)
	{
	    System.out.println("Caught IOException on empty loadFile");
	}
	
	try
	{
	    ClassPathResource.addPath(null);
	    System.out.println("Able to add null path?");
	}
	catch(NullPointerException e)
	{
	    System.out.println("Caught null on addPath");
	}
	
	try
	{
	    ClassPathResource.hidePath(null);
	    System.out.println("Able to hide null path?");
	}
	catch(NullPointerException e)
	{
	    System.out.println("Caught null on hidePath");
	}
	
	try
	{
	    ClassPathResource.removePath(null);
	    System.out.println("Able to remove null path?");
	}
	catch(NullPointerException e)
	{
	    System.out.println("Caught null on removePath");
	}
	
	try
	{
	    /*
	     * Try to create a team with a bogus class path, in the hopes that
	     * it will throw a resource exception.  Since the kernel maintains
	     * a list of what paths are valid for other teams this is pretty
	     * easy to check.
	     */
	    String cp[] = { "ThisIsAFakeClassPath" };
	    ResourceSpec spec[] = {
		cps = new ClassPathSpec(cp)
	    };
	    child = Team.create("Child 3 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				spec);
	    System.out.println("Bogus class path was accepted");
	    child.terminate();
	}
	catch(ResourceSpecException e)
	{
	    if( e.getUnsatisfiedSpec() == cps )
	    {
		System.out.println("Bogus class path rejected");
	    }
	    else
	    {
		System.out.println("Bogus class path rejected, "
				   + "but unsatisfied spec isn't right");
	    }
	}
	
	try
	{
	    ClassPathResource.addPath("XXXbogusXXXpathXXX");
	    System.out.println("Able to add bogus path?  Does it exist "
			       + "in the file system?");
	}
	catch(IOException e)
	{
	    System.out.println("Couldn't add bogus path");
	}
	
	try
	{
	    ClassPathResource.hidePath("XXXbogusXXXpathXXX");
	    System.out.println("Able to hide bogus path?  Does it exist "
			       + "in the file system?");
	}
	catch(IOException e)
	{
	    System.out.println("Couldn't hide bogus path");
	}
    }
}

/* Expected Output:
Caught null on loadFile
Caught IOException on bogus loadFile
Caught IOException on empty loadFile
Caught null on addPath
Caught null on hidePath
Caught null on removePath
Bogus class path rejected
Couldn't add bogus path
Couldn't hide bogus path
*/
