
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;
import edu.utah.janosvm.resources.ResourceSpecException;
import edu.utah.janosvm.resources.InvalidClassGroupException;
import edu.utah.janosvm.resources.ClassGroupNotFoundException;

public class IllegalNameSpace
{
    public static class DanglingClass
    {
	private int value;

	public DanglingClass(int value)
	{
	    this.value = value;
	}

	public int getValue()
	{
	    return this.value;
	}
    }

    public static class ServerClass
    {
	private DanglingClass dc = new DanglingClass(0xDEADBEEF);

	public DanglingClass getDanglingClass()
	{
	    return this.dc;
	}
    }

    public static void main(String args[])
	throws Exception
    {
	Team team = Team.current();
	TeamHandle kernel = team.getKernelTeam();

	try
	{
	    NameSpaceResource.exportClassGroup(null);
	}
	catch(NullPointerException e)
	{
	    System.out.println("Caught NullPointerException");
	}
	
	try
	{
	    NameSpaceResource.exportClassGroup("bogus-class-group");
	}
	catch(ClassGroupNotFoundException e)
	{
	    System.out.println("Caught ClassGroupNotFoundException");
	}
	
	try
	{
	    NameSpaceResource.exportClassGroup("emptyNameSpace");
	}
	catch(InvalidClassGroupException e)
	{
	    System.out.println("Caught InvalidClassGroupException");
	}
	
	try
	{
	    NameSpaceResource.exportClassGroup("badClassNameSpace");
	}
	catch(ClassNotFoundException e)
	{
	    System.out.println("Caught ClassNotFoundException");
	}
	
	/* Export a class group for our child teams */
	NameSpaceResource.exportClassGroup("IllegalNameSpace");

	{
	    NameSpaceSpec nss = null;
	    
	    try
	    {
		NameSpaceGroup nsg[] = {
		    new NameSpaceGroup(kernel, "kernel", "identity"),
		    new NameSpaceGroup(team, "IllegalNameSpace", "identity"),
		};
		TeamHandle th = Team.create("Child of "
					    + team.getName()
					    + " "
					    + team.getID(),
					    new ResourceSpec[] {
						StdioSpec.currentFD(),
						nss = new NameSpaceSpec(nsg),
					    });
		System.out.println("Subscribed to illegal name space?");
		th.terminate();
	    }
	    catch(ResourceSpecException e)
	    {
		if( e.getUnsatisfiedSpec() == nss )
		{
		    System.out.println("Caught illegal name space");
		}
		else
		{
		    System.out.println("Caught illegal name space, "
				       + "but unsatisfied spec is different.");
		}
	    }
	    catch(DuplicateExportException e)
	    {
	    }
	}

	try
	{
	    NameSpaceGroup nsg[] = {
	    };
	    TeamHandle th = Team.create("Child of "
					+ team.getName()
					+ " "
					+ team.getID(),
					new ResourceSpec[] {
					    StdioSpec.currentFD(),
					    new NameSpaceSpec(nsg),
					});
	    System.out.println("Subscribed to illegal name space (no kern)?");
	    th.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Caught illegal name space (no kernel)");
	}
	catch(DuplicateExportException e)
	{
	}

	try
	{
	    NameSpaceGroup nsg[] = {
		new NameSpaceGroup(kernel, "kernel", "identity"),
		new NameSpaceGroup(kernel, "BogusClassGroup", "identity")
		    };
	    TeamHandle th = Team.create("Child of "
					+ team.getName()
					+ " "
					+ team.getID(),
					new ResourceSpec[] {
					    StdioSpec.currentFD(),
					    new NameSpaceSpec(nsg),
					});
	    System.out.println("Subscribed to nonexistant name space?");
	    th.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Caught nonexistant name space");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Couldn't make team?");
	}

	try
	{
	    new NameSpaceGroup(null, "", "");
	}
	catch(IllegalArgumentException e)
	{
	    System.out.println("Caught illegal argument exception for team");
	}
	try
	{
	    new NameSpaceGroup(team, null, "");
	}
	catch(IllegalArgumentException e)
	{
	    System.out.println("Caught illegal argument exception for class "
			       + "group");
	}
	try
	{
	    new NameSpaceGroup(team, "", null);
	}
	catch(IllegalArgumentException e)
	{
	    System.out.println("Caught illegal argument exception for view "
			       + "group");
	}

	{
	    NameSpaceGroup nsg;

	    nsg = new NameSpaceGroup(team, "kernel", "identity");
	    if( nsg.getTeam() != team )
		System.out.println("Team isn't the same?");
	    if( !nsg.getClassGroup().equals("kernel") )
		System.out.println("Class group isn't the same?");
	    if( !nsg.getViewGroup().equals("identity") )
		System.out.println("View group isn't the same?");
	}

	try
	{
	    new NameSpaceSpec(new NameSpaceGroup[] {
		null
	    });
	    System.out.println("Able to make spec with null entry");
	}
	catch(Throwable e)
	{
	    System.out.println("Unable to make spec with null entry?");
	    e.printStackTrace();
	}
	
	try
	{
	    new NameSpaceSpec(null);
	    System.out.println("Able to make spec with null groups?");
	}
	catch(Throwable e)
	{
	    System.out.println("Unable to make spec with null groups");
	}
    }
}

/* Expected Output:
Caught NullPointerException
Caught ClassGroupNotFoundException
Caught InvalidClassGroupException
Caught ClassNotFoundException
Caught illegal name space
Caught illegal name space (no kernel)
Caught nonexistant name space
Caught illegal argument exception for team
Caught illegal argument exception for class group
Caught illegal argument exception for view group
Able to make spec with null entry
Unable to make spec with null groups
*/
