
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.Importable;
import edu.utah.janosvm.sys.Exportable;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.NoSuchExportException;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;

public class ImportExport
{
    private static class GenericImport
	extends Importable
    {
	public Object obj;

	protected void bindTo(Exportable ex)
	{
	    this.obj = ex;
	}

	protected void unbindFrom()
	{
	    this.obj = null;
	}
    }

    private static class GenericExport
	extends Exportable
    {
	public Object obj;

	public GenericExport(Object val)
	{
	    this.obj = val;
	}
    }

    public static void main(String args[])
	throws Exception
    {
	TeamHandle kernel = Team.current().getKernelTeam();
	Team parent = Team.current();
	Integer i = new Integer(1);
	GenericExport ge = new GenericExport(i);

	parent.exportObject("i", ge);
	try
	{
	    /* Try to export something with the same identifier twice */
	    parent.exportObject("i", new GenericExport(i));
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Can't export duplicate objects");
	}

	/* Export a class group for our child teams */
	NameSpaceResource.exportClassGroup("ImportExport");

	NameSpaceSpec nss = new NameSpaceSpec(new NameSpaceGroup[] {
		new NameSpaceGroup(kernel, "kernel", "identity"),
		new NameSpaceGroup(kernel, "janosvm", "identity"),
		new NameSpaceGroup(parent, "ImportExport", "identity")
	});
	
	/* Create a team and try to import `i' from this team */
	TeamHandle child = Team.create("Child of "
				       + parent.getName()
				       + parent.getID(),
				       new ResourceSpec[] {
					       StdioSpec.currentFD(),
					       nss,
				       });

	child.switchTo();
	{
	    Team child_team = Team.current();
	    GenericImport gi = new GenericImport();
	    
	    try
	    {
		parent.importObject(gi, "i");
		if( gi.obj == ge )
		{
		    System.out.println("Got object");
		    if( !parent.owns(gi.obj) )
			System.out.println("The parent doesn't own it?");
		    if( child_team.owns(gi.obj) )
			System.out.println("I own the parent's object?");
		}
		else
		{
		    System.out.println("Didn't get object");
		}
		if( !child_team.owns(gi) )
		    System.out.println("I don't own the object?");
	    }
	    catch(NoSuchExportException e)
	    {
		System.out.println("Couldn't import from parent");
		System.out.println(e);
	    }
	}
	child.returnFrom();
	child.terminate();
    }
    
}

/* Expected Output:
Can't export duplicate objects
Got object
*/
