
import edu.utah.janosvm.kit.isolate.Isolate;

public class IsolateTest
{
    public static void main(String args[])
	throws Exception
    {
	if( args.length == 0 )
	{
	    /* The main guy... */
	    Isolate iso;

	    iso = new Isolate("IsolateTest",
			      new String[] { "Simple" },
			      null,
			      "IsolateTest/simple");
	    if( !iso.getName().equals("IsolateTest/simple") )
		System.out.println("Isolate name incorrect?");
	    iso.start();
	    if( !iso.isStarted() )
		System.out.println("Isolate not started?");
	    if( iso.isStopped() )
		System.out.println("Isolate not started?");
	    if( !iso.equals(Isolate.getIsolate(iso.getUniqueId())) )
		System.out.println("Isolate's not equal?");
	    iso.join();
	    
	    iso = new Isolate("IsolateTest",
			      new String[] { "Inifinite" },
			      null,
			      "IsolateTest/infinite");
	    iso.start();
	    Thread.sleep(100);
	    iso.stop();
	    iso.join();
	    
	    iso = new Isolate("IsolateTest",
			      new String[] {
				  "MultipleArgs",
				  "Nick Diamond",
				  "Johnny Gomez",
				  "Stacy Cornbread",
				  "Judge Mills Lane"
			      },
			      null,
			      "IsolateTest/multiple-args");
	    iso.start();
	    iso.join();
	    
	    iso = new Isolate("IsolateTest",
			      new String[] { "LongRunning" },
			      null,
			      "IsolateTest/long-running");
	    iso.start();
	    iso.join();
	    
	    iso = new Isolate("IsolateTest",
			      new String[] { "ParentChild" },
			      null,
			      "IsolateTest/parent-child");
	    iso.start();
	    iso.stop();
	    iso.join();

	    /* Wait a sec to make sure everything has been cleaned up. */
	    Thread.sleep(100);
	    
	    iso = new Isolate("IsolateTest",
			      new String[] { "Tree" },
			      null,
			      "  IsolateTest/tree");
	    iso.start();
	    iso.join();
	}
	else if( args[0].equals("Simple") )
	{
	    Isolate iso = Isolate.getCurrentIsolate();
	    
	    if( !iso.getName().equals("IsolateTest/simple") )
		System.out.println("Isolate name incorrect?");
	    if( iso.getParent() == null )
		System.out.println("Child isolate has no parent?");
	    System.out.println("This is simple");
	}
	else if( args[0].equals("Inifinite") )
	{
	    System.out.println("This is the infinite looper");
	    while( true )
	    {
		Thread.sleep(10000);
	    }
	}
	else if( args[0].equals("MultipleArgs") )
	{
	    int lpc;

	    for( lpc = 1; lpc < args.length; lpc++ )
	    {
		System.out.println("Hello " + args[lpc]);
	    }
	}
	else if( args[0].equals("LongRunning") )
	{
	    System.out.println("Starting thread");
	    new Thread(new Runnable() {
		    public void run()
		    {
			int lpc;

			System.out.println("Looper started");
			try { Thread.sleep(50); }
			catch(InterruptedException e) { }
			for( lpc = 0; lpc < 10; lpc++ )
			{
			    System.out.println("Loop " + lpc);
			}
			System.out.println("Looper ended");
		    }
		}).start();
	}
	else if( args[0].equals("ParentChild") )
	{
	    Isolate iso;
	    
	    iso = new Isolate("IsolateTest",
			      new String[] { "ParentChild" },
			      null,
			      "IsolateTest/parent-child");
	    iso.start();
	    
	    while( true )
	    {
		Thread.sleep(10000);
	    }
	}
	else if( args[0].equals("Child") )
	{
	    Thread.sleep(10000);
	    System.out.println("I'm still alive?");
	}
	else if( args[0].equals("Tree") )
	{
	    Isolate iso;

	    iso = new Isolate("IsolateTest",
			      new String[] { "Branch" },
			      null,
			      "    IsolateTest/branch");
	    iso.start();
	    iso = new Isolate("IsolateTest",
			      new String[] { "Branch" },
			      null,
			      "    IsolateTest/branch");
	    iso.start();
	    
	    try { Thread.sleep(500); }
	    catch(InterruptedException e) { }
	    
	    iso = Isolate.getCurrentIsolate();
	    iso = iso.getParent();
	    walkChildren(iso);

	    Isolate.getCurrentIsolate().stop();
	}
	else if( args[0].equals("Branch") )
	{
	    Isolate iso;
	    
	    iso = new Isolate("IsolateTest",
			      new String[] { "Leaf" },
			      null,
			      "      IsolateTest/leaf");
	    iso.start();
	    
	    while( true )
	    {
		Thread.sleep(10000);
	    }
	}
	else if( args[0].equals("Leaf") )
	{
	    while( true )
	    {
		Thread.sleep(10000);
	    }
	}
	else
	{
	    System.out.println("Incorrect dispatch argument? " + args[0]);
	}
    }

    private static void walkChildren(Isolate iso)
    {
	Isolate parent, children[];

	if( (parent = iso.getParent()) != null )
	    System.out.println("Parent: " + parent.getName());
	System.out.println("Walking " + iso.getName());
	if( (children = iso.getChildren()) != null )
	{
	    int lpc;

	    for( lpc = 0; lpc < children.length; lpc++ )
	    {
		walkChildren(children[lpc]);
	    }
	}
    }
}

/* Expected Output:
This is simple
This is the infinite looper
Hello Nick Diamond
Hello Johnny Gomez
Hello Stacy Cornbread
Hello Judge Mills Lane
Starting thread
Looper started
Loop 0
Loop 1
Loop 2
Loop 3
Loop 4
Loop 5
Loop 6
Loop 7
Loop 8
Loop 9
Looper ended
Walking root
Parent: root
Walking   IsolateTest/tree
Parent:   IsolateTest/tree
Walking     IsolateTest/branch
Parent:     IsolateTest/branch
Walking       IsolateTest/leaf
Parent:   IsolateTest/tree
Walking     IsolateTest/branch
Parent:     IsolateTest/branch
Walking       IsolateTest/leaf
*/
