
import edu.utah.janosvm.sys.Team;

import java.io.IOException;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIChannel;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.TCPChannel;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.TCPProvider;
import edu.utah.janosvm.kit.comm.jsi.JSIProvider;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerRoot;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.JSIConnectionHandler;

public class JavaJSITest
{
    private static class EchoHandler
	implements JSIHandler
    {
	public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
	{
	    return this;
	}

	public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
	{
	    System.out.println("echo: " + jm);
	    jm.clear();
	    jm.setWhat(JSIMessage.REPLY);
	    jm.add("result", "ok");
	}

	public void getSuites(JSIMessage jm)
	{
	}
    }
    
    private static class MyHandlerThread
	extends Thread
    {
	private JSIChannel jc;
	
	MyHandlerThread(JSIChannel jc)
	{
	    this.jc = jc;
	}
	
	public void run()
	{
	    JSIHandlerContext jhc = new JSIHandlerContext(this.jc);
	    JSIHandlerRoot root = jhc.getRootHandlers();
	    byte msg[];

	    try
	    {
		while( (msg = this.jc.recvMessage()) != null )
		{
		    JSIMessage jm;
		    
		    if( (jm = JSIMessage.instantiate(msg)) != null )
		    {
			root.dispatchMessage(jhc, jm);
			if( (msg = jm.flatten()) != null )
			{
			    this.jc.replyMessage(msg);
			}
		    }
		    else
		    {
			System.err.println("Couldn't instantiate message!");
		    }
		}
	    }
	    catch(Throwable th)
	    {
		th.printStackTrace();
	    }
	    finally
	    {
		this.jc.disconnect();
	    }
	}

	public String toString()
	{
	    return "MyHandlerThread[jc="
		+ this.jc
		+ "]";
	}
    }
    
    public static void main(String args[])
	throws Exception
    {
	JSIMessage jm = new JSIMessage(), jm2;
	final int portNumber = 25430;
	final TCPProvider tp;
	JSIPropertyInfo jpi;
	byte flattenedMsg[];
	TCPChannel tc;

	JSIHandlerContext.addDefaultRootHandler("echo", new EchoHandler());
	System.out.println(jm);
	jm.setWhat(JSIMessage.EXECUTE_PROPERTY);
	jm.pushSpecifier("echo");
	jm.add("class", "foo");
	jpi = new JSIPropertyInfo("blah", 0, "", "not implemented");
	jm.add(new JSIPair("result", jpi));
	System.out.println(jm);
	flattenedMsg = jm.flatten();
	jm2 = JSIMessage.instantiate(flattenedMsg);
	System.out.println(jm2);
	if( !jm2.getPair("result").getPropertyInfoValue().equals(jpi) )
	    System.out.println("The properties aren't equal?");
	tp = new TCPProvider(portNumber);
	new Thread() {
		public void run()
		{
		    tp.handle(new JSIConnectionHandler() {
			    public void handle(JSIProvider jp, JSIChannel jc)
			    {
				MyHandlerThread th;
				
				th = new MyHandlerThread(jc);
				th.start();
			    }
			});
		}
	    }.start();
	tc = new TCPChannel("localhost", portNumber);
	    tc.sendMessage(flattenedMsg);
	    tc.recvMessage();
	tc.disconnect();
	tp.detach();
	Team.current().terminate();
    }
}

/* Expected Output:
JSIMessage[messageSize=16; what=0; specifiers=[]; specifierIndex=0; identifiers=[]]
JSIMessage[messageSize=116; what=11; specifiers=[JSIPair[tag=echo; value=]]; specifierIndex=1; identifiers=[JSIPair[tag=class; value=foo], JSIPair[tag=result; value=<:blah:><:0x0:><::><:not implemented:>]]]
JSIMessage[messageSize=116; what=11; specifiers=[JSIPair[tag=echo; value=]]; specifierIndex=1; identifiers=[JSIPair[tag=class; value=foo], JSIPair[tag=result; value=<:blah:><:0x0:><::><:not implemented:>]]]
echo: JSIMessage[messageSize=92; what=11; specifiers=[JSIPair[tag=echo; value=]]; specifierIndex=0; identifiers=[JSIPair[tag=class; value=foo], JSIPair[tag=result; value=<:blah:><:0x0:><::><:not implemented:>]]]
*/
