
import java.io.IOException;
import java.io.FileNotFoundException;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;

import edu.utah.janosvm.kit.server.memfs.MemoryFile;
import edu.utah.janosvm.kit.server.memfs.MemoryFileSystem;
import edu.utah.janosvm.kit.server.memfs.MemoryFileInputStream;
import edu.utah.janosvm.kit.server.memfs.MemoryFileOutputStream;

public class MemoryFileTest
{
    private static final Team current = Team.current();
    private static TeamHandle kernel;
    private static TeamHandle memfs;
    
    private static void terminateFS()
	throws Exception
    {
	kernel.importObject(memfs, MemoryFileSystem.TEAM_NAME);
	memfs.terminate();
    }
    
    public static void main(String args[])
	throws Exception
    {
	MemoryFileOutputStream mfos;
	MemoryFileInputStream mfis;
	MemoryFile mf;

	kernel = current.getKernelTeam();
	memfs = new TeamHandle();

	try
	{
	    new MemoryFileInputStream("XXXbogusXXX");
	    System.out.println("Able to open non-existent file?");
	}
	catch(FileNotFoundException e)
	{
	    System.out.println("Caught exception trying to read from bogus "
			       + "memory file");
	}
	
	mfos = new MemoryFileOutputStream("/test");
	try
	{
	    mfos.write("Hello World!".getBytes());
	}
	finally
	{
	    mfos.close();
	}
	mfos = null;

	terminateFS();
	
	try
	{
	    byte buffer[] = new byte[1024];
	    int actual;
	    
	    mfis = new MemoryFileInputStream("/test");
	    try
	    {
		actual = mfis.read(buffer);
	    }
	    finally
	    {
		mfis.close();
	    }
	    System.out.println(new String(buffer, 0, actual));
	}
	catch(FileNotFoundException e)
	{
	    System.out.println("Couldn't find file?");
	}

	mf = new MemoryFile("/test");
	mf.delete();
	mf = null;
	
	try
	{
	    new MemoryFileInputStream("/test");
	    System.out.println("Able to open non-existent file?");
	}
	catch(FileNotFoundException e)
	{
	    System.out.println("Caught exception trying to read from `/test' "
			       + "memory file");
	}
    }
}

/* Expected Output:
Caught exception trying to read from bogus memory file
Hello World!
Caught exception trying to read from `/test' memory file
*/
