
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.DeadTeamException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.MemorySpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.MemoryResource;
import edu.utah.janosvm.resources.ResourceSpecException;

public class MemoryLimit
{

    public static void main(String args[])
	throws Exception
    {
	/* Create a Team.create with a specific memory limit */
	Team lt = Team.current();
	ResourceSpec specs[] = {
	    new MemorySpec(8 * 1024 * 1024),
	    StdioSpec.currentFD()
	};
	TeamHandle child = Team.create("Child of "
				       + lt.getName()
				       + "("
				       + lt.getID()
				       + ")",
				       specs);

	/* Visit the team and start allocating memory */
	child.switchTo();
	{
	    Object arr[] = { null, null, null };
	    long memUsage = 0;
	    int lpc = 0;
	    
	    try
	    {
		for( lpc = 0; lpc < 3; lpc++ )
		{
		    /* Allocate some fat arrays to use up the memory */
		    arr[lpc] = new byte[3 * 1024 * 1024];
		    /* Check the usage stats */
		    if( memUsage > MemoryResource.getUsage() )
			System.out.println("We're using less memory?");
		    memUsage = MemoryResource.getUsage();
		    if( memUsage > MemoryResource.getMax() )
			System.out.println("We're past our limit?");
		}
		/*
		 * Hmmm... We were able to allocate 9+ meg of memory, thats
		 * bad.
		 */
		System.out.println("No exception thrown?");
	    }
	    catch(OutOfMemoryError oom)
	    {
		if( lpc == 2 )
		{
		    /* Ran out of memory at the right time */
		    System.out.println("Memory limit worked");
		}
		else
		{
		    /* Ran out at an odd time */
		    System.out.println("Memory limit failed");
		}
	    }
	}
	child.returnFrom();
	child.terminate();

	MemorySpec ms = null;

	try
	{
	    /* Try to create a Team.create that wants too much memory */
	    child = Team.create("Child 2 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				new ResourceSpec[] {
				    ms = new MemorySpec(1024 * 1024 * 1024)
				});
	    System.out.println("Memory resource maximum didn't work");
	    child.terminate();
	}
	catch(ResourceSpecException e)
	{
	    if( e.getUnsatisfiedSpec() == ms )
		System.out.println("Memory resource maximum worked");
	    else
		System.out.println("Memory resource maximum worked, "
				   + "but unsatisfied spec is wrong.");
	}
	
	try
	{
	    /* Try to create a Team.create that wants too little memory */
	    child = Team.create("Child 3 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				new ResourceSpec[] {
				    new MemorySpec(1024),
				});
	    System.out.println("Memory resource minimum didn't work");
	    child.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Memory resource minimum worked");
	}
    }
    
}

/* Expected Output:
Memory limit worked
Memory resource maximum worked
Memory resource minimum worked
*/
