
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamReservation;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;
import edu.utah.janosvm.resources.ResourceSpecException;

public class NameSpaceBoundary
{

    public static class TraceClass
    {
	public TraceClass()
	    throws Exception
	{
	    throw new Exception("Map me");
	}
    }
    
    private static Team curr = Team.current();
    private static TeamHandle child = null;
    private static TeamHandle team1 = null;
    private static TeamHandle team2 = null;

    private static void main(String args[])
    {
	try
	{
	    nsTest(args);
	}
	catch(Throwable th)
	{
	    /* Make sure everyone is dead */
	    if( child != null )
		child.terminate();
	    if( team1 != null )
		team1.terminate();
	    if( team2 != null )
		team2.terminate();
	    th.printStackTrace();
	}
    }
    
    public static void nsTest(String args[])
	throws Exception
    {
	/* Get the Class objects for a shared and unshared class */
	final Class nexCL;
	final Class exCL;

	nexCL = NameSpaceResource.loadTeamClass("NameSpaceBoundary");
	exCL = NameSpaceResource.loadTeamClass("java.lang.Object");
	child = Team.create("Child 1 of " + curr.getName() + curr.getID(),
			    new ResourceSpec[] {
				StdioSpec.currentFD(),
				NameSpaceSpec.current()
			    });

	child.switchTo();
	{
	    /* Get the same classes as the other teams for comparison */
	    Class nexChildCL;
	    Class exChildCL;

	    nexChildCL = NameSpaceResource.loadTeamClass("NameSpaceBoundary");
	    exChildCL = NameSpaceResource.loadTeamClass("java.lang.Object");
	    if( nexChildCL == nexCL )
	    {
		System.out.println("Non-exported classes are the same");
	    }
	    else
	    {
		System.out.println("Non-exported classes are different");
	    }
	    if( exChildCL == exCL )
	    {
		System.out.println("Exported classes are the same");
	    }
	    else
	    {
		System.out.println("Exported classes are different");
	    }
	}
	child.returnFrom();
	child.terminate();
	child = null;

	TeamHandle kernel = curr.getKernelTeam();

	/* Export a class group for our child teams */
	NameSpaceResource.exportClassGroup("nameSpaceTest");
	NameSpaceResource.exportClassGroup("nameSpaceTrace");

	/*
	 * Create specs for two teams that import our newly exported class
	 * group
	 */
	NameSpaceGroup nsg1[] = {
	    new NameSpaceGroup(kernel, "kernel", "identity"),
	    new NameSpaceGroup(kernel, "janosvm", "identity"),
	    new NameSpaceGroup(curr, "nameSpaceTest", "identity")
	};
	ResourceSpec spec1[] = {
	    new NameSpaceSpec(nsg1),
	    StdioSpec.currentFD()
	};
	TeamReservation t1a;
	team1 = Team.create("Child 2 of " + curr.getName() + curr.getID(), spec1);
	
	NameSpaceGroup nsg2[] = {
	    new NameSpaceGroup(kernel, "kernel", "identity"),
	    new NameSpaceGroup(kernel, "janosvm", "identity"),
	    new NameSpaceGroup(curr, "nameSpaceTest", "team2"),
	    new NameSpaceGroup(curr, "nameSpaceTrace", "team2")
	};
	ResourceSpec spec2[] = {
	    new NameSpaceSpec(nsg2),
	    StdioSpec.currentFD()
	};
	TeamReservation t2a;
	team2 = Team.create("Child 3 of " + curr.getName() + curr.getID(), spec2);
	Class team1CL;

	/*
	 * Visit the first team and get a pointer to the shared "Spinner"
	 * class.  Note that we're using Reservations here since we'll be
	 * holding raw pointers referring to objects in this child team.
	 */
	t1a = team1.pushReservation();
	t1a.visit();
	{
	    Class cl = NameSpaceResource.loadTeamClass("Spinner");
	    
	    Team.current().getData().add(cl);
	    team1CL = cl;
	}
	t1a.leave();

	/*
	 * Visit the second team and get a pointer to the "rennipS" class which
	 * should be the "Spinner" class with a mapped name.
	 */
	team2.switchTo();
	{
	    Class cl = NameSpaceResource.loadTeamClass("rennipS");
	    
	    if( cl == team1CL )
		System.out.println("Mapped classes work");
	    else
		System.out.println("Mapping is broken");
	    System.out.println("cl = " + cl.getName());
	    
	    cl = NameSpaceResource.loadTeamClass("Spinner");
	    if( cl == team1CL )
		System.out.println("Mapped classes failed");
	    
	    try
	    {
		new TraceClass();
	    }
	    catch(Exception e)
	    {
		e.printStackTrace();
	    }
	}
	team2.returnFrom();
	
	/* We're done with team1CL pointer so drop it and pop the Reservation */
	team1CL = null;
	team1.popReservation();

	/* Kill the teams */
	team1.terminate();
	team1 = null;
	team2.terminate();
	team2 = null;
    }
    
}

/* Expected Output:
Non-exported classes are different
Exported classes are the same
Mapped classes work
cl = rennipS
java.lang.Exception: Map me
	at java.lang.Throwable.fillInStackTrace(Throwable.java:native)
	at java.lang.Throwable.<init>(Throwable.java:40)
	at java.lang.Exception.<init>(Exception.java:24)
	at NameSpaceBoundary$MappedTraceClass.<init>(NameSpaceBoundary.java:21)
	at NameSpaceBoundary.nsTest(NameSpaceBoundary.java:164)
	at NameSpaceBoundary.main(NameSpaceBoundary.java:34)
*/
