
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.VMInstance;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.ReservationOverflowError;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;

public class ReservationTest
{
    public static void main(String args[])
	throws Exception
    {
	Team lt = Team.current();
	TeamHandle kernel = lt.getKernelTeam();
	TeamReservation tr;

	/* Push a kernel Reservation */
	try
	{
	    tr = kernel.pushReservation();
	    /*
	     * TeamReservation's are funny creatures since they're java objects
	     * that live on a specially allocated segment of memory.  So we
	     * GC just to make sure we don't blow up.
	     */
	    System.gc();
	    System.out.println("Still here");
	    try
	    {
		String kernelMessage;

		/* Switch to the kernel to do something */
		tr.visit();
		{
		    /* Can't print since it will go out on the kernels stdio */
		    kernelMessage = "Current TeamHandle = "
			+ Team.current().getName();
		}
		/* Switch back to our TeamHandle */
		tr.leave();
		System.out.println(kernelMessage);
	    }
	    catch(TeamEjectionException e)
	    {
	    }
	    System.out.println("Current TeamHandle = "
			       + Team.current().getName());

	    try
	    {
		tr.visit();
		throw new Exception();
	    }
	    catch(TeamEjectionException e)
	    {
	    }
	    catch(Exception e)
	    {
		System.out.println("Current TeamHandle = "
				   + Team.current().getName());
	    }

	    Object synchro = new Object();

	    try
	    {
		tr.visit();
		synchronized(synchro)
		{
		    throw new Exception();
		}
	    }
	    catch(TeamEjectionException e)
	    {
	    }
	    catch(Exception e)
	    {
		System.out.println("Current TeamHandle = "
				   + Team.current().getName());
	    }

	    try
	    {
		synchro.wait();
	    }
	    catch(IllegalMonitorStateException e)
	    {
		System.out.println("Dropped lock on synchro");
	    }
	    
	    try
	    {
		tr.visit();
		{
		    /* Make sure catastrophes still work wrt Reservations */
		    VMInstance.crash();
		}
		tr.leave();
	    }
	    catch(TeamEjectionException e)
	    {
		System.out.println("Reservation crash worked");
	    }
	    TeamHandle.popReservation();
	}
	catch(DeadTeamException dte)
	{
	}

	try
	{
	    tr = kernel.pushReservation();
	    tr.visit();
	    throw new Exception();
	}
	catch(TeamEjectionException e)
	{
	}
	catch(DeadTeamException dte)
	{
	}
	catch(Exception e)
	{
	    System.out.println("Current TeamHandle = "
			       + Team.current().getName());
	}
	
	/*
	 * Create and kill a TeamHandle just to make sure we can't push a
	 * Reservation for it.
	 */
	TeamHandle child = Team.create();

	child.terminate();

	try
	{
	    tr = child.pushReservation();
	    System.out.println("Able to push Reservation for dead TeamHandle");
	    tr.pop();
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Can't push Reservation for dead TeamHandle");
	}

	try
	{
	    child = Team.create();
	    
	    while( true )
		child.pushReservation();
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Eh? Team is dead, this isn't right...");
	}
	catch(ReservationOverflowError e)
	{
	    System.out.println("Overflow captured");
	}
	System.out.println("Still here");
	try
	{
	    tr = child.pushReservation();
	    tr.pop();
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Can't push new reservation after overflow?");
	}
	child.terminate();
    }
}

/* Expected Output:
Still here
Current TeamHandle = kernel
Current TeamHandle = ReservationTest
Current TeamHandle = ReservationTest
Current TeamHandle = ReservationTest
Dropped lock on synchro
Reservation crash worked
Current TeamHandle = ReservationTest
Can't push Reservation for dead TeamHandle
Overflow captured
Still here
*/
