
import edu.utah.janosvm.kit.comm.amiga.SignalSet;

public class SignalTest
    extends Thread
{
    /* Some interesting signal numbers */
    private static final int SIG_EXIT = 0;
    private static final int SIG_PRINT = 1;
    private static final int SIG_FAKE = 2;

    private static final int SIG_CONTINUE = 63;

    private SignalSet parent;
    private SignalSet ss;
    
    public SignalTest(SignalSet parent, SignalSet ss)
    {
	this.parent = parent;
	this.ss = ss;
    }

    public void run()
    {
	long recvMask, waitMask;
	boolean done = false;

	/* Allocate some signals we care about */
	this.ss.allocate(SIG_EXIT);
	this.ss.allocate(SIG_PRINT);
	this.ss.allocate(SIG_FAKE);
	/* Only wait for exit and print */
	waitMask =
	    (1L << SIG_EXIT) |
	    (1L << SIG_PRINT);
	this.parent.signal(SIG_CONTINUE);
	while( !done )
	{
	    /* Wait for our signals */
	    recvMask = this.ss.waitFor(waitMask);
	    if( (recvMask & (1L << SIG_PRINT)) != 0 )
	    {
		System.out.println("This is the server.");
		recvMask &= ~(1L << SIG_PRINT);
	    }
	    if( (recvMask & (1L << SIG_EXIT)) != 0 )
	    {
		System.out.println("Server exiting...");
		recvMask &= ~(1L << SIG_EXIT);
		done = true;
	    }
	    if( (recvMask & (1L << SIG_FAKE)) != 0 )
	    {
		System.out.println("Got the fake signal");
	    }
	    if( recvMask != 0 )
	    {
		System.out.println("Got a signal I wasn't waiting for...");
	    }
	}
    }

    public static void main(String args[])
    {
	try
	{
	    SignalSet me = new SignalSet();
	    SignalSet ss = new SignalSet();
	    SignalTest st = new SignalTest(me, ss);

	    me.allocate(SIG_CONTINUE);
	    st.start();
	    me.waitFor((1L << SIG_CONTINUE));
	    /* Make sure the basic thing works */
	    ss.signal(SIG_PRINT);
	    Thread.sleep(1);
	    /*
	     * Make sure it doesn't wake up if its not waiting for this
	     * signal at the time.
	     */
	    ss.signal(SIG_FAKE);
	    Thread.sleep(1);
	    try
	    {
		/*
		 * Make sure it doesn't wake up if the signal itself is bogus
		 */
		ss.signal(23);
	    }
	    catch(Error e)
	    {
	    }
	    Thread.sleep(1);
	    /* Make the thread exit */
	    ss.signal(SIG_EXIT);
	    st.join();
	}
	catch(InterruptedException e)
	{
	}
    }
}

/* Expected Output:
This is the server.
Server exiting...
*/
