
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.TeamHandle;

import edu.utah.janosvm.kit.comm.amiga.MsgPort;
import edu.utah.janosvm.kit.comm.amiga.Message;
import edu.utah.janosvm.kit.comm.amiga.SignalSet;

import edu.utah.janosvm.kit.management.team.TeamListMessage;
import edu.utah.janosvm.kit.management.team.TeamListMessenger;

public class TeamListTest
{
    public static void main(String args[])
	throws Exception
    {
	Team parent = Team.current();
	SignalSet ss = new SignalSet();
	MsgPort mp = new MsgPort(ss);
	TeamListMessenger messenger = new TeamListMessenger(mp);
	long recvMask, waitMask;
	TeamListMessage tlm;
	TeamHandle th;
	int id;

	th = Team.create("Team 1");
	id = th.getID();
	tlm = (TeamListMessage)mp.waitForMessage();
	if( (tlm.getEvent() == TeamEvent.CREATED) &&
	    (tlm.getName().equals("Team 1")) &&
	    (tlm.getID() == id) )
	{
	    System.out.println("Got team creation message");
	}
	else
	{
	    System.out.println("Got a bogus message? " + tlm);
	}

	th.switchTo();
	{
	    Team team = Team.current();
	    
	    team.notifyOnFinish();
	    team.addDependency(parent);
	    new Thread(new Runnable() {
		public void run()
		{
		    try { Thread.sleep(100); }
		    catch(InterruptedException e) { }
		    Thread.currentThread().setDaemon(true);
		    while( true );
		}
	    }).start();
	}
	th.returnFrom();
	tlm = (TeamListMessage)mp.waitForMessage();
	if( (tlm.getEvent() == TeamEvent.FINISHED) &&
	    (tlm.getID() == id) )
	{
	    System.out.println("Got team finished message");
	}
	else
	{
	    System.out.println("Got a bogus message? " + tlm);
	}
	
	th.terminate();
	
	tlm = (TeamListMessage)mp.waitForMessage();
	if( (tlm.getEvent() == TeamEvent.TERMINATED) &&
	    (tlm.getID() == id) )
	{
	    System.out.println("Got team termination message");
	}
	else
	{
	    System.out.println("Got a bogus message? " + tlm);
	}
	tlm = (TeamListMessage)mp.waitForMessage();
	if( (tlm.getEvent() == TeamEvent.BURIED) &&
	    (tlm.getID() == id) )
	{
	    System.out.println("Got team buried message");
	}
	else
	{
	    System.out.println("Got a bogus message? " + tlm);
	}
    }
}

/* Expected Output:
Got team creation message
Got team finished message
Got team termination message
Got team buried message
*/
