
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.NoSuchExportException;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.MemorySpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.MemoryResource;
import edu.utah.janosvm.resources.ResourceSpecException;
import edu.utah.janosvm.resources.DuplicateResourceSpecException;

public class TeamTest
{
    private static class BogusSpec
	extends ResourceSpec
    {
    }
    
    public static void main(String args[])
    {
	ResourceSpec smallSpecs[] = {
	    new MemorySpec(1 * 1024 * 1024),
	    StdioSpec.currentFD()
	};
	Team team = Team.current();
	TeamHandle th;
	int lpc;

	for( lpc = 0; lpc < 10; lpc++ )
	{
	    try
	    {
		th = Team.create("Child of TeamTest "
				 + team.getID()
				 + " "
				 + lpc,
				 smallSpecs);
		System.out.println("Created child " + lpc);
		th.terminate();
		th.terminate();
	    }
	    catch(ResourceSpecException e)
	    {
		System.out.println("Caught resource exception?");
		e.printStackTrace();
	    }
	    catch(DuplicateExportException e)
	    {
		System.out.println("Caught duplicate exception?");
		e.printStackTrace();
	    }
	    catch(DeadTeamException e)
	    {
		System.out.println("Caught dead team exception?");
		e.printStackTrace();
	    }
	}

	try
	{
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " description");
	    th.switchTo();
	    {
		Team.current().setDescription("TeamTest Child");
	    }
	    th.returnFrom();
	    if( !th.getDescription().equals("TeamTest Child") )
		System.out.println("Descriptions don't match?");
	    th.terminate();
	}
	catch(TeamEjectionException e)
	{
	    System.out.println("I was ejected?");
	    e.printStackTrace();
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Its dead?");
	    e.printStackTrace();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Caught resource exception?");
	    e.printStackTrace();
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Caught duplicate exception?");
	    e.printStackTrace();
	}
	
	th = null;
	try
	{
	    TeamHandle th2;
	    
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " abuser 1",
			     smallSpecs);
	    th2 = Team.create("Child of TeamTest "
			      + team.getID()
			      + " abuser 1",
			      smallSpecs);
	    System.out.println("Able to make teams with duplicate names?");
	    th2.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Hrmph...  Can't make team...");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Unable to make duplicate team");
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Caught dead team exception?");
	    e.printStackTrace();
	}
	finally
	{
	    if( th != null )
		th.terminate();
	}
	
	th = null;

	try
	{
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " abuser 1",
			     new ResourceSpec[] {
				 new MemorySpec(1024 * 1024 * 1024),
			     });
	}
	catch(ResourceSpecException e)
	{
	}
	catch(DuplicateExportException e)
	{
	}
	catch(DeadTeamException e)
	{
	}
	try
	{
	    th = Team.create("Child of TeamTest "
			      + team.getID()
			      + " abuser 1",
			      smallSpecs);
	    System.out.println("Old name cleared appropriately");
	    th.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Hrmph...  Can't make team...");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Unable to make team with same name as bogus "
			       + "team?");
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Caught dead team exception?");
	    e.printStackTrace();
	}
	
	try
	{
	    ResourceSpec duplicateSpecs[] = {
		new MemorySpec(1 * 1024 * 1024),
		new MemorySpec(1 * 1024 * 1024)
	    };
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " abuser 2",
			     duplicateSpecs);
	    System.out.println("Able to make team with duplicate specs?");
	    th.terminate();
	}
	catch(DuplicateResourceSpecException e)
	{
	    System.out.println("Can't make team with duplicate specs");
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Can't make team with duplicate specs, "
			       + "but got wrong exception type.");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Able to make team with duplicate specs?");
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Caught dead team exception?");
	    e.printStackTrace();
	}
	
	try
	{
	    ResourceSpec nullSpecs[] = {
		new MemorySpec(1 * 1024 * 1024),
		null
	    };
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " abuser 3",
			     nullSpecs);
	    System.out.println("Able to make team with null spec");
	    th.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Can't make team with null spec?");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Can't make team with null spec?");
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Caught dead team exception?");
	    e.printStackTrace();
	}
	
	try
	{
	    ResourceSpec nullSpecs[] = {
		new MemorySpec(1 * 1024 * 1024),
		new BogusSpec(),
	    };
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " abuser 4",
			     nullSpecs);
	    System.out.println("Able to make team with unimplemented spec");
	    th.terminate();
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Can't make team with bogus spec?");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Can't make team with bogus spec?");
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Caught dead team exception?");
	    e.printStackTrace();
	}

	if( team.addDependency(team) )
	{
	    System.out.println("A team can have itself as a dependency?");
	}
	else
	{
	    System.out.println("Can't add self as a dependency");
	}
	try
	{
	    int thID;
	    
	    th = Team.create("Child of TeamTest "
			     + team.getID()
			     + " Parent",
			     smallSpecs);
	    thID = th.getID();
	    th.switchTo();
	    {
		TeamHandle child = Team.create("Child of TeamTest.Parent "
					       + thID
					       + " Child",
					       smallSpecs);
		Team parent = Team.current();
		
		child.switchTo();
		{
		    Team me = Team.current();

		    me.addDependency(th);
		}
		child.returnFrom();
		if( parent.addDependency(child) )
		{
		    System.out.println("Able to create a dependency cycle?");
		}
		else
		{
		    System.out.println("Unable to create dependency cycle");
		}
	    }
	    th.returnFrom();
	    th.terminate();
	    try
	    {
		Team.find("Child of TeamTest.Parent "
			  + thID
			  + " Child");
		System.out.println("Dependent still alive?");
	    }
	    catch(NoSuchExportException e)
	    {
		System.out.println("Dependent was killed");
	    }
	}
	catch(ResourceSpecException e)
	{
	    System.out.println("Can't make team with bogus spec?");
	}
	catch(DuplicateExportException e)
	{
	    System.out.println("Can't make team with bogus spec?");
	}
	catch(DeadTeamException e)
	{
	    System.out.println("Can't make team with bogus spec?");
	}
	catch(TeamEjectionException e)
	{
	    System.out.println("Can't make team with bogus spec?");
	}
    }
}

/* Expected Output:
Created child 0
Created child 1
Created child 2
Created child 3
Created child 4
Created child 5
Created child 6
Created child 7
Created child 8
Created child 9
Unable to make duplicate team
Old name cleared appropriately
Can't make team with duplicate specs
Able to make team with null spec
Able to make team with unimplemented spec
Can't add self as a dependency
Unable to create dependency cycle
Dependent was killed
*/
