
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.CrossTeamCloneable;
import edu.utah.janosvm.sys.SegmentationFaultError;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;

import java.util.Hashtable;

public class WriteBarrier
{
    public static class CrossTeamObject
	implements CrossTeamCloneable, Cloneable
    {
	private Object obj;
	private int i;

	public CrossTeamObject(Object obj, int i)
	{
	    this.obj = obj;
	    this.i = i;
	}

	public Object getObject()
	{
	    return this.obj;
	}

	public int getI()
	{
	    return this.i;
	}

	public Object clone()
	{
	    try
	    {
		return super.clone();
	    }
	    catch(CloneNotSupportedException e)
	    {
		return null;
	    }
	}

	public String toString()
	{
	    return "CrossTeamObject[obj="
		+ this.obj
		+ "; i="
		+ this.i
		+ "]";
	}
    }
    
    public static void main(String args[])
	throws Exception
    {
	TeamHandle kernel = Team.current().getKernelTeam();
	Team parent = Team.current();
	CrossTeamObject cto;
	NameSpaceSpec nss;
	Class cl;

	/* Export a class group for our child teams */
	NameSpaceResource.exportClassGroup("WriteBarrier");

	nss = new NameSpaceSpec(new NameSpaceGroup[] {
		new NameSpaceGroup(kernel, "kernel", "identity"),
		new NameSpaceGroup(kernel, "janosvm", "identity"),
		new NameSpaceGroup(parent, "WriteBarrier", "identity")
	});
	
	TeamHandle child1 = Team.create("Child 1 of "
					+ parent.getName()
					+ parent.getID(),
					new ResourceSpec[] {
					    StdioSpec.currentFD(),
					    nss,
					});

	TeamReservation cr1 = child1.pushReservation();

	TeamHandle child2 = Team.create("Child 2 of "
					+ parent.getName()
					+ parent.getID(),
					new ResourceSpec[] {
					    StdioSpec.currentFD(),
					    nss,
					});

	TeamReservation cr2 = child2.pushReservation();

	cr1.visit();
	{
	    cl = NameSpaceResource.loadTeamClass("WriteBarrier");
	    cto = new CrossTeamObject(new Integer(1), 1);
	}
	cr1.leave();

	try
	{
	    cr2.visit();
	    {
		Hashtable ht = new Hashtable();

		/* Try to make a cross team pointer */
		ht.put("This shouldn't work", cto);
	    }
	    cr2.leave();
	}
	catch(SegmentationFaultError e)
	{
	    System.out.println("Caught object <- object seg fault");
	}
	
	try
	{
	    cr2.visit();
	    {
		cl.newInstance();
	    }
	    cr2.leave();
	}
	catch(SegmentationFaultError e)
	{
	    System.out.println("Caught object <- class seg fault");
	}
	
	try
	{
	    cr2.visit();
	    {
		CrossTeamObject ccto;

		ccto = (CrossTeamObject)cto.clone();
		if( ccto.getObject() == null )
		    System.out.println("Cross team object worked");
	    }
	    cr2.leave();
	}
	catch(SegmentationFaultError e)
	{
	    e.printStackTrace();
	}

	cr2.pop();
	cr1.pop();
	
	child2.terminate();
	child1.terminate();
    }
    
}

/* Expected Output:
Caught object <- object seg fault
Caught object <- class seg fault
Cross team object worked
*/
