/*
 * DieAfter.java
 *
 * Die after running the provided class's main() method
 * exits.  This is a hack for the JanosVM which does not
 * implicitly exit after all threads have terminated.
 */

import java.lang.reflect.Method;
import java.lang.IllegalAccessException;
import java.lang.IllegalArgumentException;
import java.lang.reflect.InvocationTargetException;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.io.PrintStream;

public class DieAfter
{
	private static class ExceptionStreamCleaner
		extends PrintWriter
	{
		char[][] bufs = new char[20][];
		int idx = 0;

		ExceptionStreamCleaner(PrintStream os)
		{
			super(os);
		}

		public void println(Object str)
		{
			super.println("HIT obj");
		}

		public void println(String str)
		{
			super.println("HIT [c");
		}

		public void println(char[] str)
		{
			super.println("HIT");
			bufs[idx++] = str;
		}

		public void tweak()
		{
			super.println("gak");
			for (int i = 0; i < (idx-2); i++)
				super.println(bufs[i]);
		}
	}

	public static void main(String[] args)
		throws Throwable
	{
		if ((args == null)
		    || (args.length < 1))
		{
			System.err.println("Usage: DieAfter [-verbose] <class> [<class> args]");
			System.exit(0);
		}

		boolean verbose = false;
		String newArgs[] = new String[args.length - 1];
		int cnameIdx = 0;
		
		if (args[0].equals("-verbose"))
		{
			cnameIdx = 1;
			verbose = true;
		}

		String cname = args[cnameIdx];
		System.arraycopy(args, cnameIdx + 1,
				 newArgs, 0,
				 args.length - 1 - cnameIdx);
		
		Class c = null;
		Class[] paramTypes = null;
		Method m = null;
		Object[] params = null;
		Object rv = null;

		if (verbose)
			System.out.println("Looking up " +cname+ ".");
		try
		{
			c = Class.forName(cname);

			paramTypes = new Class[1];
			paramTypes[0] = args.getClass();
			m = c.getMethod("main", paramTypes);
		}
		catch (Exception e)
		{
			boom("reflecting main out of " +cname, e);
		}

		if (verbose)
			System.out.println("Found " +cname+ ": " +c);

		if (verbose)
			System.out.println("Invoking 'main' on " +cname);

		try
		{
			params = new Object[1];
			params[0] = newArgs;
			m.invoke(null, params);

			if (verbose)
				System.out.println("main returned, cleanly.");
		}
		catch (java.lang.reflect.InvocationTargetException ite)
		{
			Throwable th = ite.getTargetException();
			if (verbose)
				System.out.println("Got ITE containing: " +th);
						
			th.printStackTrace(System.out);

			System.exit(0);
		}
		catch (Throwable th)
		{
			if (verbose)
				System.out.println("Begin exception hackery on: " +th);

			/*
			 * Note on exceptions: We don't want to
			 * appear different from the standard run-time
			 * if the method throws something like
			 * ArrayIndexOutOfBoundsException (or even a
			 * reflection error.  So, just print the
			 * exception with a minimum of discrepancy
			 * from the Kaffe runtime.  (We have to trim
			 * off the Method.invoke() and DieAfter.main()
			 * lines from the stack trace, though...
			 */
			// ExceptionStreamCleaner esc;
			// aesc = new ExceptionStreamCleaner(System.out);
			//th.printStackTrace(esc);
			th.printStackTrace(System.out);
			//esc.tweak();
			if (verbose)
				System.out.println("End exception hackery");
			System.exit(1);
		}

		if (verbose)
			System.out.println("DieAfter exiting, cleanly.");

		System.exit(0);
	}

	static void boom(String msg, Exception e)
	{
		System.err.println("Exception while " +msg+ ".");
		e.printStackTrace(System.err);
		System.exit(1);
	}
}

/* Expected Output:
Usage: DieAfter [-verbose] <class> [<class> args]
*/
